      program rec_2d

c-----------------------------------------------
c Two-dimensional flow in a rectangular domain.
c
c This program performs the following functions:
c
c (a) It generates a uniform Cartesian grid
c     in a rectangular domain in the xy plane
c
c (b) It prints velocity vectors at the grid points
c
c (c) It interpolates for the velocity vector at arbitrary
c     points.
c
c (d) It computes the rate of strain tensor,
c     the vorticity, the eigenvalues and eigenvectors
c     of the rate of deformation tensor
c
c
c  SYMBOLS:
c  --------
c
c  ax, bx:  domain is confined in ax<x<bx
c  ay, by:  domain is confined in ay<y<by
c
c  Nx :  number of x intervals
c  Ny :  number of y intervals
c
c  Dx :  length of x interval
c  Dy :  length of y interval
c
c  glx(i) :  x-position of vertical ith grid line
c  gly(i) :  y-position of horizontal ith grid line
c
c  [ gx(i,j),  gy(i,j)] : coordinates of (i,j) grid point
c  [gux(i,j), guy(i,j)] : velocity at (i,j) grid point
c
c  Axx, Axy:  velocity gradient tensor
c  Ayx, Ayy:  velocity gradient tensor
c
c  Exx, Exy:  rate of strain tensor
c  Eyx, Eyy:  rate of strain tensor
c
c  roe: rate of expansion
c       at the grid points
c
c  omega: vorticity
c         at the grid points
c
c  strain1: eigenvalue of the rate of strain tensor
c           at the grid points
c
c  strain2: eigenvalue of the rate of strain tensor
c
c  egv1_x, egv1_y:  first eigenvector of the rate of strain tensor
c                   at the grid points
c  egv2_x, egv2_y:  second eigenvector of the rate of strain tensor
c                   at the grid points
c
c-----------------------------------------------

      Dimension glx(129), gly(129)

      Dimension  gx(129,129), gy(129,129)
      Dimension gux(129,129),guy(129,129)

      Dimension Axx(129,129),Axy(129,129)
      Dimension Ayx(129,129),Ayy(129,129)

      Dimension Exx(129,129),Exy(129,129)
      Dimension Eyx(129,129),Eyy(129,129)

      Dimension roe  (129,129)
      Dimension omega(129,129)

      Dimension strain1(129,129),strain2(129,129)

      Dimension egv1_x(129,129),egv1_y(129,129)
      Dimension egv2_x(129,129),egv2_y(129,129)

      Dimension vector (5,2)     ! auxiliary 5-point arrowed vector

c--------------
c common blocks
c--------------

      common/grid_2d_1/glx,gly
      common/grid_2d_2/gx,gy
      common/grid_2d_3/gux,guy

c----------
c constants
c----------

      Null   = 0
      None   = 1
      Ntwo   = 2
      Nthree = 3
      Nfour  = 4
      Nfive  = 5

      zero = 0.0

      pi  = 3.14159265358
      pi2 = 2.0*pi

c---------------------
c Read parameters
c from file: rec_2d.dat
c----------------------

      open (3,file="rec_2d.dat")

       read (3,*) ax,bx                  ! x limits
       read (3,*) ay,by                  ! y limits
       read (3,*) Nx,Ny                  ! divisions

      close (3)

c--------
c prepare
c--------

      Nx1 = Nx+1
      Ny1 = Ny+1

      Dx = (bx-ax)/Nx
      Dy = (by-ay)/Ny

c----
c choose the flow
c---

 98   Continue      ! return to repeat

      write (6,*)
      write (6,*)   "      MENU OF FLOWS"
      write (6,*)
      write (6,*)   " Enter:"
      write (6,*)
      write (6,*)   " 0 to quit"
      write (6,*)   " 1 for extensional flow"
      write (6,*)   " 2 for rotational flow"
      write (6,*)   " 3 for simple shear flow"
      write (6,*)   " 4 for a stagnation-point flow"
      write (6,*)   " 5 for a wake-type of flow (Kavasznay)"
      write (6,*)   "--------------------------------------"
      read  (5,*) Iflow

      If(Iflow.eq.0) Go to 99

c-----
c Menu
c-----

 97   Continue      ! return to repeat

      write (6,*)
      write (6,*)   "      MENU OF SERVICES"
      write (6,*)
      write (6,*)   " Enter 0 to quit"
      write (6,*)   "       1 to draw velocity vectors in 2D"
      write (6,*)   "         at the grid points"
      write (6,*)   "       2 to interpolate and draw velocity vectors"
      write (6,*)   "         in 2D at points within the cells"
      write (6,*)   "       3 to compute and plot the components of the"
      write (6,*)   "         (a) velocity gradient tensor"
      write (6,*)   "         (b) vorticity"
      write (6,*)   "         (c) eigenvalues of the rate of strain"
      write (6,*)   "         (d) eigenvectors of the rate of strain"
      write (6,*)   "------------------------------------------------"
      read  (5,*) menu

      If(menu.eq.0) Go to 98

c-----------------------------------
c compute grid lines and grid points
c-----------------------------------

      call grid_2d
     +
     +   (ax,bx
     +   ,ay,by
     +   ,Nx,Ny
     +   )

c----------------------------------------
c specify the velocity at the grid points
c----------------------------------------

      Do i=1,Nx1
       Do j=1,Ny1

         px = gx(i,j)
         py = gy(i,j)

        If(Iflow.eq.1) then             ! elongational flow
          gux(i,j) = px
          guy(i,j) =-py
        Else If(Iflow.eq.2) then        ! rotational flow
          gux(i,j) =-py
          guy(i,j) = px
        Else If(Iflow.eq.3) then        ! simple shear flow
          gux(i,j) =  py
          guy(i,j) =  0.0
        Else If(Iflow.eq.4) then
          gux(i,j) = py*(3.0*py-2.0*px) 
          guy(i,j) = py**2
        Else If(Iflow.eq.5) then
          wn  = pi2/0.5
          pxs = px+0.5
          gux(i,j) = 1.0-cos(wn*py)*exp(-1.0*wn*pxs)
          guy(i,j) =    -sin(wn*py)*exp(-1.0*wn*pxs)
        Else
         write (6,*) "Sorry, this choice is not implemented"
         stop
        End If

       End Do
      End Do

c--------------------------------------
c Print: grid lines
c        velocity vectors at grid points
c
c in file: PLOTDAT
c--------------------------------------

      If(menu.eq.1) then

      open (2,file="PLOTDAT")

c-----------------
c print grid lines
c-----------------

      Do i=1,Nx1
       write (2,100) Ntwo
       write (2,100) None,glx(i),ay
       write (2,100) Ntwo,glx(i),by
      End Do

      Do j=1,Ny1
       write (2,100) Ntwo
       write (2,100) None,ax,gly(j)
       write (2,100) Ntwo,bx,gly(j)
      End Do

c---------------------------------
c Prepare to draw velocity vectors
c
c Define vector plotting scale.
c The maximum length of the velocity vector
c will be 0.75 times the maximum of the
c grid spacing: dx or dy
c---------------------------------

      scale = 0.0D0

      Do i=1,Nx1
       Do j=1,Ny1
        vmax = sqrt(gux(i,j)**2+guy(i,j)**2)
        If(vmax.gt.scale) scale = vmax
       End Do
      End Do

      Dmin = Dx
      If(Dy.lt.Dmin) Dmin = Dy

      scale = 0.75*Dmin/scale

c------------------------------
c print 5-point velocity arrows
c------------------------------

      Do i=1,Nx1
       Do j=1,Ny1

        xl = gux(i,j)*scale
        yl = guy(i,j)*scale

        call draw_arrow_2d
     +
     +   (gx(i,j),gy(i,j)
     +   ,xl,yl
     +   ,vector
     +   )

       write (2,100) Nfive
       Do k=1,5
        write (2,100) k,vector(k,1),vector(k,2)
       End Do

       End Do
      End Do

      write (2,*) Null
      close (2)                  

c------------------------------------------
c Print:  grid lines
c         velocity vectors at cell mid-points
c
c in file: PLOTDAT
c------------------------------------------

      Else If(menu.eq.2) then

      open (2,file="PLOTDAT")

c-----
c Prepare to draw velocity vectors
c
c Define vector plotting scale.
c The maximum length of the velocity vector
c will be 0.75 times the maximum of the
c grid spacing: dx or dy
c------

      scale = 0.0

      Do i=1,Nx1
       Do j=1,Ny1
        vmax = sqrt(gux(i,j)**2+guy(i,j)**2)
        If(vmax.gt.scale) scale = vmax
       End Do
      End Do

      dmax = dx
      If(dy.gt.dmax) dmax = dy

      scale = 0.75*dmax/scale

c---
c print grid lines
c in file:  PLOTDAT
c---

      Do i=1,Nx1
       write (2,100) Ntwo
       write (2,100) None,glx(i),ay
       write (2,100) Ntwo,glx(i),by
      End Do

      Do j=1,Ny1
       write (2,100) Ntwo
       write (2,100) None,ax,gly(j)
       write (2,100) Ntwo,bx,gly(j)
      End Do

c---
c Interpolate within each cell
c---

      xf = 0.223    ! x location factor within a cell; between 0  and 1
      yf = 0.567    ! y location factor within a cell; between 0  and 1

      xfc = 1.0-xf  ! complementary
      yfc = 1.0-yf  ! complementary

      Do i=1,Nx

       vpx = xf*glx(i) + xfc*glx(i+1)   ! velocity point

       Do j=1,Ny

        vpy = yf*gly(j) + yfc*gly(j+1)  ! velocity point
       
c----
c interpolate the velocity vector
c at the point" vpx, vpy
c
c The results is: vux, vuy
c----

         call rec_2d_int
     +
     +   (ax,bx
     +   ,ay,by
     +   ,Nx,Ny
     +   ,vpx,vpy
     +   ,vux,vuy
     +   )

c---
c print 5-point velocity arrows
c in file PLOTDAT
c---

         xl = vux*scale
         yl = vuy*scale

         call draw_arrow_2d
     +
     +     (vpx,vpy
     +     ,xl,yl
     +     ,vector
     +     )

        write (2,100) Nfive
        Do k=1,5
         write (2,100) k,vector(k,1),vector(k,2)
        End Do

       End Do
      End Do

      write (2,*) Null
      close (2)

c------------------------------------------
c Compute the velocity gradient tensor
c         by numerical differentiation;
c         the vorticity 
c         the eigenvalues of the rate of strain
c         the eigenvectors of the rate of strain
c
c in file: PLOTDAT
c------------------------------------------

      Else If(menu.eq.3) then

      open (2,file="PLOTDAT")

c---
c First print grid lines
c in file:  PLOTDAT
c
c Also print a column of zeros representing
c the z coordinate to be used for xyz plots
c---

      Do i=1,Nx1
       write (2,100) Ntwo
       write (2,100) i,glx(i),ay,zero
       write (2,100) i,glx(i),by,zero
      End Do

      Do j=1,Ny1
       write (2,100) Ntwo
       write (2,100) j,ax,gly(j),zero
       write (2,100) j,bx,gly(j),zero
      End Do

c---
c compute the velocity gradient tensor
c         at the grid points
c         by numerical differentiation
c---

         call rec_2d_vgt
     +
     +     (ax,bx
     +     ,ay,by
     +     ,Nx,Ny
     +     ,Axx,Axy
     +     ,Ayx,Ayy
     +     )

c---
c compute the rate of expansion
c         the rate of strain tensor 
c         the strains
c         the vorticity
c---

      scale = 0.80*Dx    ! scale for the eigenvectors

      Do i=1,Nx
       Do j=1,Ny

        roe(i,j) = Axx(i,j)+Ayy(i,j)          ! rate of expansion
      omega(i,j) = Axy(i,j)-Ayx(i,j)          ! vorticity

        Exx(i,j) = Axx(i,j)-0.5*roe(i,j)      ! rate of deformation tensor
        Exy(i,j) = 0.5*(Axy(i,j)+Ayx(i,j))    ! rate of deformation tensor
        Eyx(i,j) = Exy(i,j)                   ! rate of deformation tensor
        Eyy(i,j) = Ayy(i,j)-0.5*roe(i,j)      ! rate of deformation tensor

        det = 4.0*(Exx(i,j)**2+Exy(i,j)**2)      ! eigenvalues of rate of strain
        srd = sqrt(det)
        strain1(i,j) =  0.5*srd
        strain2(i,j) = -0.5*srd

c---
c compute the eigenvectors of the rate of strain
c---

        If(abs(Exy(i,j)).lt.0.0001) then         ! E is diagonal

          If(abs(Exx(i,j)-strain1(i,j)).gt.0.0001) then
            egv1_x(i,j) = 0.0
            egv1_y(i,j) = 1.0
          Else
            egv1_x(i,j) = 1.0
            egv1_y(i,j) = 0.0
          End If
          If(abs(Exx(i,j)-strain2(i,j)).gt.0.0001) then
            egv2_x(i,j) = 0.0
            egv2_y(i,j) = 1.0
          Else
            egv2_x(i,j) = 1.0
            egv2_y(i,j) = 0.0
          End If

         Else                                 ! E is not diagonal

          egv1_x(i,j) = 1.0
          egv1_y(i,j) = - (Exx(i,j)-strain1(i,j))/Exy(i,j)
          egv2_x(i,j) = 1.0
          egv2_y(i,j) = - (Exx(i,j)-strain2(i,j))/Exy(i,j)

         End If

c---
c normalize eigenvectors
c---

        fc1 = scale/sqrt(egv1_x(i,j)**2+egv1_y(i,j)**2)
        egv1_x(i,j) = fc1*egv1_x(i,j)
        egv1_y(i,j) = fc1*egv1_y(i,j)
        fc2 = scale/sqrt(egv2_x(i,j)**2+egv2_y(i,j)**2)
        egv2_x(i,j) = fc2*egv2_x(i,j)
        egv2_y(i,j) = fc2*egv2_y(i,j)

       End Do
      End Do

c---
c proceed to print
c---

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 to print data for a 3D-plot of the"
      write (6,*) "   rate of expansion"
      write (6,*) " 2 to print data for a 3D-plot of the"
      write (6,*) "   rate of the vorticity"
      write (6,*) " 3 to print data for a 3D-plot of the"
      write (6,*) "   first eigenvalue of the rate of the strain"
      write (6,*) " 4 to print data for a 3D-plot of the"
      write (6,*) "   second eigenvalue of the rate of the strain"
      write (6,*) " 5 to print data for a 2D-plot of the"
      write (6,*) "   first eigenvector of the rate of the strain"
      write (6,*) " 6 to print data for a 2D-plot of the"
      write (6,*) "   second eigenvector of the rate of the strain"
      write (6,*) "-----------------------------------------------"
      read  (5,*) Ichoose

      If(Ichoose.eq.0) Go to 99

c------------------------------------
c print data for a 3D plot or the roe
c------------------------------------

      If(Ichoose.eq.1) then

      Do i=1,Nx
       Do j=1,Ny
         write (2,100) Ntwo
         write (2,100) j,gx(i,j),gy(i,j),zero
         write (2,100) j,gx(i,j),gy(i,j),roe(i,j)
       End Do
      End Do

      write (2,*) Null
      close (2)

      End If

c------------------------------------------
c print data for a 3D plot of the vorticity
c------------------------------------------

      If(Ichoose.eq.2) then

      Do i=1,Nx
       Do j=1,Ny
         write (2,100) Ntwo
         write (2,100) j,gx(i,j),gy(i,j),zero
         write (2,100) j,gx(i,j),gy(i,j),omega(i,j)
       End Do
      End Do

      write (2,*) Null
      close (2)

      End If

c-------------------------------------------------
c print data for a 3D plot of the first eigenvalue
c of the rate of strain
c-------------------------------------------------

      If(Ichoose.eq.3) then

      Do i=1,Nx
       Do j=1,Ny
         write (2,100) Ntwo
         write (2,100) j,gx(i,j),gy(i,j),zero
         write (2,100) j,gx(i,j),gy(i,j),strain1(i,j)
       End Do
      End Do

      write (2,*) Null
      close (2)

      End If

c-------------------------------------------------
c print data for a 3D plot of the second eigenvalue
c of the rate of strain
c-------------------------------------------------

      If(Ichoose.eq.4) then

      Do i=1,Nx
       Do j=1,Ny
         write (2,100) Ntwo
         write (2,100) j,gx(i,j),gy(i,j),zero
         write (2,100) j,gx(i,j),gy(i,j),strain2(i,j)
       End Do
      End Do

      write (2,*) Null
      close (2)

      End If

c--------
c print data for a 2D plot of the first eigenvector
c of the rate of strain
c--------

      If(Ichoose.eq.5) then

      Do i=1,Nx
       Do j=1,Ny

         call draw_arrow_2d
     +
     +     (gx(i,j),gy(i,j)
     +     ,egv1_x(i,j),egv1_y(i,j)
     +     ,vector
     +     )

        write (2,100) Nfive
        Do k=1,5
         write (2,100) k,vector(k,1),vector(k,2)
        End Do

       End Do
      End Do

      write (2,*) Null
      close (2)

      End If

c--------
c print data for a 2D plot of the second eigenvector
c of the rate of strain
c--------

      If(Ichoose.eq.6) then

      Do i=1,Nx
       Do j=1,Ny
         call draw_arrow_2d
     +
     +    (gx(i,j),gy(i,j)
     +    ,egv2_x(i,j),egv2_y(i,j)
     +    ,vector
     +    )

        write (2,100) Nfive
        Do k=1,5
         write (2,100) k,vector(k,1),vector(k,2)
        End Do

       End Do
      End Do

      write (2,*) Null
      close (2)

      End If

c---
c end of local options
c---

      End If

c-------------------
c Return to the menu
c-------------------

      write (6,*)
      write (6,*)  " Done"
      write (6,*)  " Return to the main menu"
      write (6,*)

      Go to 97

c-------------
c Finishing up
c-------------

  99  Continue

c-----
c Done
c-----

  100 Format (1x,i3,5(1x,f12.8))
  101 Format (5(1x,f12.8))

      Stop
      End
