%---
close all
clear all
%---

%=============================================
% velocity interpolation in a rectangular cavity
% computation of the vorticity, rate-of-strain, etc
%
% confined in ax<x<bx
%             ay<y<by
%=============================================

ax = -1.3; bx = 1.3;
ay = -1.0; by = 1.0;

Nx = 16; Ny = 8;
Nx = 16; Ny = 18;

%---
% prepare to plot
%---

figure(1)
hold on
axis([ax-0.1, bx+0.1, ay-0.1, by+0.1])
axis equal
xlabel('x','fontsize',14)
ylabel('y','fontsize',14)
set(gca,'fontsize',14)
box on
title('Velocity vector field')

plot([ax, bx, bx, ax, ax],[ay, ay, by, by, ay],'-k')

%-----
% grid generation
%-----

[glx,gly,gx,gy] = grid_2d (ax,bx,ay,by,Nx,Ny);

%---
% specify the grid velocity
%---

Ichoose = 3;
Ichoose = 1;
Ichoose = 2;
Ichoose = 4;

 for i=1:Nx+1
   for j=1:Ny+1
     if(Ichoose==1)        % extensional flow
      gux(i,j) = gx(i,j);
      guy(i,j) =-gy(i,j);
     elseif(Ichoose==2)    % simple shear flow
      gux(i,j) = gy(i,j);
      guy(i,j) = 0.0;
     elseif(Ichoose==3)    % parabolic flow
      gux(i,j) =  (gy(i,j)-ay)*(by-gy(i,j));
      guy(i,j) = 0.0;
     elseif(Ichoose==4)    % periodic flow
      px = gx(i,j);
      py = gy(i,j);
      wnx  = 2*pi/(bx-ax);
      wny  = 2*pi/(by-ay);
      gux(i,j) = cos(wnx*px)*cos(wny*py);
      guy(i,j) = sin(wnx*px)*sin(wny*py);
     end
   end
 end

%---------------------------
% compute and plot the velocity vector field
% in the middle of each cell
%---------------------------

%---
% plot the grid
%---

for i=1:Nx+1
 plot([glx(i),glx(i)],[ay,by],'c')
end

for j=1:Ny+1
 plot([ax,bx],[gly(j),gly(j)],'c')
end

%---
% points
%---

 for i=1:Nx
   x = 0.5*(glx(i)+glx(i+1));

   for j=1:Ny
   y = 0.5*(gly(j)+gly(j+1));

   plot(x,y,'k.')

   [ux,uy] = rec_2d_int ...
    ...
      (ax,bx ... % x end points
      ,ay,by ... % y end points
      ,Nx,Ny ... % grid size
      ,glx,gly ... % grid lines
      ,gux,guy ... % grid velocity
      ,x,y ...      % interpolation point
      );

   vector = draw_arrow_2d ...
     ...
     (x,y,1.5*ux/Nx,1.5*uy/Nx);

   plot(vector(:,1),vector(:,2),'k');

  end
 end

%---
% plot
%----

%=========================
% velocity gradient tensor
%=========================

 [Lxx,Lxy,Lyx,Lyy] = rec_2d_vgt ...
...
  (glx,gly,Nx,Ny,gux,guy);

%---
% compute the rate of expansion
%         the rate of strain tensor 
%         the strains
%         the vorticity
%---

 for i=1:Nx+1
  for j=1:Ny+1

     roe(i,j) = Lxx(i,j)+Lyy(i,j);    % rate of expansion
   omega(i,j) = Lxy(i,j)-Lyx(i,j);   % vorticity

   Exx(i,j) = Lxx(i,j)-0.5*roe(i,j);     % rate of deformation tensor
   Exy(i,j) = 0.5*(Lxy(i,j)+Lyx(i,j));   % rate of deformation tensor
   Eyx(i,j) = Exy(i,j);                  % rate of deformation tensor
   Eyy(i,j) = Lyy(i,j)-0.5*roe(i,j);     % rate of deformation tensor

   det = 4.0*(Exx(i,j)^2+Exy(i,j)^2);   % eigenvalues of rate of strain
   srd = sqrt(det);
   strain1(i,j) =  0.5*srd;
   strain2(i,j) = -0.5*srd;

%---
% compute the eigenvectors of the rate of strain tensor
%---

if(abs(Exy(i,j))<0.0001) % E is diagonal

  if(abs(Exx(i,j)-strain1(i,j))>0.0001)
     egv1_x(i,j) = 0.0;
     egv1_y(i,j) = 1.0;
  else
     egv1_x(i,j) = 1.0;
     egv1_y(i,j) = 0.0;
  end
  if(abs(Exx(i,j)-strain2(i,j))>0.0001) 
     egv2_x(i,j) = 0.0;
     egv2_y(i,j) = 1.0;
  else
     egv2_x(i,j) = 1.0;
     egv2_y(i,j) = 0.0;
  end

 else  % E is not diagonal

  egv1_x(i,j) = 1.0;
  egv1_y(i,j) = -(Exx(i,j)-strain1(i,j))/Exy(i,j);
  egv2_x(i,j) = 1.0;
  egv2_y(i,j) = -(Exx(i,j)-strain2(i,j))/Exy(i,j);

 end

%---
% normalize the eigenvectors
%---

  fc1 = 1/sqrt(egv1_x(i,j)^2+egv1_y(i,j)^2);
  egv1_x(i,j) = fc1*egv1_x(i,j);
  egv1_y(i,j) = fc1*egv1_y(i,j);
  fc2 = 1/sqrt(egv2_x(i,j)^2+egv2_y(i,j)^2);
  egv2_x(i,j) = fc2*egv2_x(i,j);
  egv2_y(i,j) = fc2*egv2_y(i,j);

 end
end

%=================
% massive plotting
%=================

%---
% vorticity
%---

figure(2)
mesh(glx,gly,omega')
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('\omega','fontsize',15)
set(gca,'fontsize',15)
title('vorticity')
box on

%---
% rate of expansion
%---

figure(3)
mesh(glx,gly,roe')
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('\alpha','fontsize',15)
set(gca,'fontsize',15)
title('rate of expansion')
box

%---
% first rate-of-strain
%---

figure(4)
mesh(glx,gly,strain1')
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('s_1','fontsize',15)
set(gca,'fontsize',15)
title('first rate of strain')
box

%---
% second rate-of-strain
%---

figure(5)
mesh(glx,gly,strain2')
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('s_2','fontsize',15)
set(gca,'fontsize',15)
title('second rate of strain')
box

%---
% eigenvector 1
%---

figure(6)
hold on
axis equal
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
set(gca,'fontsize',15)
title('first principal eigevector')
box

for i=1:Nx+1
 plot([glx(i),glx(i)],[ay,by],'c')
end

for j=1:Ny+1
 plot([ax,bx],[gly(j),gly(j)],'c')
end

for i=1:Nx+1
 for j=1:Ny+1
     vector = draw_arrow_2d ...
     ...
     (gx(i,j),gy(i,j),1.5*egv1_x(i,j)/Ny,1.5*egv1_y(i,j)/Ny);
   plot(vector(:,1),vector(:,2),'k');
  end
end

%---
% eigenvector 2
%---

figure(7)
hold on
axis equal
xlabel('x','fontsize',14)
ylabel('y','fontsize',14)
set(gca,'fontsize',14)
title('second principal eigevector')
box on

for i=1:Nx+1
 plot([glx(i),glx(i)],[ay,by],'c')
end

for j=1:Ny+1
 plot([ax,bx],[gly(j),gly(j)],'c')
end

for i=1:Nx+1
 for j=1:Ny+1
     vector = draw_arrow_2d ...
     ...
     (gx(i,j),gy(i,j),1.5*egv2_x(i,j)/Ny,1.5*egv2_y(i,j)/Ny);
   plot(vector(:,1),vector(:,2),'k');
  end
end
