      program rec_2d_strml

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement
c----------------------------------------

c-----------------------------------------------
c Flow in a rectangular domain
c
c This program performs the following functions:
c
c (a) It generates a uniform Cartesian grid
c     in a rectangular domain in the xy plane
c
c (b) It prints velocity vectors at the grid points
c
c (c) It interpolates for the velocity vector at arbitrary
c     points.
c
c (d) It draws streamlines
c
c  SYMBOLS:
c  --------
c
c  ax, bx:  domain is confined in ax<x<bx
c  ay, by:  domain is confined in ay<y<by
c
c  Nx :  number of x intervals
c  Ny :  number of y intervals
c
c  Dx :  length of x interval
c  Dy :  length of y interval
c
c  glx(i) :  x-position of vertical ith grid line
c  gly(i) :  y-position of horizontal ith grid line
c
c  [ gx(i,j),  gy(i,j)] : coordinates of (i,j) grid point
c  [gux(i,j), guy(i,j)] : velocity at (i,j) grid point
c
c-----------------------------------------------

      Dimension glx(129), gly(129)

      Dimension  gx(129,129), gy(129,129)
      Dimension gux(129,129),guy(129,129)

      Dimension x0(200),y0(200)
      Dimension xstr(500),ystr(500)

      Dimension vector (5,2)   ! auxiliary five-point arrowed vector

      Parameter(Nmax=500)

c---
c common blocks
c---

      common/grid_2d_1/glx,gly
      common/grid_2d_2/gx,gy
      common/grid_2d_3/gux,guy

c---
c constants
c---

      Null   = 0
      None   = 1
      Ntwo   = 2
      Nthree = 3
      Nfour  = 4
      Nfive  = 5

      zero = 0.0D0

      pi  = 3.14159265358D0
      pi2 = 2.0*pi

c---
c Read parameters
c from file: rec_2d.dat
c---

      open (3,file="rec_2d_strml.dat")
       read (3,*) ax,bx                  ! x limits
       read (3,*) ay,by                  ! y limits
       read (3,*) Nx,Ny                  ! divisions
       read (3,*) Ds                     ! travel distance for streamlines
      close (3)

c--------
c prepare
c--------

      Nx1 = Nx+1
      Ny1 = Ny+1

      Dx = (bx-ax)/Nx
      Dy = (by-ay)/Ny

      xmin = ax
      xmax = bx
      ymin = ay
      ymax = by

c----
c choose the flow
c---

 98   Continue      ! return to repeat

      write (6,*)
      write (6,*)   " MENU OF FLOWS"
      write (6,*)
      write (6,*)   " 1 for extensional flow"
      write (6,*)   " 2 for rotational flow"
      write (6,*)   " 3 for simple shear flow"
      write (6,*)   " 4 for a stagnation-point flow"
      write (6,*)   " 5 for a wake-type of flow (Kavasznay)"
      write (6,*)   " 0 to quit"
      write (6,*)   "----------"
      read  (5,*) Iflow

      if(Iflow.eq.0) Go to 99

c-----
c Menu
c-----

 97   Continue      ! return to repeat

      write (6,*)
      write (6,*)   " MENU OF SERVICES"
      write (6,*)
      write (6,*)   " 1 to draw velocity vectors in 2D"
      write (6,*)   "   at the grid points"
      write (6,*)   " 2 to interpolate and draw velocity vectors"
      write (6,*)   "   in 2D at points within the cells"
      write (6,*)   " 3 to draw streamlines"
      write (6,*)   " 0 to quit"
      write (6,*)   "----------"
      read  (5,*) menu

      if(menu.eq.0) Go to 98

c--------
c prepare
c--------

      Nx1 = Nx+1
      Ny1 = Ny+1

      Ntot = Nx1*Ny1   ! total number of grid points

      dx = (bx-ax)/Nx
      dy = (by-ay)/Ny

c-----------------
c open output file
c-----------------

      open (2,file="rec_2d_strml.out")

c-----------------------------------
c compute grid lines and grid points
c-----------------------------------

      call grid_2d
     +
     +    (ax,bx
     +    ,ay,by
     +    ,Nx,Ny
     +    )

c----------------------------------------
c specify the velocity at the grid points
c----------------------------------------

      Do i=1,Nx1
       Do j=1,Ny1

         px = gx(i,j)
         py = gy(i,j)

        if(Iflow.eq.1) then             ! elongational flow
          gux(i,j) = px
          guy(i,j) =-py
        elseif(Iflow.eq.2) then        ! rotational flow
          gux(i,j) =-py
          guy(i,j) = px
        elseif(Iflow.eq.3) then        ! simple shear flow
          gux(i,j) =  py
          guy(i,j) =  0.0
        elseif(Iflow.eq.4) then
          gux(i,j) = py*(3.0*py-2.0*px) 
          guy(i,j) = py**2
        elseif(Iflow.eq.5) then
          wn  = pi2/0.5
          pxs = px+0.5
          gux(i,j) = 1.0-cos(wn*py)*exp(-1.0*wn*pxs)
          guy(i,j) =    -sin(wn*py)*exp(-1.0*wn*pxs)
         else
          write (6,*) "Sorry, this choice is not available"
          stop
         end if

       End Do
      End Do

c--------------------------------------
c Print: grid lines
c        velocity vectors at grid points
c
c in file: rec_2d_strml.out
c--------------------------------------

      if(menu.eq.1) then

c-----
c Prepare to draw velocity vectors
c
c Define vector plotting scale.
c The maximum length of the velocity vector
c will be 0.75 times the maximum of the
c grid spacing: dx or dy
c------

      scale = 0.0

      Do i=1,Nx1
       Do j=1,Ny1
        vmax = sqrt(gux(i,j)**2+guy(i,j)**2)
        If(vmax.gt.scale) scale = vmax
       End Do
      End Do

      dmax = dx
      If(dy.gt.dmax) dmax = dy

      scale = 0.75*dmax/scale

c---
c print grid lines
c in file: rec_2d_strml.out
c---

      Do i=1,Nx1
       write (2,100) Ntwo
       write (2,100) None,glx(i),ay
       write (2,100) Ntwo,glx(i),by
      End Do

      Do j=1,Ny1
       write (2,100) Ntwo
       write (2,100) None,ax,gly(j)
       write (2,100) Ntwo,bx,gly(j)
      End Do

c---
c print the 5-point velocity arrows
c---

      Do i=1,Nx1
       Do j=1,Ny1

        xl = gux(i,j)*scale
        yl = guy(i,j)*scale

        call draw_arrow_2d
     +
     +   (gx(i,j),gy(i,j)
     +   ,xl,yl
     +   ,vector
     +   )

       write (2,100) Nfive
       Do k=1,5
        write (2,100) k,vector(k,1),vector(k,2)
       End Do

       End Do
      End Do

c------------------------------------------
c Print:  grid lines
c         velocity vectors at cell mid-points
c
c in file:  PLOTDAT
c------------------------------------------

      elseif(menu.eq.2) then

c-----
c Prepare to draw velocity vectors
c
c Define vector plotting scale.
c The maximum length of the velocity vector
c will be 0.75 times the maximum of the
c grid spacing: dx or dy
c------

      scale = 0.0

      Do i=1,Nx1
       Do j=1,Ny1
        vmax = sqrt(gux(i,j)**2+guy(i,j)**2)
        If(vmax.gt.scale) scale = vmax
       End Do
      End Do

      dmax = dx
      If(dy.gt.dmax) dmax = dy

      scale = 0.75*dmax/scale

c---
c print grid lines
c in file: rec_2d_strml.out
c---

      Do i=1,Nx1
       write (2,100) Ntwo
       write (2,100) None,glx(i),ay
       write (2,100) Ntwo,glx(i),by
      End Do

      Do j=1,Ny1
       write (2,100) Ntwo
       write (2,100) None,ax,gly(j)
       write (2,100) Ntwo,bx,gly(j)
      End Do

c---
c interpolate within each cell
c---

      xf = 0.223    ! x location factor within a cell; between 0  and 1
      yf = 0.567    ! y location factor within a cell; between 0  and 1

      xfc = 1.0-xf  ! complementary
      yfc = 1.0-yf  ! complementary

      Do i=1,Nx

       vpx = xf*glx(i) + xfc*glx(i+1)   ! velocity point

       Do j=1,Ny

        vpy = yf*gly(j) + yfc*gly(j+1)  ! velocity point
       
c----
c Interpolate for the velocity vector
c at the point" vpx, vpy
c The results is: vux, vuy
c----

         call rec_2d_int
     +
     +    (ax,bx
     +    ,ay,by
     +    ,Nx,Ny
     +    ,vpx,vpy
     +    ,vux,vuy
     +    )

c---
c print five-point velocity arrows
c in file: rec_2d_strml.out
c---

         xl = vux*scale
         yl = vuy*scale

         call draw_arrow_2d
     +
     +    (vpx,vpy
     +    ,xl,yl
     +    ,vector
     +    )

        write (2,100) Nfive
        Do k=1,5
         write (2,100) k,vector(k,1),vector(k,2)
        End Do

       End Do
      End Do

c------------------------------------------
c compute streamlines and record
c in file: rec_2d_strml.out
c------------------------------------------

      elseif(menu.eq.3) then

c---
c first print grid lines
c in file:  PLOTDAT
c---

      Do i=1,Nx1
       write (2,100) Ntwo
       write (2,100) i,glx(i),ay
       write (2,100) i,glx(i),by
      End Do

      Do j=1,Ny1
       write (2,100) Ntwo
       write (2,100) j,ax,gly(j)
       write (2,100) j,bx,gly(j)
      End Do

c---
c read the starting points
c stop when last entry is > 998
c---

      if(Iflow.eq.1) then
         open (4,file="ext_init.dat")
      elseif(Iflow.eq.2) then
         open (4,file="rot_init.dat")
      elseif(Iflow.eq.3) then
         open (4,file="ssf_init.dat")
      elseif(Iflow.eq.4) then
         open (4,file="spf_init.dat")
      elseif(Iflow.eq.5) then
         open (4,file="kov_init.dat")
      else
         open (4,file="str_init.dat")
      end if

      read (4,*)
      Nstr = 0
        Do i=1,200
          read (4,*) x0(i),y0(i) 
          if(x0(i).gt.998) Go to 29
          Nstr = Nstr+1
        End Do
 29   Continue
      close (4)

      if(Nstr.eq.0) then
        write (6,*) 
        write (6,*) " Could not find data for streamlines"
        write (6,*) " Please check the main program"
        Stop
      elseif(Nstr.eq.1) then
        write (6,*)
        write (6,*) " Will plot ",Nstr," streamline"
        write (6,*)
        write (6,*) " Starting point:"
        write (6,*)
      else
        write (6,*)
        write (6,*) " Will plot ",Nstr," streamlines"
        write (6,*)
        write (6,*) " Starting points:"
        write (6,*)

      end if

      Do i=1,Nstr
        write (6,100) i,x0(i),y0(i)
      End Do

c---------------------------------
c Begin generating the streamlines
c---------------------------------

      Do 700 l=1,Nstr       ! loop over streamlines

      xn = x0(l)
      yn = y0(l)

      xstr(1) = xn            ! new point
      ystr(1) = yn            ! new point

c----
c Integrate by second-order Runge-Kutta
c---- 

      Do 701 i=2,Nmax         ! step in time

      call rec_2d_int
     +
     +   (ax,bx
     +   ,ay,by
     +   ,Nx,Ny
     +   ,xn,yn
     +   ,ux,uy
     +   )

      Umag = sqrt(ux*ux+uy*uy)
      Dt   = Ds/Umag             ! establish time step
                                 !  for constant travel distance
      x1 = xn + Dt*ux
      y1 = yn + Dt*uy

         If(x1.gt.xmin.and.x1.lt.xmax
     + .and.y1.gt.ymin.and.y1.lt.ymax
     +     ) then

      call rec_2d_int
     +
     +   (ax,bx
     +   ,ay,by
     +   ,Nx,Ny
     +   ,x1,y1
     +   ,ux1,uy1
     +   )
        Else
         ux1 = ux
         uy1 = uy
        End If

      Dth = 0.5*Dt
      xn  = xn + Dth*(ux+ux1)
      yn  = yn + Dth*(uy+uy1)

c---
c Begin stopping checks
c---

      Istop = 0

c---
c crossed plotting boundaries ?
c interpolate to place the point on the boundaries
c and stop
c---

      if(xn.lt.xmin) then
        yn = ystr(i-1)+(yn-ystr(i-1))*(xmin-xstr(i-1))
     +                                /(xn-xstr(i-1))
        xn = xmin
        Istop = 1
      end if

      if(xn.gt.xmax) then
        yn = ystr(i-1)+(yn-ystr(i-1))*(xmax-xstr(i-1))
     +                                 /(xn-xstr(i-1))
        xn = xmax
        Istop = 1
      end if

      if(yn.lt.ymin) then
        xn = xstr(i-1)+(xn-xstr(i-1))*(ymin-ystr(i-1))
     +                                 /(yn-ystr(i-1))
        yn = ymin
        Istop = 1
      end if

      if(yn.gt.ymax) then
        xn = xstr(i-1)+(xn-xstr(i-1))*(ymax-ystr(i-1))
     +                                 /(yn-ystr(i-1))
        yn = ymax
        Istop = 1
      end if

c---
c done with the checks
c---

      xstr(i) = xn
      ystr(i) = yn

c---
c check for a closed streamline after 5 steps
c---

      If(i.gt.5) then
       Dist = sqrt((xn-xstr(1))**2+(yn-ystr(1))**2)
       If(Dist.lt.Ds) Istop = 1
      End If

      If(Istop.eq.1) Go to 702

  701 Continue

c---------------

      i = i-1

  702 Continue

      write (2,100) i

      Do j=1,i
        write (2,100) j,xstr(j),ystr(j)
      End Do

      If(Istop.eq.0) write (6,*) "More than ",Nmax,"steps"
      write (6,*) "Stream line ",l," computed"

 700  Continue

c-----------

      End If

c---
c graphics
c---

      write (2,*) Null
      close (2)

      Go to 97

c-------------------
c return to the menu
c-------------------

      write (6,*)
      write (6,*)  " Done"
      write (6,*)  " Return to the main menu"
      write (6,*)

      Go to 98

c-------------
c Finishing up
c-------------

  99  Continue

c---
c Done
c---

  100 Format (1x,i3,5(1x,f12.8))
  101 Format (5(1x,f12.8))

      Stop
      End
