   close all
   clear all

%------------------------------------------------------
% Hydrostatic shape of a two-dimensional
% sessile drop resting on an inclined plane
% or pendant drop hanging underneath an inclined plane,
% for a specified drop area and contact points
%------------------------------------------------------

  global area capls Isp npts beta alpha1 alpha2 d x y

  Jsp=-1; % pendant
  Jsp= 1; % sessile

  gac = 1.0;   % acceleration of gravity
  gamma = 3.0; % surface tension
  rhod = 1.0;  % density of the drop
  rhoa = 0.0;  % density of the ambient fluid
  area = pi;  % drop area

  alpha=0.75*pi;   % contact angle
  alpha=0.95*pi;   % contact angle
  alpha=0.50*pi;   % contact angle

  npts =2*32;     %  number of interfacial markers

  epsilon=0.01; % for the shooting method
  maxiter=16; % for the shooting method
  tol=0.000000001; % for the shooting method

%---
% plotting
%---

 figure(1)
 hold on
 xlabel('y','fontsize',15)
 ylabel('x','fontsize',15)
 set(gca,'fontsize',15)
% axis([-1.25 0.75 -0.5 1.0])
 axis equal
 box on

%---
% compute the drop shape
% on a horizontal plane
%---

  rcrc = sqrt(area/(alpha-0.5*sin(2.0*alpha)));
  B = 1.0/rcrc;

  [B,x,y] = drop_2d  ...
 ...
   (Jsp,gac,gamma,rhod,rhoa,area ...
   ,alpha,npts,epsilon,maxiter,tol ...
   ,B);

%---
% position of the contact line
%---

  d=2*y(npts+1);

%---
%  shift to reset the origin at the first contact point
%---

  xreflect = x;
  yreflect =-y;

  shiftx = x(npts+1);
  shifty = y(npts+1);

  x = x-shiftx;
  y = y-shifty;

  xreflect = xreflect-shiftx;
  yreflect = yreflect-shifty;

%---
% plot
%---

 xwall(1)=-3.5;
 ywall(1)= 0;
 xwall(2)= 1.0;
 ywall(2)= 0;

%---
% circular interface
%---

 Ncrc = 64;
 dtheta = alpha/Ncrc;
 xc = rcrc*cos(alpha);

 for i=1:Ncrc+1
  theta = pi-alpha+(i-1)*dtheta;
  xcrc(i) =xc+rcrc*cos(theta);
  ycrc(i) =   rcrc*sin(theta);
 end

%---
% wall
%---

  patch([xwall xwall(2) xwall(1)] ,[ywall ywall(2)-0.2 ywall(1)-0.2],'g')
%  plot( ycrc,-xcrc,'--r')
%  plot(-ycrc,-xcrc,'--r')
 plot(xwall,ywall,'k')

 plot( y,Jsp*x,'-k.')
 plot( yreflect,Jsp*xreflect,'-k.')

 pause(2)

%------------
% prepare to rotate
%------------

 drho  = rhod-rhoa;            % density difference

 Isp=1.0;
 if(drho<0)
   Isp = -1.0;
 end

 capls = gamma/(gac*abs(drho));  % square of the capillary length

 C = B;
 C = B-shiftx/capls;

 npts = 2*npts;

 alpha1=alpha;
 alpha2=alpha;

% options = optimset('TolFun',0.0000001,'MaxFunEvals',200000,'Display','iter');
% options = optimset('TolFun',0.0000001,'MaxFunEvals',200000);

%------------
% start rotating
%------------

 Nrepeat=4*128;

 for repeat=1:Nrepeat+1

  beta = (repeat-1.0)*2*pi/Nrepeat;
  csb = cos(beta);
  snb = sin(beta);
  ROT = [csb, snb; -snb, csb];  % rotation matrix for graphics

  solution = [C alpha1 alpha2];

%---
% matlab solver
%---

% solution = fsolve(@drop_2di1_ode, solution,options)

%---
% newton's method
%---

  Niter = 10;
  eps = 0.001;

  [solution,F,Iflag] = drop_2di2_newton3 ...
     ...
    (Niter ...
    ,eps ...
    ,solution ...
    );

  if(Iflag==1) 
   disp (' drop_2di2: nonlinear system  solver failed')
   break;
  end

%---
% distribute the solution
%---

  C = solution(1)
  alpha1 = solution(2)
  alpha2 = solution(3)

%---
% plot
%---

  xplot = -y;
  yplot =  x;

 for i=1:npts+1
   xx = [xplot(i), yplot(i)];
   xx = ROT*xx';
   xplot(i) = xx(1);
   yplot(i) = xx(2);
 end

 plot(xplot,yplot,'k-')
 patch(xplot,yplot,'y')

 hold on
 plot(csb*[1.0,-4.5],snb*[-1.0,4.5],'k','LineWidth',3)
 axis equal
 xlabel('x','fontsize',15)
 ylabel('y','fontsize',15)
 set(gca,'fontsize',15)
 axis off
 hold off
 pause(0.1)

% if(alpha2<0.05*pi) break; end
% if(alpha2>0.95*pi) break; end

%===
end
%===

%---
% done
%---
