      program chan_2d_2l_dr

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Flow of two superposed layers in a channel
c with parallel walls
c
c The channel walls are located at y = +-a
c
c Fluid 1 is the lower fluid,
c       2 is the upper fluid.
c
c
c SYMBOLS:
c --------
c
c nprof: number of points for a velocity profile
c h1:    lower layer thickness
c visc1: lower fluid viscosity
c visc2: upper fluid viscosity
c rho1:  lower fluid density
c rho2:  upper fluid density
c gac:   acceleration of gravity
c V1:    translational velocity of the lower wall
c V2:    translational velocity of the upper wall
c chi:   negative of the pressure gradient
c beta: inclination angle
c===============================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      pi = 3.14159265358D0

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

c------
c input
c------

      open (1,file='chan_2d_2l.dat')
        read (1,*) nprof
        read (1,*) a
        read (1,*) h1
        read (1,*) visc1
        read (1,*) visc2
        read (1,*) rho1
        read (1,*) rho2
        read (1,*) gac
        read (1,*) V1
        read (1,*) V2
        read (1,*) chi
        read (1,*) beta
      close (1)

c---
c prepare
c---

      beta = beta*pi

      h2 = 2.0*a-h1   ! upper layer thickness

      cth0 = Dcos(beta)
      sth0 = Dsin(beta)

      open (2,file='chan_2d_2l.out')

c---
c Draw the walls so that they appear inclined
c----

c lower wall

      write (2,900) Ntwo
      xx = -a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c upper wall

      write (2,900) Ntwo
      xx = -a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---
c draw a vertical line
c---

      write (2,900) Ntwo
      xx =  zero
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = zero
      yy = a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c------------------------------
c draw a horizontal line at y=0
c------------------------------

      write (2,900) Ntwo

      xx = -1.2D0*a
      yy = 0.0D0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  1.2D0*a
      yy = 0.0D0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---
c draw a horizontal line at the interface
c---

      write (2,900) Ntwo
      xx = -a
      yy = -a+h1
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  a
      yy = -a+h1
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---
c draw the velocity profile
c---

      write (6,*)
      write (6,*) " Velocity profile"
      write (6,*)

      nproftot = 2*nprof+1

      write (2,900) nproftot

c---
c point on lower wall
c---

      Ic = 1  ! counter

      y  = -a
      u  = V1

      xx = u
      yy = y

      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0

      write (2,900) Ic,xprint,yprint
      write (6,900) Ic,xprint,yprint

c---
c span the layers
c---

      Do i=1,2

       if(i.eq.1) then
         step = h1/(nprof+1.0D0-1.0D0)
       elseif(i.eq.2) then
         step = h2/(nprof+1.0D0-1.0D0)
       end if

       Do j=1,nprof

        Ic = Ic+1

        y = y+step

        call chan_2d_2l 
     +
     +     (a
     +     ,h1
     +     ,visc1
     +     ,visc2
     +     ,rho1
     +     ,rho2
     +     ,gac
     +     ,V1
     +     ,V2
     +     ,chi
     +     ,beta
     +     ,y
     +     ,yI
     +     ,uI
     +     ,xi1,xi2
     +     ,Q1,Q2
     +     ,u
     +     )

       xx = u
       yy = y

       xprint = xx*cth0+yy*sth0
       yprint =-xx*sth0+yy*cth0

       write (2,900) Ic,xprint,yprint
       write (6,900) Ic,xprint,yprint

       End Do

      End Do

      write (2,*) null
      close (2) 

c---
c interfacial location, velocity, shear rates
c flow rates and mean velocity
c---

      write (6,499) yI
      write (6,500) uI
      write (6,501) xi1,xi2
      write (6,502) Q1,Q2

      velmean1 = Q1/h1
      velmean2 = Q2/h2

      write (6,503) velmean1,velmean2

c----------------------
c test on shear rates
c it should be: ratio = 1
c---
c
c     ratio = xi1*visc1/(xi2*visc2)
c     write (6,*) ratio
c----------------------

c-----
c Done
c-----

 499  Format ("Interface located at y=: ",f15.10)
 500  Format ("Interfacial velocity   : ",f15.10)
 501  Format ("Interfacial shear rates: ",f15.10,1x,f15.10)
 502  Format ("Flow rates             : ",f15.10,1x,f15.10)
 503  Format ("Mean velocity          : ",f15.10,1x,f15.10)

 900  Format (1x,i3,4(1x,f10.5))

      Stop
      End
