      subroutine chan_2d_ml 
     +
     +   (NLR
     +   ,a
     +   ,thick
     +   ,visc
     +   ,den
     +   ,gac
     +   ,V1
     +   ,V2
     +   ,chi
     +   ,beta
     +   ,y
     +   ,yI
     +   ,uI
     +   ,xi1,xi2
     +   ,Q
     +   ,u
     +   )

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Multi-layer flow through a channel confined
c between two parallel plane walls located
c at y = +-a
c
c Film numbered 1 is adjacent to the lower wall
c Film numbered NLR is adjacent to the upper wall
c
c The velocity in the ith film is given by:
c
c u(i) = af(i) + bf(i)*y - gf(i)*y**2
c
c SYMBOLS:
c --------
c
c NLR:    Number of layers
c thick:  film thicknesses
c yI(i):  y-position of the ith interface
c uI(i):  velocity of the ith interface
c xi1(i): shear rate on lower side of the ith interface
c xi2(i): shear rate on upper side of the ith interface
c Q(i):   flow rate of the ith film
c beta:   inclination angle
c gac:    magnitude of the acceleration of gravity
c chi:    negative of the pressure gradient
c==========================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension gf(10),af(10),bf(10)
      Dimension yI(10),uI(10),xi1(10),xi2(10),Q(10)

c----------
c constants
c----------

      pi = 3.14159 265358D0

c----------------------------
c set the interface positions
c----------------------------

      yI(1) = -a+thick(1)

      Do i=2,NLR
       yI(i) = yI(i-1) + thick(i)
      End Do
 
c-----------------------------------------
c the velocity in the ith film is given by:
c
c u(i) = af(i) + bf(i)*y - gf(i)*y**2
c
c compute gf(i)
c-----------------------------------------

      fc = gac*Dsin(beta)

      Do i=1,NLR
        gf(i) = 0.5D0*(chi+fc*den(i))/visc(i)
      End Do

c---------------------------------------------------
c In the first pass,
c set the shear rate at upper wall "alpha" to 0
c and compute the residual of the upper wall boundary
c condition: error0
c
c In the second pass,
c set the shear rate at upper wall "alpha" to 1
c and compute the residual of the upper wall boundary
c condition: error1
c
c In the third pass,
c set the proper shear rate:  
c
c alpha = error0/(error1-error0)
c---------------------------------------------------

      alpha = 0.0D0

      Do Ipass=1,3

c---
c specified upper wall shear stress
c---

      bf(NLR) = alpha + 2.0*gf(NLR)*a

c---
c compute bf(i) by recursion starting from the top
c and requiring continuity of shear stress
c across the interfaces
c---

      Do i=NLR-1,1,-1
        bf(i) = 2.0D0*gf(i)*yI(i)
     +          + visc(i+1)/visc(i)*(bf(i+1)-2.0D0*gf(i+1)*yI(i))
      End Do

c---
c compute af(1) to satisfy the non-slip condition
c at the lower wall
c---

      af(1) = V1+(bf(1)+gf(1)*a)*a

c---
c compute af(i), i=2,...,NLR by recursion
c starting from the bottom
c and requiring continuity of velocity
c---

      Do i=1,NLR-1
       af(i+1) = af(i)+( (bf(i)-bf(i+1)) 
     +                 -(gf(i)-gf(i+1))*yI(i))*yI(i)
      End Do

c---
c error in the no-slip condition on the upper wall
c error = u(y=a)-V2
c---

      if(Ipass.eq.1) then   ! return for the second pass

        error0 = af(NLR)+(bf(NLR)-gf(NLR)*a)*a-V2
        alpha = 1.0D0

      else if(Ipass.eq.2) then    ! return for the final pass

        error1 = af(NLR)+(bf(NLR)-gf(NLR)*a)*a-V2
        alpha = error0/(error0-error1)

      end if

c---
      End Do
c---

c-----
c Done with Ipass
c-----

c---
c error2 should be zero
c---


      error2 = af(NLR)+(bf(NLR)-gf(NLR)*a)*a-V2
c     write (6,*) error0,error1,error2

c---
c compute interfacial velocities
c and shear rates
c---

      Do i=1,NLR-1
       uI(i) = af(i)+(bf(i)-gf(i)*yI(i))*yI(i)
       xi1(i) = bf(i)  -2.0*gf(i)  *yI(i)
       xi2(i) = bf(i+1)-2.0*gf(i+1)*yI(i)
      End Do

c----
c compute the flow rates
c ith film is confined between i-1 and i interfaces
c----

      yI0 = -a
      Q(1) = af(1)*(yI(1)-yI0)
     +      +bf(1)*(yI(1)**2-yI0**2)/2.0D0
     +      -gf(1)*(yI(1)**3-yI0**3)/3.0D0

      Do i=2,NLR
       ia = i-1
       Q(i) = af(i)*(yI(i)   -yI(ia))
     +       +bf(i)*(yI(i)**2-yI(ia)**2)/2.0D0
     +       -gf(i)*(yI(i)**3-yI(ia)**3)/3.0D0
      End Do

c--------------------------
c compute the velocity at y
c--------------------------

c---
c identify the host layer
c---

      Do i=1,NLR
       if(y.lt.yI(i)) then
         Go to 92
       end if
      End Do

 92   Continue

      if(i.gt.NLR) i=NLR
     
c---
c evaluate the velocity
c---

      u = af(i) + (bf(i)-gf(i)*y)*y

c-----
c done
c-----

      Return
      End
