function [Q,u,xi1,xi2,yI,uI] = chan_2d_ml ...
     ...
       (NLR ...
       ,a ...
       ,thick ...
       ,visc ...
       ,den ...
       ,gac ...
       ,V1 ...
       ,V2 ...
       ,chi ...
       ,beta ...
       ,y ...
       )

%================================================
% Multi-layer flow through a channel confined
% between two parallel plane walls located at y = +-h
%
% Film numbered 1 is adjacent to the lower wall
% Film numbered NLR is adjacent to the upper wall
%
% The velocity in the ith film is given by:
%
% u(i) = af(i) + bf(i)*y - gf(i)*y^2
%
% SYMBOLS:
% --------
%
% NLR:    Number of layers
% thick:  film thicknesses
% yI(i):  y-position of the ith interface
% uI(i):  velocity of the ith interface
% xi1(i): shear rate on lower side of the ith interface
% xi2(i): shear rate on upper side of the ith interface
% Q(i):   flow rate of the ith film
% th0:    plane inclination angle
% gac:    magnitude of the acceleration of gravity
% chi:    negative of the pressure gradient
%================================================

%----------------------------
% set the interface positions
%----------------------------

   yI(1) = -a + thick(1);

   for i=2:NLR
     yI(i) = yI(i-1) + thick(i);
   end

%-----------------------------------------
% The velocity in the ith film is given by
%
% u(i) = af(i) + bf(i)*y - gf(i)*y^2
%
% compute gf(i)
%-----------------------------------------

  fc = gac*sin(beta);

  for i=1:NLR
    gf(i) = 0.5*(chi+fc*den(i))/visc(i);
  end

%---------------------------------------------------
% In the first pass,
% set the shear rate at upper wall "alpha" to 0
% and compute the residual of the upper wall boundary
% condition: error0
%
% In the second pass,
% set the shear rate at upper wall "alpha" to 1
% and compute the residual of the upper wall boundary
% condition: error1
%
% In the third pass,
% set the proper shear rate:
%
% alpha = error0/(error0-error1)
%-------------------

alpha = 0.0;

for Ipass=1:3

%---
% specified upper wall shear stress
%---

  bf(NLR) = alpha + 2.0*gf(NLR)*a;

%---
% Compute bf(i) by recursion starting from the top
% and requiring continuity of shear stress
% across the interfaces
%---

  for i=NLR-1:-1:1
    bf(i) = 2.0*gf(i)*yI(i) ...
          + visc(i+1)/visc(i)*(bf(i+1)-2.0*gf(i+1)*yI(i));
  end

%---
% compute af(1) to satisfy the non-slip condition
% at the lower wall
%---

  af(1) = V1+(bf(1)+gf(1)*a)*a;

%---
% compute af(i), i=2,...,NLR by recursion
% starting from the bottom
% and requiring continuity of velocity
%---

  for i=1:NLR-1
    af(i+1) = af(i)+( (bf(i)-bf(i+1)) ...
                -(gf(i)-gf(i+1))*yI(i))*yI(i);
  end

%---
% error in the no-slip condition on the upper wall
% error = u(y=a)-V2
%---

  if(Ipass==1) 

    error0 = af(NLR)+(bf(NLR)-gf(NLR)*a)*a-V2;
    alpha = 1.0;

  elseif(Ipass==2) 

    error1 = af(NLR)+(bf(NLR)-gf(NLR)*a)*a-V2;
    alpha = error0/(error0-error1);

  end

%---
   end % of Ipass
%---

%---
% compute interfacial velocities
% and shear rates
%---

  for i=1:NLR-1
     uI(i) = af(i)+(bf(i)-gf(i)*yI(i))*yI(i);
     xi1(i) = bf(i)  -2.0*gf(i)  *yI(i);
     xi2(i) = bf(i+1)-2.0*gf(i+1)*yI(i);
   end

%----
% compute the flow rates
% the ith layer is confined between i-1 and i interfaces
%----

  yI0 = -a;
  Q(1) = af(1)*(yI(1)-yI0) ...
        +bf(1)*(yI(1)^2-yI0^2)/2.0 ...
        -gf(1)*(yI(1)^3-yI0^3)/3.0;

 for i=2:NLR
   Q(i) = af(i)*(yI(i)-yI(i-1)) ...
         +bf(i)*(yI(i)^2-yI(i-1)^2)/2.0 ...
         -gf(i)*(yI(i)^3-yI(i-1)^3)/3.0;
 end

%--------------------------
% compute the velocity at y
%--------------------------

%---
% identify the host layer
%---

 for i=1:NLR
   if(y<yI(i))
     break
   end
 end

 if(i>NLR) i=NLR; end

%---
% evaluate the velocity
%---

  u = af(i)+(bf(i)-gf(i)*y)*y;

%-----
% done
%-----

 return
