      program chan_2d_ml_dr

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Multi-film flow through a channel confined
c between two parallel plane walls located at y = +-a
c
c Layer numbered 1 is adjacent to the the lower wall
c Layer numbered NLR is adjacent to the the upper wall
c
c The velocity profile inside the ith film is given by:
c
c u(i) = af(i) + bf(i)*y - gf(i)*y**2
c
c SYMBOLS:
c -------
c
c NLR:       Number of layers
c thick(i):  thickness of the ith layer
c yI(i):     y position of the ith interface
c uI(i):     velocity of the ith interface
c xi1(i):    shear rate on lower side of the ith interface
c xi2(i):    shear rate on upper side of the ith interface
c Q(i):      flow rate of ith film
c beta:      inclination angle
c gac:       magnitude of the acceleration of gravity
c chi:       negative of the pressure gradient
c nprof:     number of points per layer for a profile
c==========================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension yI(10),uI(10),xi1(10),xi2(10),Q(10)

c---
c constants
c---

      pi = 3.14159265358D0

      Null   = 0
      None   = 1
      Ntwo   = 2
      Nthree = 3
      Nfour  = 4
      Nfive  = 5

      zero   = 0.0D0

c-------
c  input
c-------

      open (1,file='chan_2d_ml.dat')
       read (1,*) nprof
       read (1,*) a
       read (1,*) V1
       read (1,*) V2
       read (1,*) chi
       read (1,*) gac
       read (1,*) beta
       read (1,*) 
       read (1,*) NLR
       read (1,*) (thick(i),i=1,NLR-1)
       read (1,*) ( visc(i),i=1,NLR)
       read (1,*) (  den(i),i=1,NLR)

      close (1)

c----------------------------
c check for consistency
c and set top-layer thickness
c----------------------------

      sum = 0.0D0

      Do i=1,NLR-1
        sum = sum + thick(i)
      End Do

      thick(NLR) = 2.0*a-sum

      if(thick(NLR).lt.0) then
       write (6,*)
       write (6,*) " chan_2d_ml_dr: layer thicknesses are improper"
       stop
      end if

c---
c  interface positions
c---

      yI(1)=-a+thick(1)

      Do i=2,NLR
       yI(i) = yI(i-1)+thick(i)
      End Do

c--------
c prepare
c--------

      beta  = beta*pi

      cth0 = Dcos(beta)
      sth0 = Dsin(beta)

      open (2,file='chan_2d_ml.out')
      write (2,900) NLR

c--------------------------------------------
c draw the walls so that they appear inclined
c--------------------------------------------

c---
c paint the lower wall
c---

      write (2,900) Nfive

      xx = -2.0*a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  2.0*a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx =  2.0*a
      yy = -1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -2.0*a
      yy = -1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -2.0*a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c---
c paint the upper wall
c---

      write (2,900) Nfive

      xx = -2.0*a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  2.0*a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx =  2.0*a
      yy =  1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -2.0*a
      yy =  1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -2.0*a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c---
c Draw lines at walls and interfaces
c---

c--- bottom wall

      write (2,900) Ntwo

      xx = -2.0*a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  2.0*a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c--- upper wall

      write (2,900) Ntwo

      xx = -2.0*a
      yy = a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  2.0*a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c--- interfaces
 
      Do i=1,NLR-1

       write (2,900) Ntwo

       xx = -2.0*a
       yy = yI(i)
       xprint = xx*cth0+yy*sth0
       yprint =-xx*sth0+yy*cth0
       write (2,900) None,xprint,yprint

       xx = 2.0*a
       yy = yI(i)
       xprint = xx*cth0+yy*sth0
       yprint =-xx*sth0+yy*cth0
       write (2,900) Ntwo,xprint,yprint

      End Do

c------------------------------
c draw a horizontal line at y=0
c------------------------------

      write (2,900) Ntwo

      xx = -2.5D0*a
      yy = 0.0D0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  2.5D0*a
      yy = 0.0D0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---------------------
c Draw a vertical line
c---------------------

      write (2,900) Ntwo

      xx =  zero
      yy = -1.8*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = zero
      yy = 1.8*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c--------------------------
c draw the velocity profile
c--------------------------
     
c     write (6,*)
c     write (6,*)  " velocity profile"
c     write (6,*)

      nproftot = NLR*nprof+1    ! total number of points for profile

      write (2,900) NLR*nprof+1    ! total number of points for profile

      Ic = 1           ! counter
      y  = -a
      u  = V1

      xx = u
      yy = y

      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0

      write (2,900) Ic,xprint,yprint
c     write (6,900) Ic,xprint,yprint


c---
      Do i=1,NLR
       step = thick(i)/(nprof+1.0D0-1.0D0)
       Do j=1,nprof

         y  = y+step
         Ic = Ic+1

         call chan_2d_ml
     +
     +     (NLR
     +     ,a
     +     ,thick
     +     ,visc
     +     ,den
     +     ,gac
     +     ,V1
     +     ,V2
     +     ,chi
     +     ,beta
     +     ,y
     +     ,yI,uI
     +     ,xi1,xi2
     +     ,Q
     +     ,u
     +     )

        xx = u
        yy = y

        xprint = xx*cth0+yy*sth0
        yprint =-xx*sth0+yy*cth0

        write (2,900) Ic,xprint,yprint
c       write (6,900) Ic,xprint,yprint

       End Do
      End Do

c---
c end of drawing
c---

      write (2,*) null
      close (2)

c-----------------
c printing session
c-----------------

      write (6,*)
      write (6,*) " Interf y pos, velocity, shear rate 1, shear rate 2"
      write (6,*)

      Do i=1,NLR-1
       write (6,900) i,yI(i),uI(i),xi1(i),xi2(i)
c      ratio = xi1(i)*visc(i)/(xi2(i)*visc(i+1))  ! should be unity
c      write (6,*) ratio 
      End Do

      write (6,*)
      write (6,*) " Layer flow rates and mean velocity"
      write (6,*)

      Do i=1,NLR
       umean = Q(i)/(yI(i)-yI(i-1))
       write (6,900) i,Q(i),umean
      End Do

c-----
c Done
c-----

  99  Continue

 900  Format (1x,i3,9(1x,f10.5))

      Stop
      End
