      subroutine tube_ann_sw_ml
     +
     +  (NLR
     +  ,a1
     +  ,a2
     +  ,thick
     +  ,visc
     +  ,Omega1
     +  ,Omega2
     +  ,s
     +  ,sI
     +  ,uI
     +  ,xi1,xi2
     +  ,Q
     +  ,u
     +  )

c===========================================
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c===========================================
c Multi-layer swirling flow inside an annular tube
c
c Layer numbered 1 is adjacent to the inner cylinder
c Layer numbered NLR is adjacent to outer cylinder
c
c The velocity in the ith film is given by:
c
c u(i) = bf(i)/sigma + cf(i)*sigma
c
c SYMBOLS
c ------
c
c a1:        inner tube radius
c a2:        outer tube radius
c Omega1:    inner tube angular velocity
c Omega2:    outer tube angular velocity
c NLR:       Number of layers
c thick(i):  thickness of the ith layer
c sI(i):     radial position of the ith interface
c uI(i):     velocity of the ith interface
c xi1(i):    shear rate on lower side of the ith interface
c xi2(i):    shear rate on upper side of the ith interface
c Q(i):      flow rate of ith layer
c===========================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10)
      Dimension sI(10),uI(10)
      Dimension xi1(10),xi2(10),Q(10)
      Dimension bf(10),cf(10)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c---
c set the radial interface positions
c---

      sI(1) = a1+thick(1)

      Do i=2,NLR
       sI(i) = sI(i-1)+thick(i)
      End Do


c-----------------------------------------
c The velocity in the ith layer is given by:
c
c u(i) =  bf(i)/sigma + cf(i)*sigma
c
c In the first pass, set bf(1)=0
c and compute the residual of the inner cylinder boundary
c condition: error0
c
c In the second pass, set bf(1)=1
c and compute the residual of the inner cylinder boundary
c condition: error1
c
c In the third pass, set the proper value:
c
c bf(1) = error0/(error0-error1)
c-----------------------------------------

      bf(1) = 0.0D0

c---
      Do Ipass=1,3
c---

c---
c compute bf(i) by recursion
c starting from the center and requiring:
c continuity of shear stress across the interfaces
c
c The sigma-phi shear stress is given by:
c
c shear_stress = sigma d(u/sigma)/d(sigma)
c---

      Do i=1,NLR-1
        bf(i+1) = visc(i)/visc(i+1)*bf(i)
      End Do

c---
c compute cf(i) by recursion
c to satisfy continuity of velocity across the iterfaces
c---

      cf(1) = (Omega1*a1-bf(1)/a1)/a1

      Do i=1,NLR-1
        cf(i+1) = cf(i) + (bf(i)-bf(i+1))/sI(i)**2
      End Do

c---
c error in the no-slip condition on the inner wall
c error = u(s=a1)-V1
c---

      if(Ipass.eq.1) then
        error0 = bf(NLR)/a2+cf(NLR)*a2-Omega2*a2
        bf(1)  = 1.0D0
      elseif(Ipass.eq.2) then
        error1 = bf(NLR)/a2+cf(NLR)*a2-Omega2*a2
        bf(1)  = error0/(error0-error1)
      end if

c---
      End Do    ! of Ipass
c---


c---
c error2 should be equal to zero
c---

c     error2 = bf(NLR)/a2+cf(NLR)*a2-Omega2*a2
c     write (6,*) "error2=",error2
                                        
c---
c compute interfacial velocities
c and shear rates
c---

      Do i=1,NLR-1
       uI(i) = bf(i)/sI(i) + cf(i)*sI(i)
       xi1(i) = -2.0*bf(i) /sI(i)**3 
       xi2(i) = -2.0*bf(i+1)/sI(i)**3 
      End Do

c--------------------------------------------------
c compute the flow rates
c ith film is confined between i-1 and i interfaces
c--------------------------------------------------

      Q(1) = bf(1)*(log(sI(1)/a1))
     +     + cf(1)*0.5D0*(sI(1)**2-a1**2)

      Do i=2,NLR
       Q(i) = bf(i)*log(sI(i)/sI(i-1))
     +      + cf(i)*0.5D0*(sI(i)**2-sI(i-1)**2)
      End Do

c--------------------------
c compute the velocity at s
c--------------------------

c---
c identify the host layer
c numbered j
c---

c---
c identify the host layer numbered j
c---

      Do i=1,NLR
       if(s.lt.sI(i)) then
         Go to 92
       end if
      End Do

 92   Continue

      if(i.gt.NLR) i=NLR

c---
c evaluate the velocity at s
c---

      u  = bf(i)/s+cf(i)*s

c-----
c Done
c-----

      Return
      End
