      subroutine tube_crc_ml
     +
     +  (NLR
     +  ,a
     +  ,thick
     +  ,visc
     +  ,den
     +  ,gac
     +  ,chi
     +  ,beta
     +  ,s
     +  ,sI,uI
     +  ,xi1,xi2
     +  ,Q
     +  ,u
     +  )

c=============================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=============================================

c=============================================
c Multi-layer core-annular flow through a circular tube
c
c Film numbered 1 is at the centerline
c Film numbered NLR is adjacent to the wall
c
c The velocity in the ith film is given by:
c
c u(i) = af(i) + bf(i)*ln(sigma/a) - gf(i)*sigma**2
c
c SYMBOLS
c ------
c
c NLR:       Number of layers
c a:         tube radius
c thick(i):  thickness of the ith layer
c sI(i):     position of the ith interface
c uI(i):     velocity of the ith interface
c xi1(i):    shear rate on lower side of the ith interface
c xi2(i):    shear rate on upper side of the ith interface
c Q(i):      flow rate of ith layer
c beta:      tube inclination angle
c gac:       magnitude of the acceleration of gravity
c chi:       negative of the pressure gradient
c gac:       acceleration of gravity
c=============================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension sI(10),uI(10)
      Dimension xi1(10),xi2(10),Q(10)
      Dimension gf(10),af(10),bf(10)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c---
c interface positions
c---

      sI(1) = thick(1) 

      Do i=2,NLR
       sI(i) = sI(i-1)+thick(i)
      End Do

c-----------------------------------------
c The velocity in the ith layer is given by:
c
c u(i) = af(i) + bf(i)*ln(sigma/a) - gf(i)*sigma**2
c
c compute gf(i)
c-----------------------------------------

      fc = gac*sin(beta)

      Do i=1,NLR
        gf(i) = 0.25D0*(chi+fc*den(i))/visc(i)
      End Do

c---
c compute bf(i) by recursion
c starting from the centerline and
c by requiring continuity of shear stress
c across the interfaces
c---

      bf(1) = 0.0

      Do i=2,NLR
        bf(i) =  2.0D0*gf(i)*sI(i-1)**2
     +    + visc(i-1)/visc(i) * (bf(i-1)-2.0D0*gf(i-1)*sI(i-1)**2) 
      End Do

c---
c compute af(i) by recursion
c starting from the tube wall and
c and requiring continuity of velocity
c---

c     af(NLR) = -bf(NLR)*log(sI(NLR)/a) + gf(NLR)*sI(NLR)**2
      af(NLR) = gf(NLR)*a**2

      Do i = NLR-1,1,-1
       af(i) = af(i+1)+(bf(i+1)-bf(i))*log(sI(i)/a)
     +               -(gf(i+1)-gf(i))*sI(i)**2
      End Do

c---
c compute interfacial velocities
c and shear rates
c---

      Do i=1,NLR-1
       uI(i) = af(i) + bf(i)*log(sI(i)/a) - gf(i)*sI(i)**2
       xi1(i) = bf(i)  /sI(i) - 2.0D0*gf(i)  *sI(i)
       xi2(i) = bf(i+1)/sI(i) - 2.0D0*gf(i+1)*sI(i)
      End Do

c----
c compute the flow rates
c the ith film is confined between i-1 and i interfaces
c----

      sI1s = sI(1)**2
      Q(1) = af(1)*sI1s/2.0
     +     + bf(1)*0.5D0*sI1s*(log(sI(1)/a)-0.5D0)
     +      -gf(1)*sI1s**2/4.0D0
      Q(1) = pi2*Q(1)

      Do i=2,NLR
       sIis  = sI(i)**2
       sIias = sI(i-1)**2
       Q(i) = af(i)*(sIis-sIias)/2.0
     +       +bf(i)*0.5D0*(sIis *(log(sI(i)/a)-0.5D0)
     +                  -sIias*(log(sI(i-1)/a)-0.5D0))
     +       -gf(i)*(sIis**2-sIias**2)/4.0D0
       Q(i) = pi2*Q(i)
      End Do

c--------------------------
c compute the velocity at y
c--------------------------

c---
c identify the host layer
c numbered j
c---

      i = 1
 92   Continue
      if(i.gt.NLR) then
       i = NLR
       Go to 93
      end if

      if(s.gt.sI(i)) then
       i = i+1
       Go to 92
      end if

      j = i

  93  Continue

c---
c evaluate the velocity at s
c---

      if(s.gt.0.000001) then
       u = af(j) + bf(j)*log(s/a) - gf(j)*s**2
      else
       u = af(j)-gf(j)*s**2
      end if

c-----
c Done
c-----

      Return
      End
