clear all
close all

%=======================================
% Simulation of the normal motion of a curved body
% near a translating plane due to a lubrication force
%=======================================

%---
% parameters
%---

a = 0.25;       %  lubrication zone
b = 0.75;       %   lubrication zone
delta = 0.05;  %  parabolicity delta
c = 0.01;       %  initial minimum film thickness c
pleft = 0.0;      %  pressure at left end
prite =-50.0;      %  pressure at right end
visc = 1.0;    %  viscosity
Wload = 20.0;  %  normal load
V = 5.0;       %  velocity of translation V
Dt = 0.020;     %  time step
N = 2*2*32;        % discretization level
Nstep = 100;    % number of time steps

%---
% prepare
%---

Dx = (a+b)/N;
Dx2 = 2.0*Dx;

%---
% prepare to plot
%---

figure(1)
hold on
xlabel('x','fontsize',15)
ylabel('pressure','fontsize',15)
set(gca,'fontsize',15)
box on
axis([-b a -4000 4000])

%----------------------------
% Define nodes 
% extending from x = -b to a
%----------------------------

for i=1:N+1
   x(i) = -b+(i-1.0)*Dx;
   h(i) = c+delta*(x(i)/a)^2;
end

%---------------------------
% Compute dh/dx at interior nodes
% using central differences
%---------------------------

for i=2:N
 dhdx(i) = (h(i+1)-h(i-1))/Dx2;
end

%===============
 for Istep=1:Nstep   % time stepping
%===============

time(Istep) = (Istep-1)*Dt;
gap(Istep) = c;   % c for plotting

%---
% solve for the pressures
%----

[pi1,pi2] =  bear_2d_pressure  ...
...
       (visc ...
       ,x ...
       ,h ...
       ,dhdx ...
       ,Dx ...
       ,V ...
       ,N ...
       ,pleft,prite ...
       );

%--------------------------------------------
% Integrate the pressure using the trapezoidal
% rule to find the normal force exerted on the body
%--------------------------------------------

      F1 = 0.5*(pi1(1)+pi1(N+1));
      F2 = 0.5*(pi2(1)+pi1(N+1));

      for i=2:N
        F1 = F1 + pi1(i);
        F2 = F2 + pi2(i);
      end

      F1 = F1*Dx;
      F2 = F2*Dx;

%---
% Compute dc/dt to balance the load
% using the equation:
%
% F1 + F2 * dcdt = Wload
%---

      dcdt = (Wload-F1)/F2;

%---
% recoved the physical pressure;
%---

  for i=1:N+1
      press(i) = pi1(i) + dcdt*pi2(i);
  end

  figure(1)
% plot(x,pi1,'k-o')
% plot(x,pi2,'r-o')
  if(Istep==1)
   plot(x,press,'r.-')
  else
   plot(x,press,'k-')
  end

%---
% advance c and update the profile h(i)
%---

  dc = dcdt*Dt
  c = c + dc;

  for i=1:N+1
   h(i) = h(i)+dc;
  end

%===
end % of time stepping
%===

figure(2)
hold on
plot(time,gap,'k')
xlabel('time','fontsize',15)
ylabel('c','fontsize',15)
set(gca,'fontsize',15)
box on

