function [pi1,pi2] =  bear_2d_pressure  ...
...
  (visc ...
  ,x ...
  ,h ...
  ,dhdx ...
  ,Dx ...
  ,V ...
  ,N ...
  ,pleft,prite ...
  )

%===================================
% FDLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved
%
% This program is to be used only under the
% stipulations of the licencing agreement
%===================================

%==================================
% Solve the lubrication equation for the pressure
% using a finite-difference method
%
% Solutions are generated for longitudinal motion (pi1)
% and normal motion (pi2)
%
% SYMBOLS:
%
% x(i):           grid nodes
% h(i):           gap
% visc:           viscosity
% pleft:          left-end pressure
% prite:          right-end pressure
% act, btr, ctr:  tridiagonal entries
% rhs:            right-hand side
% sln:            solution for the pressure
%===================================

%---
% prepare
%---

 Dxh = 0.5*Dx;
 Dx2 = 2.0*Dx;
 Dxs = Dx^2;

%---
% auxiliary
%---

 for i=1:N+1
   q(i) = h(i)^3/visc;
 end

 dqdx(1) = (q(2)-q(1))/Dx;         % forward difference

 for i=2:N
    dqdx(i) = (q(i+1)-q(i-1))/Dx2;  % central difference
 end

 dqdx(N+1) = (q(N+1)-q(N))/Dx;     % backward difference

%---
% compute a particular solution
% with exact boundary conditions (pleft and prite)
%
% act, btr, ctr are the tridiagonal entries
% of the coefficient matrix
%---

      factor = 6.0*V*Dxs;

      atr(1) = -2.0*q(2);
      tmp    =   dqdx(2)*Dxh;
      btr(1) =      q(2)+tmp;
      rhs(1) =  factor*dhdx(2) - pleft*(q(2)-tmp);

      for i=3:N-1
        atr(i-1) = -2.0*q(i);
        tmp    =   dqdx(i)*Dxh;
        btr(i-1) =      q(i)+tmp;
        ctr(i-1) =      q(i)-tmp;
        rhs(i-1) =  factor*dhdx(i);
      end

      atr(N-1) = -2.0*q(N);
      ctr(N-1) =      q(N)-tmp;
      rhs(N-1) =  factor*dhdx(N)-prite*(q(N)+tmp);

%---
% solve N-1 equations by the Thomas algorithm
%---

sln = thomas  ...
...
   (N-1 ...
   ,atr,btr,ctr ...
   ,rhs ...
   );

pi1(1)   = pleft;     % end node
pi1(N+1) = prite;      %end node

for i=1:N-1
  pi1(i+1) = sln(i);
end

%---
% compute a homogeneous solution with dcdt=1.0
% and homogeneous boundary conditions
%---

cns = 12.0*Dxs;

for i=1:N-1
  rhs(i) = cns;
end

%---
% solve N-1 equations with the new rhs
%---

 sln = thomas  ...
...
   (N-1 ...
   ,atr,btr,ctr ...
   ,rhs ...
   );

   pi2(1)   = 0.0;   % end node
   pi2(N+1) = 0.0;    % end node

   for i=1:N-1
      pi2(i+1) = sln(i);
   end

%---
% Done
%---

return
