%---
close all
clear all
%----

%=========================================
% Flow of two viscous layers in a horizontal or inclined
% channel confined between two parallel plane walls.
%
% Shear-, pressure-, and gravity-driven flow.
%
% The program uses an explicit finite-difference method
% with uniform grid spacing.
%
% Legend:
% ------
%
% a:    walls are located at y = +-a
% beta: inclination angle
% RL:   period
%
% yIunp:  unperturbed position of the interface
% aI0:    initial amplitude of the interface
% asrf0:  initial amplitude of the surfactant
% hr:      layer thickness ratio: h2/h1
%
% vs1,vs2       layer viscosities
% rho1,rho2     layer densities
% vsr:          lamda (viscosity ratio)
%
% pdrop:  pressure drop over one period (positive)
%
% method: 1 for central differences
%         2 for backward differences
%
% DyDx1:  first derivative of y(x)
% DyDx2:  second derivative
% DyDx3:  third derivative
%================================================

a = 0.2;
beta = 0.125*pi;  % inclination angle
L = 1.0;        % wave length
yIunp = 0.0; % unperturbed position
aI0 = 0.1;
rho1 = 1.0;
rho2 = 1.0;
vs1 = 1.0;
vs2 = 0.4;
gamma = 0.4; % surface tension
V1 =-0.2;
V2 = 1.0;
pdrop = 0.3;
gac = 1.0;
NSG = 32;

Dt=0.001;   % time step
nstep = 1; % number of steps
nstep = 128*128; % number of steps

method = 1;

%----
% prepare
%----

sn0 = sin(beta);
cs0 = cos(beta);
ROT = [cs0,sn0;-sn0,cs0];
gx =  gac*sn0;
gy = -gac*cs0;
as = a*a;
vsr = vs2/vs1;  % viscosity ratio
Drho  = rho1-rho2;  % density difference
dnr = rho2/rho1;  % density ratio 
wn = 2.0*pi/L;   % wave number
Dx = L/NSG;
Dx2  = 2.0*Dx;
Dx23 = 2.0*Dx*Dx*Dx;
Dxs  = Dx*Dx;

figure(1)
hold on

%----
% plot walls
%----

xx = ROT*[0 -a]';
xplot0(1)=xx(1)/L;
yplot0(1)=xx(2)/L;
xx = ROT*[L -a]';
xplot0(2)=xx(1)/L;
yplot0(2)=xx(2)/L;
xx = ROT*[L  a]';
xplot0(3)=xx(1)/L;
yplot0(3)=xx(2)/L;
xx = ROT*[0  a]';
xplot0(4)=xx(1)/L;
yplot0(4)=xx(2)/L;
xplot0(5)=xplot0(1);
yplot0(5)=yplot0(1);

%----------------
% initial profile
%----------------

for i=1:NSG+4
 x(i) = (i-1.0)*Dx;
 arg  = wn*x(i);
 y(i) = yIunp + aI0*cos(arg);
 srtn(i) = gamma;
end

%----------------
% time stepping
%----------------

for step=1:nstep

  for i=3:NSG+2
    ib = i-2; ia=i-1; i1 = i+1; i2 = i+2;
    DyDx1(i) = (y(i1)-y(ia))/Dx2;
    DyDx2(i) = (y(i1)-2.0*y(i)+y(ia))/Dxs;
    DyDx3(i) = (y(i2)-2.0*y(i1)+2.0*y(ia)-y(ib))/Dx23;
    h1(i) = a+y(i);
    h2(i) = a-y(i);
    hr(i)=h2(i)/h1(i);
  end

  % wrap

  DyDx1(1) = DyDx1(NSG+1);
  DyDx2(1) = DyDx2(NSG+1);
  DyDx3(1) = DyDx3(NSG+1);
     h1(1) =    h1(NSG+1);
     h2(1) =    h2(NSG+1);
     hr(1) =    hr(NSG+1);

  DyDx1(2) = DyDx1(NSG+2);
  DyDx2(2) = DyDx2(NSG+2);
  DyDx3(2) = DyDx3(NSG+2);
     h1(2) =    h1(NSG+2);
     h2(2) =    h2(NSG+2);
     hr(2) =    hr(NSG+2);

%---------------
% compute dp1/dx 
%---------------

 for i=1:NSG
  tmp = 1.0/(vsr+hr(i));
  Den = vsr*h1(i)^3 + h2(i)^3 + 12.0*vsr*tmp*as*h2(i);
  t1 = - Drho*gy*(h2(i)+6.0*vsr*tmp*a)*h2(i)^2*DyDx1(i);
  t2 = - srtn(i)*(h2(i)+6.0*vsr*tmp*a)*h2(i)^2*DyDx3(i);
  t3 =  rho1*gx* (vsr*h1(i)^3 + dnr*h2(i)^3 ...
       +6.0*vsr*tmp*(1.0+dnr*hr(i))*a*h1(i)*h2(i));
  t4 = 6.0*vs2*(V1*h1(i)+V2*h2(i))+12.0*vs2*a*tmp*(vsr*V2+hr(i)*V1);
  dp1dx(i) = (t1+t2+t3+t4)/Den;
  store(i) = 1.0/Den;
 end

%--------------------------
% compute the function f(t)
%--------------------------

  sum1 = 0.0;
  sum2 = 0.0;
  for i=1:NSG
    sum1 = sum1 + dp1dx(i);
    sum2 = sum2 + store(i);
   end
  sum1 = sum1*Dx;
  sum2 = sum2*Dx;

  f = (pdrop-sum1)/sum2;

%-----------------------------------
% complete the computation of dp1/dx
%-----------------------------------

  for i=1:NSG
    dp1dx(i) = dp1dx(i) + f*store(i);
  end

  dp1dx(NSG+1) = dp1dx(1);
  dp1dx(NSG+2) = dp1dx(2);

%------------
% compute the interfacial velocity u_I
%------------

 for i=1:NSG
   ri1 = 1.0+hr(i);
   uI(i) = a*h2(i)/vs1 *(-dp1dx(i) ...
           +rho1*gx*(1.0+dnr*hr(i))/ri1 ) ...
           +0.5*h2(i)^2/vs1*(-Drho*gy*DyDx1(i)-srtn(i)*DyDx3(i)) ...
           + hr(i)*V1+vsr*V2;
   uI(i) = uI(i)/(vsr+hr(i));
 end

 uI(NSG+1) = uI(1);
 uI(NSG+2) = uI(2);

%------------------------------------
% compute dQ1/dx
%
% then set dy/dt = - dQ1/dx
%------------------------------------

 pr1 = 12.0*Dx2*vs1;
 pr2 =  2.0*Dx2;
 pr3 = 12.0*Dx*vs1;
 pr4 =  2.0*Dx;

 for i=2:NSG+1

 ia = i-1;
 i1 = i+1;

%---
    if(method==1) % central differences
%---

       dq1dx = ( h1(i1)^3 *(-dp1dx(i1)+rho1*gx) ...
                -h1(ia)^3 *(-dp1dx(ia)+rho1*gx))/pr1 ...
           + ( (uI(i1)+V1)*h1(i1) ...
              -(uI(ia)+V1)*h1(ia))/pr2;
%---
     else  % backward differences
%---

       dq1dx = ( h1(i)^3 *(-dp1dx(i) +rho1*gx) ...
                -h1(ia)^3 *(-dp1dx(ia)+rho1*gx))/pr3 ...
           + ( (uI(i) +V1)*h1(i) ...
              -(uI(ia)+V1)*h1(ia))/pr4;
%---
   end
%---

   DyDt(i) =-dq1dx;

  end

  for i=2:NSG+1
     y(i) = y(i)+Dt*DyDt(i);
  end
  y(1)=y(NSG+1);
  y(NSG+2)=y(2);
  y(NSG+3)=y(3);
  y(NSG+4)=y(4);

%---
% plotting
%---

 for k=1:NSG+1
   xx = ROT*[x(k) y(k)]';
   xplot1(k)=xx(1)/L;
   yplot1(k)=xx(2)/L;
 end

 if(step==1)
   Handle0 = plot(xplot0,yplot0,'k');
   hold on
   Handle1 = plot(xplot1,yplot1,'r.-');
   xlabel('x/L','fontsize',15)
   ylabel('y/L','fontsize',15)
   set(gca,'fontsize',15)
   axis equal
%   axis([0 1.2*cs0 -sn0 sn0])
   box
 end
 set(Handle0,'XData',xplot0,'YData',yplot0)
 set(Handle1,'XData',xplot1,'YData',yplot1)
 drawnow

%=====
 end   % of time stepping
%=====

