      program airf_2d_csdp

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c------------------------------------------------------
c
c  Potential flow around a two-dimensional airfoil
c  with the Kutta condition satisfied at the trailing edge.
c 
c  The airfoil contour is discretized into
c  point source and point-source-dipole panels.
c
c  The panels are straight segments, and the
c  source and dipole densities are constant over the panels.
c
c  csdp stands for: constant source - dipole panels.
c
c  This formulation is based on the Dirichlet
c  boundary condition for the potential
c  on the inner side of the airfoil.
c
c  This program computes the strength of the dipole panels,
c  the distribution of the tangential velocity,
c  the pressure distribution around the airfoil,
c  and then draws streamline patterns.
c
c
c  SYMBOLS:
c  --------
c
c  a:   influence matrix
c
c  ep:	coordinates of marker points
c  pt:	coordinates of panel end-points 
c  th:  panel inclination angle
c  dl:  panel length
c
c  co:  coordinates of collocation points
c  phi: harmonic potential
c
c  udn: U.dot.normal_vector
c
c  cp:  pressure coefficient
c  cl:	lift coefficient
c
c  velt: tangential velocity at end-points
c
c  Umag:  magnitude of incident velocity
c  alpha: angle of attack
c
c  Important:
c  ---------
c
c  Points must be distributed in the clockckwise sense
c  starting from the trailing end and going around
c  the bottom first.
c
c---------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension ep(500,2),pt1(500,2),pt2(500,2)
      Dimension co(500,2)

      Dimension  th(500),dl(500)
      Dimension tnx(500),tny(500)
      Dimension vnx(500),vny(500)
      Dimension udn(500)
      Dimension phi(500),velt(500),cp(500)

      Dimension a(500,500),rhs(500),xs(500)

c----------
c constants
c----------

      pi  = 3.14159 265358
      pi2 = 2.0*pi

      null = 0

c--------------------------
c Select the airfoils shape
c--------------------------

 94   Continue

      write (6,*)
      write (6,*) ' Select the airfoil shape'
      write (6,*)
      write (6,*) ' Enter 0 to quit'
      write (6,*) '       1 to read shape from file:'
      write (6,*) '         airf_2d_contour.dat'
      write (6,*) '       2 for a NACA 4-digit section'
      write (6,*) '       3 for a Joukowski airfoil'
      write (6,*) '       4 for a circle'
      write (6,*) '       5 for the NACA 23012 section'
      write (6,*) ' ----------------------------------'

      read (5,*) Ishape

      If(Ishape.eq.0) Go to 99

c----------------------
c Simulation parameters
c----------------------

      write (6,*)
      write (6,*) ' Enter:'
      write (6,*) '      0 to quit'
      write (6,*) '      1 to read them from '
      write (6,*) '        input file: airf_2d_v_**s.dat'
      write (6,*) '      2 to type in parameters'
      write (6,*)
      write (6,*) '  Warning: If you type 2 will spend a lot'
      write (6,*) '           of time typing'
      write (6,*) ' ---------------------------------------'
      read  (5,*)   Ienrd

      If(Ienrd.eq.0) Go to 99

c------------------------
      If(Ienrd.eq.2) then
c------------------------

      write (6,*) 
      write (6,*) ' Uniform flow past a two-dimensional airfoil'
      write (6,*) '         computed by the combined point source'
      write (6,*) '         source dipole representation'
      write (6,*) 
      write (6,*) 'Enter the magnitude of the incident velocity'
      write (6,*) '--------------------------------------------'
      read  (5,*) Umag

      write (6,*) 
      write (6,*) 'Enter angle of attack (in degrees)'
      write (6,*) '----------------------------------'
      read  (5,*) alpha

c----
c contour specified from input file: airf_2d_contour.dat
c----

      If(Ishape.eq.1) then

       chord = 1.0           ! for reduction only

c----
c NACA 4-digit section
c----

      Else If(Ishape.eq.2) then

      write (6,*) 
      write (6,*) 'Will  generate a NACA 4-digit section'
 96   write (6,*) 
      write (6,*) 'Enter number of panels; must be even'
      write (6,*) '------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then
       write (6,*) 
       write (6,*) " Sorry; the number of panels must be even"
       write (6,*) " Try again"
       Go to 96
      End If

      write (6,*) 
      write (6,*) 'Enter the maximum section thickness'
      write (6,*) '                      for unit cord'
      write (6,*) '-----------------------------------'
      read  (5,*) thick

      write (6,*) 
      write (6,*) 'Enter the maximum section camber'
      write (6,*) '                   for unit cord'
      write (6,*) '--------------------------------'
      read  (5,*) cam

      write (6,*) 
      write (6,*) 'Enter the chordwise location of max camber'
      write (6,*) '                             for unit cord'
      write (6,*) 
      write (6,*) 'Must be between 0 and 1.0'
      write (6,*) '------------------------------------------'
      read  (5,*) xcam

      write (6,*) 
      write (6,*) 'Enter the chord length'
      write (6,*) '----------------------'
      read  (5,*) chord

c-------------------
c  Joukowski airfoil
c-------------------

      Else If(Ishape.eq.3) then

      write (6,*)
      write (6,*) 'Will generate a Joukowski airfoil'
      write (6,*) 'by conformal mapping'
  97  write (6,*)
      write (6,*) 'Enter the number of panels; must be even'
      write (6,*) '----------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then

       write (6,*)
       write (6,*) " Sorry; the number of panels must be even"
       write (6,*) " Try again"
       Go to 97

      End If

      write (6,*)
      write (6,*) 'Please enter the unit circle center xc,yc'
      write (6,*)
      write (6,*) '0.2 and 0.2 are good values for testing'
      write (6,*) '----------------------------------------'
      read  (5,*) xc,yc

      write (6,*)
      write (6,*) 'Please enter the chord length'
      write (6,*)
      write (6,*) '1.0 is a good value for testing'
      write (6,*) '-------------------------------'
      read  (5,*) chord

c---------
c a circle
c---------

      Else If(Ishape.eq.4) then

      write (6,*)
      write (6,*) 'Please enter the circle radius'
      write (6,*) '------------------------------'
      read  (5,*) radius

 89   write (6,*)
      write (6,*) 'Enter number of panels; must be even'
      write (6,*) '------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then

       write (6,*)
       write (6,*) " Sorry; the number of panels must be even"
       write (6,*) " Try again"
       Go to 89

      End If

c----
c NACA 23012 section
c----

      Else If(Ishape.eq.5) then

      write (6,*)
      write (6,*) 'Will generate the NACA 23012 section'
  90  write (6,*)
      write (6,*) 'Enter the number of panels; must be even'
      write (6,*) '----------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then

       write (6,*)
       write (6,*) " Sorry; the number of panels must be even"
       write (6,*) " Try again"
       Go to 90

      End If

      write (6,*)
      write (6,*) 'Enter the maximum section thickness'
      write (6,*) '                      for unit cord'
      write (6,*) '-----------------------------------'
      read  (5,*) thick

      write (6,*)
      write (6,*) 'Enter the chord length'
      write (6,*) '----------------------'
      read  (5,*) chord

c-----------
      End If   ! End of options
c-----------

c-----------------------------
      Else If(Ienrd.eq.1) then
c-----------------------------

      If(Ishape.eq.1) then

        open (3,file='airf_2d_gnc.dat',status='old')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) chord

      Else If(Ishape.eq.2) then

        open (3,file='airf_2d_naca.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) thick
        read (3,*) cam
        read (3,*) xcam
        read (3,*) chord

      Else If(Ishape.eq.3) then

        open (3,file='airf_2d_jouk.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) xc,yc
        read (3,*) chord

      Else If(Ishape.eq.4) then

        open (3,file='airf_2d_circle.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) radius

      Else If(Ishape.eq.5) then

        open (3,file='airf_2d_naca_23012.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) thick
        read (3,*) chord

      Else

        write (6,*)
        write (6,*)  " Sorry, this selection is not valid"
        write (6,*)

      End If

c-----------
      End If        ! End of reading parameters
c-----------

c--------------------------------
c Read or generate the end-points
c--------------------------------

c-------------------------
      If(Ishape.eq.1) then
c-------------------------

      open (4,file='airf_2d_contour.dat')

       read (4,*) Npl1        ! Npl1 is the number of end-point
                              ! chord used for normalization
       Do i=1,Npl1
        read (4,*) idle,ep(i,1),ep(i,2)  ! x and y coordinates
       End Do                             ! of panel end points

       Npl = Npl1-1    ! number of panels

      close (4)

c------------------------------
      Else If(Ishape.eq.2) then             ! NACA 4-digit
c------------------------------

      Npla  = Npl-1
      Npl1  = Npl+1
      Nplh  = Npl/2
      Nplh1 = Nplh+1

      dpsi = pi2/(Npl1-1.0)

      ep(1,1) = chord
      ep(1,2) = 0.0

      Do i=2,Npl

         psi = (i-1.0)*dpsi
         x   = 0.5*(1.0+cos(psi))

c---
c camber line
c---

         If(x.lt.xcam) then
          yc = cam*(2.0*xcam*x-x**2)/xcam**2
         Else
          yc = cam/(1.0-xcam)**2*((1.0-2.0*xcam)+2.0*xcam*x-x**2)
         End If

c---
c thickness
c---

         yt = 5.0*thick*
     +                  (0.2969*sqrt(x)
     +                  -0.1260*x
     +                  -0.3516*x**2
     +                  +0.2843*x**3
     +                  -0.1036*x**4
     +                  )

         If(i.le.Nplh1) then
          y = yc-yt
         Else
          y = yc+yt
         End If

         ep(i,1) = x*chord
         ep(i,2) = y*chord

      End Do

      ep(Npl1,1) = chord    ! trailing edge
      ep(Npl1,2) = 0.0      ! trailing edge

c------------------------------
      Else If(Ishape.eq.3) then     !   Joukowski airfoil
c------------------------------

      Npl1 = Npl+1

      r  = sqrt((1.0-xc)**2+yc**2)

      psi  = pi2-atan(yc/(1.0-xc))
      dpsi = pi2/(Npl1-1.0)

      Do i=1,Npl1

        If(i.eq.1.or.i.eq.Npl1) then
          ep(i,1) = 0.5*chord
          ep(i,2) = 0.0
        Else
          psi     = psi+dpsi
          cs      = cos(psi)
          sn      = sin(psi)
          denom   = (xc+r*cs)**2+(yc+r*sn)**2
          ep(i,1) =((xc+r*cs)*(1.0+1.0/denom))*chord/4.0
          ep(i,2) =((yc+r*sn)*(1.0-1.0/denom))*chord/4.0
        End If

      End Do

c------------------------------
      Else If(Ishape.eq.4) then       ! a circle
c------------------------------

      chord = 2.0*radius

      Npl1   = Npl+1
      dtheta = pi2/(Npl1-1.0)
      theta  = pi2

      Do i=1,Npl1
       ep(i,1) = radius*cos(theta)
       ep(i,2) = radius*sin(theta)
       theta   = theta-dtheta
      End Do

c------------------------------
      Else If(Ishape.eq.5) then        ! NACA
c------------------------------

      Npl1  = Npl+1
      Nplh  = Npl/2
      Nplh1 = Nplh+1

      dpsi = pi2/(Npl1-1.0)

      ep(1,1) = chord
      ep(1,2) = 0.0

      Do i=2,Npl

         psi = (i-1.0)*dpsi
         x   = 0.5*(1.0+cos(psi))
c---
c camber line
c---

         If(x.lt.0.2025) then
          yc = 2.6595*x*(0.1147+x*(-0.6075+x))
         Else
          yc = 0.02208*(1.0-x)
         End If

c---
c thickness
c---

         yt = 5.0*thick*(0.2969*sqrt(x)
     +                  -0.1260*x
     +                  -0.3516*x**2
     +                  +0.2843*x**3
     +                  -0.1036*x**4
     +                  )

         If(i.le.Nplh1) then
          y = yc-yt
         Else
          y = yc+yt
         End If

         ep(i,1) = x*chord
         ep(i,2) = y*chord

      End Do

      ep(Npl1,1) = chord    ! trailing edge
      ep(Npl1,2) = 0.0      ! trailing edge

c-----------
      End If  ! End of options
c-----------

c------------------
c open output files
c------------------

      open (2,file="airf_2d.xy",status="unknown")

c--------------------------------
c  End of panel generation module
c
c  Print panel end-points
c--------------------------------

      write (2,*) Npl1," panel end-nodes"
      write (6,*)
      write (6,*) Npl1," panel end-nodes"

      Do i=1,Npl1
        write (2,100) i,ep(i,1),ep(i,2)
        write (6,100) i,ep(i,1),ep(i,2)
      End Do

c---
c prepare
c---

      al  = alpha*pi/180.0
      cal = cos(al)
      sal = sin(al)

      Ux = Umag*cal
      Uy = Umag*sal

c---
c Set coordinates of the panel end-points
c---

      Do i=1,Npl
       pt1(i,1) = ep(i,1)
       pt1(i,2) = ep(i,2)
       pt2(i,1) = ep(i+1,1)
       pt2(i,2) = ep(i+1,2)
      End Do

c---------------------------------------
c compute: 
c
c         panel inclination angle th
c         (between -pi and pi) 
c         panel lengths 
c         tangential and normal unit vectors
c---------------------------------------

      Do i=1,Npl
       dx     = pt2(i,1)-pt1(i,1)
       dy     = pt2(i,2)-pt1(i,2)
       dl(i)  = sqrt( (pt2(i,1)-pt1(i,1))**2       ! panel length
     +             + (pt2(i,2)-pt1(i,2))**2)
       th(i)  = atan2(dy,dx)
       tnx(i) =  cos(th(i))     ! tangential vector
       tny(i) =  sin(th(i))
       vnx(i) = -tny(i)         ! normal vector points in flow
       vny(i) =  tnx(i)
       udn(i) = Ux*vnx(i)+Uy*vny(i)
      End Do

c---
c colocation points are panel mid-points
c---

      Do i=1,Npl
       co(i,1) = 0.5*(pt2(i,1)+pt1(i,1))
       co(i,2) = 0.5*(pt2(i,2)+pt1(i,2))
      End Do

c-----------------------------
c initialize the linear system
c-----------------------------

      Do i=1,Npl1
       rhs(i) = 0.0
       Do j=1,Npl1
        a(i,j) = 0.0
       End Do
      End Do

c-------------------------------------
c compute influence coefficients
c and right-hand side of linear system
c-------------------------------------

      Do i=1,Npl       ! LOOP OVER COLOCATION POINTS
                       ! ENDS AT THE RABBIT

       sptl = 0.0      ! source potential

c---
c Compute the influence of jth panel
c on ith coloation point
c---

       Do j=1,Npl     ! LOOP OVER PANELS
                      ! ENDS AT THE DOG
	 
c---
c Compute the local panel coordinates
c
c The first panel point becomes the shifted origin;
c the panel is horizontal
c---

        xt = co(i,1)-pt1(j,1)       ! shift colocation point
        yt = co(i,2)-pt1(j,2)

        x = xt*tnx(j) + yt*tny(j)   ! and rotate
        y = xt*vnx(j) + yt*vny(j)

        x1 = 0.0
        y1 = 0.0

        x2t = pt2(j,1)-pt1(j,1)        ! shift second-end point
        y2t = pt2(j,2)-pt1(j,2)

        x2  = x2t*tnx(j) + y2t*tny(j)  ! and rotate
        y2  = 0.0

c----------------------------------
c Compute radial distances:  r1, r2
c         subtended angles: th1, th2
c----------------------------------

        r1 = sqrt( x**2    +y**2)
        r2 = sqrt((x-x2)**2+y**2)

        th1 = atan2(y,x)
        th2 = atan2(y,x-x2)

        If(i.eq.j) then
          a(i,j) = 0.5
          sptl = sptl + udn(j)*x*log(r1)/pi
        Else
          dth = th2-th1
          a(i,j) = -dth/pi2
          sptl = sptl + udn(j)
     +                *(-(x-x2)*log(r2)+x*log(r1)+y*dth)/pi2
        End If

       End Do           ! End of loop over planels  -  DOG

c----
c include a point vortex with unknown
c strength placed at the trailing edge
c----

       xw  = co(i,1)-pt2(Npl,1)
       yw  = co(i,2)-pt2(Npl,2)
       thw = -atan(-yw/xw)

       a(i,Npl1) = thw/pi2

c---
c Compute the rhs of the linear system
c---

       rhs(i) = sptl

      End Do          ! End of loop over collocation points -  RABBIT

c--------------------------------
c Add a Kutta condition
c expressed by the Npl+1 equation
c--------------------------------

      a(Npl1,1)    = 1.0
      a(Npl1,Npl)  =-1.0
      a(Npl1,Npl1) = 1.0

c---
c solve the linear system
c---

      Isym_g = 0      ! system is not symmetric
      Iwlpvt = 1      ! pivoting enabled

      call gauss 
     +           (Npl1
     +           ,a
     +           ,rhs
     +           ,xs
     +           ,Isym_g
     +           ,Iwlpvt
     +           ,Det
     +           ,Istop
     +           )

c     Do i=1,Npl
c      write (6,888) (a(i,j),j=1,Npl),rhs(i)
c     End Do

c---
c distribute the solution
c and add the potential of the incident flow
c---

      Do i=1,Npl
       phi(i) = - xs(i) + co(i,1)*Ux + co(i,2)*Uy
      End Do

      rkappa = xs(Npl1)  ! strength of the trailing edge
                         ! point vortex

c---
c printing session
c---

      write (6,*)
      write (6,*) "Dipole-sheet strength"
      write (2,*) Npl," Dipole-sheet strength"

      Do j=1,Npl
       write (6,134) j,co(j,1),phi(j)
       write (2,134) j,co(j,1),phi(j)
      End Do

      write (6,*)
      write (6,*) "Strength of the trailing-edge point vortex:"
      write (6,*) rkappa

      write (2,*)
      write (2,*) "Strength of the trailing-edge point vortex:"
      write (2,*) rkappa

c------------------------------------------------------------
c  post-processing
c
c  compute:
c           (a) the tangential velocity at end-points 
c               by numerical differentiation
c           (b) the panel pressure coefficients
c
c  phi is the potential on the outside of the airfoil
c
c------------------------------------------------------------

      Do i=2,Npl
       velt(i)  = 2.0*(phi(i)-phi(i-1))/(dl(i)+dl(i-1))
         cp(i) = 1.0-velt(i)**2/Umag**2
      End Do

      cp(1)    = 0.5*(cp(2)+cp(Npl))     ! assigned values
      cp(Npl1) = cp(1)                   ! assigned values

c---
c printing session
c---

      write (2,*) Npl-1," pressure coefficient, tangential velocity"

      write (6,*) " pressure coefficient, tangential velocity"
      write (6,*)

      Do i=2,Npl
       write (2,134) i,pt1(i,1),cp(i),velt(i)
       write (6,134) i,pt1(i,1),cp(i),velt(i)
      End Do

c---------------------------------------------------
c  compute lift and drag coefficients by integration
c  of surface pressures using the mid-point rule
c
c  Pressure coefficient over each panel is the average
c  of the values at the panel end-points.
c----------------------------------------------------

      tlift = 0.0
      tdrag = 0.0

      Do i=1,Npl

       If(i.eq.1) then
        tcp = cp(2)
       Else If(i.eq.Npl) then
        tcp = cp(Npl)
       Else
        tcp = 0.5*(cp(i)+cp(i+1))
       End If

       tlift = tlift - tcp*dl(i)*vny(i)
       tdrag = tdrag - tcp*dl(i)*vnx(i)

      End Do

c---
c normalize by chord
c---

      tlift = tlift/chord
      tdrag = tdrag/chord


c---
c Put integrated lift/drag into wind axes
c---

      wd =  tdrag*cal+tlift*sal
      wl = -tdrag*sal+tlift*cal

c---
c printing session
c---

      write(6,*) ' '
      write(6,*) 'lift and drag coefficients by integration'
      write(6,*) '(wind axes)'
      write(6,*) 'lift=',wl
      write(6,*) 'drag=',wd
      write(6,*) 

c---
c printing session
c---

      write (6,*)
      write (6,*) 'lift coefficient in wind axes'
      write (6,*) 'computed in terms of the circulation:'
      write (6,*) 
      write (6,*) -2.0*Umag*rkappa/chord

c---
c  wrap up
c---

 99   Continue

      write (2,*) null
      close (2)

 190  Format(a)
 100  Format (1x,i3,10(1x,f12.6))
 134  Format (1x,I3,4(1x,f10.5))
 120  Format (90(1x,f10.5))

 888  Format (30(1x,f7.4))

      Stop
      End
