%---
 clear all
 close all
%---

%===========================
% linear vortex panel method
% for flow past an airfoil
%===========================

Npl = 64;    % number of panels
chord = 1.0;   % chord
thick = 0.1;   % thickness
Umag = 1.0;  % incident velocity
alpha = 4.0;  % angle of atack in degrees

%----
% generate the NACA 23012 airfoil contour
% ep are the panel end-points
%----

ep = naca(chord,thick,Npl);

%---
% coordinates of the panel end-points
%---

  for i=1:Npl
    pt1(i,1) = ep(i,1);
    pt1(i,2) = ep(i,2);
    pt2(i,1) = ep(i+1,1);
    pt2(i,2) = ep(i+1,2);
  end

%---------------------------------------
% compute:
%
%  1. panel inclination angle th(j)
%     (between -pi and pi)
%  2. panel lengths
%  3. tangential and normal unit vectors
%---------------------------------------

  for i=1:Npl
   dx = pt2(i,1)-pt1(i,1);
   dy = pt2(i,2)-pt1(i,2);
   dl(i)  = sqrt( (pt2(i,1)-pt1(i,1))^2  ...    % panel length
         + (pt2(i,2)-pt1(i,2))^2);
   th(i)  =  atan2(dy,dx);
   tnx(i) =  cos(th(i));    % tangential vector
   tny(i) =  sin(th(i));
   vnx(i) = -tny(i);    % normal vector points into flow
   vny(i) =  tnx(i);
  end

%---
% colocation points are panel mid-points
%---

  for i=1:Npl
    co(i,1) = 0.5*(pt1(i,1)+pt2(i,1));
    co(i,2) = 0.5*(pt1(i,2)+pt2(i,2));
  end

%-------------------------------------
% compute the influence coefficients
% and right-hand side of linear system
%-------------------------------------

   for i=1:Npl     % LOOP OVER COLOCATION POINTS 
                   % ENDS AT THE RABBIT
%---
% Compute influence of jth panel
% on ith collocation point
%---

    for j=1:Npl     % LOOP OVER PANELS
                    % ENDS AT THE DOG
        
%---
% Compute the local panel coordinates (x, y).
% The first panel point is located at the shifted origin;
% the panel becomes horizontal
%---

  xt = co(i,1)-pt1(j,1);  % shift colocation point
  yt = co(i,2)-pt1(j,2);
  x =  xt*tnx(j)+yt*tny(j);   % and rotate
  y = -xt*tny(j)+yt*tnx(j);
  x1 = 0.0;
  y1 = 0.0;
  x2t = pt2(j,1)-pt1(j,1);  % shift second-end point
  y2t = pt2(j,2)-pt1(j,2);
  x2  = x2t*tnx(j) + y2t*tny(j);  % and rotate
  y2  = 0.0;

%---
% Compute radial distances:  r1, r2
%         subtended angles: th1, th2
%---

   r1 = sqrt( x*x+y*y );
   r2 = sqrt((x-x2)*(x-x2)+y*y);
   th1 = atan2(y,x);
   th2 = atan2(y,x-x2);

%--------------------------------------------
%  gamma1, gamma2 are the vortex sheet strengths 
%  at the first and second panel points.
%
%  Compute influence coefficients associated with
%  the velocity components (ux, uy) 
%  corresponding to gamma1 and gamma, so that:
%
%  ux = ax1*gamma1 + ax2*gamma2
%  uy = ay1*gamma1 + ay2*gamma2
%
%  These velocities are in 
%  the jth-panel reference frame.
%--------------------------------------------

   if(i==j)  % self-induced velocity
    ax1 = 0.5*(x/x2-1.0);
    ay1 = 1.0/(2*pi);
    ax2 =-0.5*x/x2;
    ay2 =-1.0/(2*pi);
   else
    dth  = th2-th1;
    rrt  = r2/r1;
    rrtl = log(rrt);
    fcc  = 1.0/(2*pi*x2);
    ax1 = fcc*(     y*rrtl + (x-x2)*dth      );
    ay1 = fcc*((x-x2)*rrtl -       y*dth + x2);
    ax2 = -fcc*(y*rrtl + x*dth     );
    ay2 = -fcc*(x*rrtl - y*dth + x2);
   end

%---
% transform the influence coefficient into the 
% global reference frame by rotation
%---
        
  ux1 = ax1*tnx(j) - ay1*tny(j);
  uy1 = ax1*tny(j) + ay1*tnx(j);
  ux2 = ax2*tnx(j) - ay2*tny(j);
  uy2 = ax2*tny(j) + ay2*tnx(j);

%-----------------------------------------------
% Compute the coefficients of gamma in the 
% master influence matrix.
%
% These are the velocity influence coefficients
% projected onto the normal vector of ith panel
%------------------------------------------------

 if(j==1)
  a(i,1)= ux1*vnx(i) + uy1*vny(i);
  holda = ux2*vnx(i) + uy2*vny(i);   % hold for the next panel
 elseif(j==Npl)
  a(i,Npl) = ux1*vnx(i) + uy1*vny(i) + holda;
  a(i,Npl+1) = ux2*vnx(i) + uy2*vny(i);
 else
  a(i,j)= ux1*vnx(i) + uy1*vny(i) + holda;
  holda = ux2*vnx(i) + uy2*vny(i);     % hold for the next panel
 end

%-------------------------------------------------------
%  b are tangential velocity influence coefficients
%
%  These are the velocity influence coefficients
%  projected onto the tangential vector of the ith panel
%-------------------------------------------------------

  if(j==1)
    b(i,1)= ux1*tnx(i) + uy1*tny(i);
    holdb = ux2*tnx(i) + uy2*tny(i);  % hold for the next panel
  elseif(j==Npl)
    b(i,Npl)  = ux1*tnx(i) + uy1*tny(i) + holdb;
    b(i,Npl+1) = ux2*tnx(i) + uy2*tny(i);
  else
    b(i,j)= ux1*tnx(i) + uy1*tny(i) + holdb;
    holdb = ux2*tnx(i) + uy2*tny(i);   % hold for the next panel
  end

  end   % End of loop over panels  -  DOG

  end   % End of loop over collocation points -  RABBIT

%--------------------------------
% Add the Kutta condition
% expressed by the Npl+1 equation:
%
% The strength of the sheet at the first point
% of the first panel is equal and opposite
% to the strength of the sheet at the last point
% of the last panel,
% so that the mean value vanishes
%--------------------------------

 a(Npl+1,1)    = 1.0;
 a(Npl+1,Npl+1) = 1.0;

%--------------------
% set right-hand side
%--------------------

 al  = alpha*pi/180.0;
 cal = cos(al);
 sal = sin(al);
 Ux = Umag*cal;
 Uy = Umag*sal;

 for i=1:Npl
  rhs(i) = -Ux*vnx(i)-Uy*vny(i);
 end
 rhs(Npl+1)=0.0;

%-----------------------------
% solve the linear system
%-----------------------------

 gamma = rhs/a';

%-----------------------------
% compute the circulation
%-----------------------------

  circul  = 0.0;  %  circulation
  circulg = 0.0;  %  circulation in terms of gamma

  for i=1:Npl
    velt = Ux*tnx(i)+Uy*tny(i);  % tangential velocity of inc flow
    for j=1:Npl+1
     velt = velt + b(i,j)*gamma(j);
    end
    circul  = circul - velt*dl(i);    % circulation
    circulg = circulg+0.5*(gamma(i)+gamma(i+1))*dl(i);  % circulation
    cp(i)  = 1.0-velt*velt/(Umag*Umag);  % pressure coefficient
  end
  cp(Npl+1)=cp(1);

%---
% plotting
%---

figure(1)
hold on
axis equal
patch(ep(:,1),ep(:,2),'r')
plot(ep(:,1),ep(:,2),'k.-')
plot(ep(:,1),cp,'k--')
plot(ep(:,1),gamma,'k.-')
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
set(gca,'fontsize',15)
box on
