      program cvt_2d

c=======================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=======================================

c-----------------------------------------------
c Potential flow in a rectangular cavity
c with impermeable left, bottom, and right walls.
c The distribution of the harmonic potential is
c specified at the top wall.
c
c This program performs the following functions:
c
c (a) It generates a uniform Cartesian grid
c     in a rectangular domain in the xy plane
c
c (b) It sets up the linear system for solving
c     Laplace's equation
c
c (c) It solves the linear system by Gauss elimination
c
c (d) It generates velocity vectors at grid points
c
c  SYMBOLS:
c  --------
c
c  ax, bx:  cavity is confined in ax<x<bx
c  ay, by:  cavity is confined in ay<y<by
c
c  Nx :  number of x intervals
c  Ny :  number of y intervals
c
c  Dx :  length of x interval
c  Dy :  length of y interval
c
c  glx(i) :  x-position of vertical ith grid line
c  gly(i) :  y-position of horizontal ith grid line
c
c  [gp_x(i,j), gp_y(i,j)] : coordinates of (i,j) grid point
c  [gu_x(i,j), gu_y(i,j)] : velocity at (i,j) grid point
c
c  Amat: coefficient matrix of the linear system
c  rhs:  right-hand side of the linear system
c  sln: unknown vector of the linear system (vector w)
c
c  ptl_top: harmonic potential at the top
c  ptl: harmonic potential at the grid points
c
c-----------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension glx(129), gly(129)

      Dimension gp_x(129,129),gp_y (129,129)
      Dimension gu_x(129,129),gu_y(129,129)
      Dimension  ptl(129,129)

      Dimension ptl_top(124)

      Dimension Amat(500,500)    ! coefficient matrix
      Dimension rhs (500)
      Dimension sln (500)

      Dimension vector (5,2)     ! auxiliary 5-point arrowed vector

c--------------
c common blocks
c--------------

      common/grid_2d_1/glx,gly
      common/grid_2d_2/gp_x,gp_y
      common/grid_2d_3/gu_x,gu_y

c----------
c constants
c----------

      pi  = 3.14159 265358D0
      pi2 = 2.0D0*pi

      Null   = 0
      None   = 1
      Ntwo   = 2
      Nthree = 3
      Nfour  = 4
      Nfive  = 5

      zero = 0.0

c----------------------
c Read parameters from file: cvt_2d.dat
c----------------------

      open (3,file="cvt_2d.dat")
       read (3,*) top_vel                ! x-velocity of top
       read (3,*) ax,bx                  ! x limits
       read (3,*) ay,by                  ! y limits
       read (3,*) Nx,Ny                  ! divisions
      close (3)

      Ntot = Nx*(Ny+1)

      if(Ntot.gt.500) then
        write (6,*)
        write (6,*) " cvt_2d: imensioning requires that Nx*(Ny+1) < 500"
        write (6,*) " Please try again"
        stop
      end if

c--------
c prepare
c--------

      Nx1 = Nx+1
      Ny1 = Ny+1

      Dx = (bx-ax)/Nx     ! grid spacing
      Dy = (by-ay)/Ny     ! grid spacing

      beta =(Dx/Dy)**2

      Nmat = Ny*Nx1           ! size of the matrix

c-----------------
c open output file
c-----------------

      open (2,file="cvt_2d.out")

c-----------------------------------
c compute grid lines and grid points
c-----------------------------------

      call grid_2d
     +
     +    (ax,bx
     +    ,ay,by
     +    ,Nx,Ny
     +    )

c-------------------------------------
c specify the potential at the top row
c-------------------------------------

      Do i=1,Nx1
       ptl_top(i) = top_vel*glx(i)
      End Do

c--------------------------------------
c Print grid lines in file:  cvt_2d.out
c--------------------------------------

      Do i=1,Nx1
       write (2,100) Ntwo
       write (2,100) None,glx(i),ay
       write (2,100) Ntwo,glx(i),by
      End Do

      Do j=1,Ny1
       write (2,100) Ntwo
       write (2,100) None,ax,gly(j)
       write (2,100) Ntwo,bx,gly(j)
      End Do

c--------------------------------------
c Generate the finite-difference matrix
c--------------------------------------

      call cvt_2d_fdm
     +
     +   (Nx,Ny
     +   ,beta
     +   ,Amat
     +   )

c------------------------
c Set the right-hand side
c------------------------

      Do i=1,Nmat-Nx1
       rhs(i) = 0.0D0
      End Do

      Do i=1,Nx1
       rhs(Nmat-Nx1+i) = - beta*ptl_top(i)
      End Do

c-----------------
c room with a view
c-----------------

      write (6,*)
      write (6,*) " Enter 1 to view the linear system"
      write (6,*) " 0 to bypass "
      write (6,*) "-------------"
      read  (5,*) Iview

      if(Iview.eq.1) then
       Do i=1,Nmat
        write (6,102) (Amat(i,j),j=1,Nmat),rhs(i)
       End Do
      end if

c------------------------
c solve the linear system
c------------------------

      Isym_gel = 0     ! system is not symmetric
      Iwlpvt   = 1     ! pivoting enabled

      call gel
     +
     +   (Nmat,Amat
     +   ,rhs
     +   ,sln
     +   ,Isym_gel
     +   ,Iwlpvt
c    +   ,l,u
     +   ,det
     +   ,Istop
     +   )

c--------
c display
c--------

      write (6,*)
      write (6,*) " Solution vector:"
      write (6,*) "----------------"

      Do i=1,Nmat
        write (6,100) i,sln(i)
      End Do

c-------------------------
c assign solution to nodes
c-------------------------

      Ic = 1           ! counter

      Do j=1,Ny
       Do i=1,Nx1
        ptl(i,j) = sln(Ic)
        Ic = Ic+1
       End Do
      End Do

c--------------------------------------
c assign top potential to global matrix
c--------------------------------------

      Do i=1,Nx1
        ptl(i,Ny1) = ptl_top(i)
      End Do

c-------------------------------------------
c compute the velocity at the internal nodes
c by numerical differentiation
c using central differences
c-------------------------------------------

      Do i=2,Nx
       Do j=2,Ny
        gu_x(i,j) =  (ptl(i+1,j) -ptl(i-1,j))
     +             /(gp_x(i+1,j)-gp_x(i-1,j))
        gu_y(i,j) =  (ptl(i,j+1) -ptl(i,j-1))
     +             /(gp_y(i,j+1)-gp_y(i,j-1))
       End Do
      End Do

c----------------------------------
c Compute the velocity on the walls
c by numerical differentiation
c----------------------------------

c---
c left wall: i=1
c---

      Do j=2,Ny
        gu_x(1,j) = 0.0
        gu_y(1,j) = ( ptl(1,j+1)- ptl(1,j-1))
     +             /(gp_y(1,j+1)-gp_y(1,j-1))
      End Do

c---
c bottom wall: j=1
c---

      Do i=2,Nx
        gu_x(i,1) = ( ptl(i+1,1)- ptl(i-1,1))
     +             /(gp_x(i+1,1)-gp_x(i-1,1))
        gu_y(i,1) = 0.0
      End Do

c---
c right wall: i=Nx1
c---

      Do j=2,Ny
        gu_x(Nx1,j) = 0.0D0
        gu_y(Nx1,j) = ( ptl(Nx1,j+1) -ptl(Nx1,j-1))
     +               /(gp_y(Nx1,j+1)-gp_y(Nx1,j-1))
      End Do

c---
c top wall: j=Ny1
c---

      Do i=2,Nx
        gu_x(i,Ny1) = top_vel
        gu_y(i,Ny1) = ( ptl(i,Ny1)- ptl(i,Ny))
     +               /(gp_y(i,Ny1)-gp_y(i,Ny))
      End Do

c---
c four corners
c velocity is zero or singular;
c set it to zero
c---

      gu_x(1  ,1)   = 0.0D0
      gu_x(Nx1,1)   = 0.0D0
      gu_x(1  ,Ny1) = 0.0D0
      gu_x(Nx1,Ny1) = 0.0D0

      gu_y(1  ,1)   = 0.0D0
      gu_y(Nx1,1)   = 0.0D0
      gu_y(1  ,Ny1) = 0.0D0
      gu_y(Nx1,Ny1) = 0.0D0

c-------------------------------------------
c Prepare to draw velocity vectors
c
c Define vector plotting scale.
c
c The maximum length of the velocity vector
c will be 0.75 times the maximum of the
c grid spacing: dx or dy
c-------------------------------------------

      scale = 0.0D0

      Do i=1,Nx1
       Do j=1,Ny1
        vmax = Dsqrt(gu_x(i,j)**2+gu_y(i,j)**2)
        If(vmax.gt.scale) scale = vmax

       End Do
      End Do

      Dmax = Dx
      if(Dy.gt.Dmax) Dmax = Dy
      scale = 0.75*Dmax/scale

c     Dmin = Dx
c     If(Dy.lt.Dmin) Dmin = Dy
c     scale = 0.75*Dmin/scale

c------------------------------
c print 5-point velocity arrows
c------------------------------

      Do i=1,Nx1
       Do j=1,Ny1

        xl = gu_x(i,j)*scale
        yl = gu_y(i,j)*scale

        call draw_arrow_2d
     +
     +     (gp_x(i,j),gp_y(i,j)
     +     ,xl,yl
     +     ,vector
     +     )

       write (2,100) Nfive

       Do k=1,5
        write (2,100) k,vector(k,1),vector(k,2)
       End Do

       End Do
      End Do

c--------------------
c Close the data file
c--------------------

      write (2,*) Null
      close (2)

c-------------
c Finishing up
c-------------

      write (6,*)
      write (6,*) " Computation ended successfully"
      write (6,*)

  99  Continue

c---
c Done
c---

  100 Format (1x,i3,5(1x,f12.8))
  101 Format (5(1x,f12.8))
  102 Format (40(1x,f5.2))

      Stop
      End
