      subroutine ann2l0 
     +
     +  (a,ai,ae
     +  ,Icase
     +  ,n
     +  ,visc1,visc2
     +  ,gm0
     +  ,beta
     +  ,Ds
     +  ,k
     +  ,x1,x2
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c----------------------------------------

c--------------------------------------------------------
c  Instability of two stationary annular layers 
c  confined between two coaxial cylinders,
c  in the presence of an insoluble surfactant.
c
c  Inertial effects are neglected, and the motion occurs
c  under conditions of Stokes flow.
c
c  The determinant is a assumed to be function of 
c  the growth rate given by:
c
c  Det = a*grh**2 + b*grh + c
c
c  where: grh = gr-gref
c         gref is an arbitrary (reference) value
c
c  NOTATION:
c  --------
c
c  k:  disturbance wave number
c
c  a:  radius of the interface
c  ai: radius of inner cylinder
c  ae: radius of outer cylinder
c
c  visc1: viscosity of the internal fluid
c  visc2: viscosity of the external fluid
c
c  gm0:   unperturbed-state surface tension
c  beta:  linear constant for surface tension/surfactant conc
c        (set equal to zero for constant surface tension)
c  Ds:    surfactant diffusivity
c
c  secm(i, j):   secular matrix
c  n:            size of the secular matrix
c
c  Icase: Index for classifying configuration as follows:
c
c   1: inner cylinder present, outer cylinder present
c   2: inner cylinder absent, outer cylinder present
c   3: inner cylinder present, outer cylinder absent
c   4: inner cylinder absent, outer cylinder absent
c
c  To make the inner cylinder disappear, set ai = 0.0
c  To make the outer cylinder disappear, set ae = 0.0
c-----------------------------------------------

      Implicit Double Precision(a-h,k,o-z)

      Dimension secm(8,8)

      Parameter (delta=0.10,gref=0.12)  ! for the evaluation
                                        ! of the cubic coefficients
      Parameter (tol=0.00000000001)  ! for detecting surfactant

c----------
c constants
c----------

      pi = 3.14159 265358 D0
      one = 1.0D0

c-----------------------------------
c prepare for the finite differences
c-----------------------------------

      grb = gref-2.0D0*delta
      gra = gref-delta
      gr  = gref
      gr1 = gref+delta
      gr2 = gref+2.0D0*delta

      If(  abs(grb).lt.0.0000001
     + .or.abs(gra).lt.0.0000001
     + .or.abs(gr ).lt.0.0000001
     + .or.abs(gr1).lt.0.0000001
     + .or.abs(gr2).lt.0.0000001) then
        write (6,*) " ann2l0: inappropriate parameters:"
        write (6,*) " the trial growth rates cannot be zero:"
        write (6,*) " Please try again"
        Go to 99
      End If

c---------------------------------------------
c Will compute the poynomial coefficients
c by exact finite differences
c---------------------------------------------

c---
c first function evaluation
c---

      call matrix
     +
     +  (grb
     +  ,a
     +  ,ai
     +  ,ae
     +  ,visc1
     +  ,visc2
     +  ,gm0
     +  ,beta
     +  ,Ds
     +  ,k
     +  ,Icase
     +  ,n
     +  ,secm
     +  ,detb
     +  ,Istop
     +  )

      If(Istop.eq.1) then
        write (6,*) " ann2l0: error in determinant evaluation"
        Go to 99
      End If

c---
c second function evaluation
c---

      call matrix
     +
     +   (gra
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,beta
     +   ,Ds
     +   ,k
     +   ,Icase
     +   ,n
     +   ,secm
     +   ,deta
     +   ,Istop
     +   )

      If(Istop.eq.1) then
        write (6,*) " ann2l0: error in determinant evaluation"
        Go to 99
      End If

c---
c third function evaluation
c---

      call matrix
     +
     +   (gr
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,beta
     +   ,Ds
     +   ,k
     +   ,Icase
     +   ,n
     +   ,secm
     +   ,det
     +   ,Istop
     +   )

      If(Istop.eq.1) then
        write (6,*) " ann2l0: error in determinant evaluation"
        Go to 99
      End If

c---
c fourth function evaluation
c---

      call matrix
     +
     +   (gr1
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,beta
     +   ,Ds
     +   ,k
     +   ,Icase
     +   ,n
     +   ,secm
     +   ,det1
     +   ,Istop
     +   )

      If(Istop.eq.1) then
        write (6,*) " ann2l0: error in determinant evaluation"
        Go to 99
      End If

c---
c fifth function evaluation
c---

      call matrix
     +
     +   (gr2
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,beta
     +   ,Ds
     +   ,k
     +   ,Icase
     +   ,n
     +   ,secm
     +   ,det2
     +   ,Istop
     +   )

      if(Istop.eq.1) then
        write (6,*) " ann2l0: error in determinant evaluation"
        Go to 99
      end If

c-------------------------------
c Compute cubic coefficients
c by five-point centered differences
c 
c  The polynomial takes the form:
c
c  Det = f2*g**2 + f1*g + f0
c
c  where: grh = gr-gref
c-------------------------------

      f0 = det

      f1 = (-det2/12.0D0+2.0D0*det1/3.0D0
     +                  -2.0D0*deta/3.0D0
     +      +detb/12.0D0)/delta

      f2 = (-det2/12.0D0 + 4.0D0*det1/3.0D0 -5.0D0*det/2.0D0
     +                   + 4.0D0*deta/3.0D0 -detb/12.0D0)
     +      /(2.0D0*delta*delta)

c---
c the coefficient f3 is the
c third derivative and should be zero,
c confirming the quadratic equation
c---

      f3 = (det2/2.0D0-det1+deta-detb/2.0D0)/(6.0D0*delta**3)

c---
c the coefficient f4 is proportional to the
c fourth derivative and should be zero,
c confirming the quadratic equation
c---

      f4 = det2-4.0*det1+6.0*det-4.0*deta+detb


      if(abs(f4).gt.0.00000001) then

       write (6,*) "ann2l0: problem encountered at station 5"
       write (6,200) f0,f1,f2,f3,f4
       Go to 99

      endif

c
c the coefficients f3 and f4 should be zero
c
c     write (6,200) f0,f1,f2,f3,f4

c------------------------
c no surfactant (beta=0); linear equation
c------------------------

      if(abs(beta).le.tol) then

        x1 = -f0/f1 + gref
        x2 = 0.0D0  ! empty

        return

      End If

c------------------------
c surfactant (beta>0); quadratic equation
c------------------------

      aq = f2
      bq = f1
      cq = f0

      Disc = bq*bq-4.0D0*aq*cq

      x1 = (-bq+Dsqrt(Disc))/(2.0D0*aq)
      x2 = (-bq-Dsqrt(Disc))/(2.0D0*aq)

      x1 = x1 + gref
      x2 = x2 + gref

c-----
c Done
c-----

  99  Continue

 100  Format (1x,i3,10(1x,f15.8))
 101  Format (10(1x,f15.8))
 102  Format ("k, gr:",f10.8,f15.10)
 200  Format ("coefficients",5(1x,f12.8))

      return
      End
