      subroutine matrix
     +
     +   (gr
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,den1
     +   ,den2
     +   ,gm0
     +   ,els
     +   ,k
     +   ,Icase
     +   ,n
     +   ,ar
     +   ,det
     +   ,Istop
     +   )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c-------------------------------------------------
c Evaluates the secular matrix for the computation
c of the complex phase velocity
c
c NOTATION:
c --------
c
c k:    wave number
c gr:  -ick (growth rate)
c
c a:  radius of the interface
c ai: radius of inner cylinder
c ae: radius of outer cylinder
c
c visc1:   viscosity of the internal fluid
c visc2:   viscosity of the external fluid

c den1: density of the internal fluid
c den2: density of the external fluid
c gm0:  unperturbed-state surface tension

c els:  modulus of elasticity
c
c am:   primary secular matrix for the general configuration
c       with the inner and outer cylinder present
c ar:   reduced secular matrix accounting for special cases
c n:    size of ar
c det:  determinant of ar
c
c Icase: Index for classifying configuration as follows:
c
c        1: inner cylinder present, outer cylinder present
c        2: inner cylinder absent, outer cylinder present
c        3: inner cylinder present, outer cylinder absent
c        4: inner cylinder absent, outer cylinder absent
c-----------------------------------------------
       
      Implicit Double Precision(a-h,k,o-z)

      Dimension am(8,8),ar(8,8)
      Dimension rhs(8),sol(8)    ! for Gauss elimination only

c---
c prepare
c---

      Iswitch = 2     ! for the computation of the Bessel functions

      ks  = k*k
      as  = a*a

      ka   = k*a
      kas  = ka*ka
      kasc = 1.0D0-kas
      kai  = k*ai
      kae  = k*ae

      k1s  = ks+gr*den1/visc1
      k1   = sqrt(k1s)
      k1a  = k1*a
      k1ai = k1*ai
      k1ae = k1*ae

      k2s  = ks+gr*den2/visc2
      k2   = sqrt(k2s)
      k2a  = k2*a
      k2ai = k2*ai
      k2ae = k2*ae

c-----------------------------------------
c compute the modified bessel functions:
c
c   I0(k*a),I1(k*a)
c   K0(k*a),K1(k*a)
c----------------------------------------

      call bess_I01K01 
     +
     +   (ka,Iswitch
     +   ,BI0k,BI1k
     +   ,BK0k,BK1k
     +   )

c----------------------------------------
c compute the modified bessel functions:
c
c    I0(k*ai),I1(k*ai)
c    K0(k*ai),K1(k*ai)      
c----------------------------------------


      If(kai.gt.0.0000001) then

      call bess_I01K01 
     +
     +    (kai,Iswitch
     +    ,BI0ki,BI1ki
     +    ,BK0ki,BK1ki
     +    )

      End If

c----------------------------------------
c compute the modified bessel functions:
c
c    I0(k*ae),I1(k*ae)
c    K0(k*ae),K1(k*ae)      
c----------------------------------------

      If(kae.gt.0.0000001) then

      call bess_I01K01 
     +
     +    (kae,Iswitch
     +    ,BI0ke,BI1ke
     +    ,BK0ke,BK1ke
     +    )

      End If

c----------------------------------------
c compute the modified bessel functions:
c
c     I0(k1*a),I1(k1*a)
c     K0(k1*a),K1(k1*a)
c----------------------------------------

      call bess_I01K01 
     +
     +   (k1a,Iswitch
     +   ,BI0k1,BI1k1
     +   ,BK0k1,BK1k1
     +   )

c----------------------------------------
c compute the modified bessel functions:
c
c    I0(k1*ai),I1(k1*ai)
c    K0(k1*ai),K1(k1*ai)      
c----------------------------------------

      If(k1ai.gt.0.0000001) then

      call bess_I01K01 
     +
     +   (k1ai,Iswitch
     +   ,BI0k1i,BI1k1i
     +   ,BK0k1i,BK1k1i
     +   )

      End If

c----------------------------------------
c compute the modified bessel functions:
c
c   I0(k2*a),I1(k2*a),
c   K0(k2*a),K1(k2*a)
c----------------------------------------

      call bess_I01K01 
     +
     +   (k2a,Iswitch
     +   ,BI0k2,BI1k2
     +   ,BK0k2,BK1k2
     +   )

c----------------------------------------
c compute the modified bessel functions:
c
c   I0(k2*ae),I1(k2*ae),
c   K0(k2*ae),K1(k2*ae)      
c----------------------------------------

      If(k2ae.gt.0.0000001) then

      call bess_I01K01 
     +
     +   (k2ae,Iswitch
     +   ,BI0k2e,BI1k2e
     +   ,BK0k2e,BK1k2e
     +   )

      End If

c-----------------------------------------
c  compute the modified bessel functions 
c  of order 2
c-----------------------------------------

      BI2K = BI0K-2.0*BI1K/ka
      BK2K = BK0K+2.0*BK1K/ka

      BI2K1 = BI0K1-2.0*BI1K1/k1a
      BK2K1 = BK0K1+2.0*BK1K1/k1a

      BI2K2 = BI0K2-2.0*BI1K2/k2a
      BK2K2 = BK0K2+2.0*BK1K2/k2a

c---
c compute the derivative of Bessel functions of the first kind
c---

      dBI1K  = 0.5*(BI0K +BI2K)
      dBI1K1 = 0.5*(BI0K1+BI2K1)
      dBI1K2 = 0.5*(BI0K2+BI2K2)

c---
c compute the derivative of Bessel functions of the second kind
c---

      dBK1K  = -0.5*(BK0K +BK2K)
      dBK1K1 = -0.5*(BK0K1+BK2K1)
      dBK1K2 = -0.5*(BK0K2+BK2K2)
                                      
c-------------------------------------------------------
c Construct the secular matrix
c When both the inner and the outer cylinder are present,
c this is an 8 x 8 matrix
c--------------------------------------------------------
       
c---- first row: continuity of psi at the interface

      am(1,1) = BI1K
      am(1,2) = BI1K1
      am(1,3) = BK1K
      am(1,4) = BK1K1
      am(1,5) =-BI1K
      am(1,6) =-BI1K2
      am(1,7) =-BK1K
      am(1,8) =-BK1K2

c---- second row: continuity of dpsi/dsigma at sigma = a

      am(2,1) = k *BI0K
      am(2,2) = k1*BI0K1
      am(2,3) =-k *BK0K
      am(2,4) =-k1*BK0K1
      am(2,5) =-k *BI0K
      am(2,6) =-k2*BI0K2
      am(2,7) = k *BK0K
      am(2,8) = k2*BK0K2

c---- third row:  psi = 0 at inner cylinder

      am(3,1) = BI1Ki
      am(3,2) = BI1K1i
      am(3,3) = BK1Ki
      am(3,4) = BK1K1i
      am(3,5) = 0.0
      am(3,6) = 0.0
      am(3,7) = 0.0
      am(3,8) = 0.0

c---- fourth row: dpsi/dsigma = 0 at innner cylinder

      am(4,1) =-k *BI0Ki
      am(4,2) =-k1*BI0K1i
      am(4,3) = k *BK0Ki
      am(4,4) = k1*BK0K1i
      am(4,5) = 0.0
      am(4,6) = 0.0
      am(4,7) = 0.0
      am(4,8) = 0.0

c---- fifth row:  psi = 0 at outer cylinder

      am(5,1) = 0.0
      am(5,2) = 0.0
      am(5,3) = 0.0
      am(5,4) = 0.0
      am(5,5) = BI1Ke
      am(5,6) = BI1K2e
      am(5,7) = BK1Ke
      am(5,8) = BK1K2e

c---- sixth row: dpsi/dsigma = 0 at innner cylinder

      am(6,1) = 0.0
      am(6,2) = 0.0
      am(6,3) = 0.0
      am(6,4) = 0.0
      am(6,5) =-k *BI0Ke
      am(6,6) =-k2*BI0K2e
      am(6,7) = k *BK0Ke
      am(6,8) = k2*BK0K2e

c---- seventh row: shear stress interfacial condition

      am(7,1) =  visc1*(ks+ks )*BI1K
      am(7,2) =  visc1*(ks+k1s)*BI1K1
      am(7,3) =  visc1*(ks+ks )*BK1K
      am(7,4) =  visc1*(ks+k1s)*BK1K1

      am(7,5) = -visc2*(ks+ks )*BI1K
      am(7,6) = -visc2*(ks+k2s)*BI1K2
      am(7,7) = -visc2*(ks+ks )*BK1K
      am(7,8) = -visc2*(ks+k2s)*BK1K2

      fc  = -2.0*els*ks/(3.0*a*gr)

      am(7,1) = am(7,1) + fc*(-2.0*a*am(2,1)+am(1,1))
      am(7,2) = am(7,2) + fc*(-2.0*a*am(2,2)+am(1,2))
      am(7,3) = am(7,3) + fc*(-2.0*a*am(2,3)+am(1,3))
      am(7,4) = am(7,4) + fc*(-2.0*a*am(2,4)+am(1,4))

c----- eighth row : normal stress interfacial condition

      am(8,1) = 2.0*visc1*ks  *dBI1K  + gr*den1*BI0K
      am(8,2) = 2.0*visc1*k*k1*dBI1K1
      am(8,3) = 2.0*visc1*ks  *dBK1K  - gr*den1*BK0K
      am(8,4) = 2.0*visc1*k*k1*dBK1K1

      am(8,5) =-2.0*visc2*ks  *dBI1K  - gr*den2*BI0K
      am(8,6) =-2.0*visc2*k*k2*dBI1K2
      am(8,7) =-2.0*visc2*ks  *dBK1K  + gr*den2*BK0K
      am(8,8) =-2.0*visc2*k*k2*dBK1K2

      fc1 = k*(-gm0*kasc+4.0*els/3.0)/(gr*as)
      fc2 = 2.0*els*k/(3.0*a*gr)

      am(8,1) = am(8,1) + fc1*am(1,1) - fc2*am(2,1)
      am(8,2) = am(8,2) + fc1*am(1,2) - fc2*am(2,2)
      am(8,3) = am(8,3) + fc1*am(1,3) - fc2*am(2,3)
      am(8,4) = am(8,4) + fc1*am(1,4) - fc2*am(2,4)

c------------------------------------------
c normalize to reduce numerical sensitivity
c------------------------------------------

c     am(7,1) = am(7,1)*gr
c     am(7,2) = am(7,2)*gr
c     am(7,3) = am(7,3)*gr
c     am(7,4) = am(7,4)*gr
c     am(7,5) = am(7,5)*gr
c     am(7,6) = am(7,6)*gr
c     am(7,7) = am(7,7)*gr
c     am(7,8) = am(7,8)*gr

c     am(8,1) = am(8,1)*gr
c     am(8,2) = am(8,2)*gr
c     am(8,3) = am(8,3)*gr
c     am(8,4) = am(8,4)*gr
c     am(8,5) = am(8,5)*gr
c     am(8,6) = am(8,6)*gr
c     am(8,7) = am(8,7)*gr
c     am(8,8) = am(8,8)*gr

c---------------------------------------------------
c adjusting the primary matrix for specialized cases
c---------------------------------------------------

      if(Icase.eq.1) then

c----
c Inner cylinder present
c Outer cylinder present
c----

          Do ll=1,8
            Do jj=1,8
               ar(ll,jj)=am(ll,jj)
            End Do
          End Do

c----
c Inner cylinder absent
c Outer cylinder present
c---

      Else If(Icase.eq.2) then

          ar(1,1)=am(1,1)
          ar(1,2)=am(1,2)
          ar(1,3)=am(1,5)
          ar(1,4)=am(1,6)
          ar(1,5)=am(1,7)
          ar(1,6)=am(1,8)

          ar(2,1)=am(2,1)
          ar(2,2)=am(2,2)
          ar(2,3)=am(2,5)
          ar(2,4)=am(2,6)
          ar(2,5)=am(2,7)
          ar(2,6)=am(2,8)

          ar(3,1)=am(5,1)
          ar(3,2)=am(5,2)
          ar(3,3)=am(5,5)
          ar(3,4)=am(5,6)
          ar(3,5)=am(5,7)
          ar(3,6)=am(5,8)

          ar(4,1)=am(6,1)
          ar(4,2)=am(6,2)
          ar(4,3)=am(6,5)
          ar(4,4)=am(6,6)
          ar(4,5)=am(6,7)
          ar(4,6)=am(6,8)

          ar(5,1)=am(7,1)
          ar(5,2)=am(7,2)
          ar(5,3)=am(7,5)
          ar(5,4)=am(7,6)
          ar(5,5)=am(7,7)
          ar(5,6)=am(7,8)

          ar(6,1)=am(8,1)
          ar(6,2)=am(8,2)
          ar(6,3)=am(8,5)
          ar(6,4)=am(8,6)
          ar(6,5)=am(8,7)
          ar(6,6)=am(8,8)

c----
c Inner cylinder present
c Outer cylinder absent
c----

       elseif(Icase.eq.3) then

          ar(1,1)=am(1,1)
          ar(1,2)=am(1,2)
          ar(1,3)=am(1,3)
          ar(1,4)=am(1,4)
          ar(1,5)=am(1,7)
          ar(1,6)=am(1,8)

          ar(2,1)=am(2,1)
          ar(2,2)=am(2,2)
          ar(2,3)=am(2,3)
          ar(2,4)=am(2,4)
          ar(2,5)=am(2,7)
          ar(2,6)=am(2,8)

          ar(3,1)=am(3,1)
          ar(3,2)=am(3,2)
          ar(3,3)=am(3,3)
          ar(3,4)=am(3,4)
          ar(3,5)=am(3,7)
          ar(3,6)=am(3,8)

          ar(4,1)=am(4,1)
          ar(4,2)=am(4,2)
          ar(4,3)=am(4,3)
          ar(4,4)=am(4,4)
          ar(4,5)=am(4,7)
          ar(4,6)=am(4,8)

          ar(5,1)=am(7,1)
          ar(5,2)=am(7,2)
          ar(5,3)=am(7,3)
          ar(5,4)=am(7,4)
          ar(5,5)=am(7,7)
          ar(5,6)=am(7,8)

          ar(6,1)=am(8,1)
          ar(6,2)=am(8,2)
          ar(6,3)=am(8,3)
          ar(6,4)=am(8,4)
          ar(6,5)=am(8,7)
          ar(6,6)=am(8,8)  

c----
c Inner cylinder absent
c Outer cylinder absent
c----

       elseif(Icase.eq.4) then

          ar(1,1)=am(1,1)
          ar(1,2)=am(1,2)
          ar(1,3)=am(1,7)
          ar(1,4)=am(1,8)

          ar(2,1)=am(2,1)
          ar(2,2)=am(2,2)
          ar(2,3)=am(2,7)
          ar(2,4)=am(2,8)

          ar(3,1)=am(7,1)
          ar(3,2)=am(7,2)
          ar(3,3)=am(7,7)
          ar(3,4)=am(7,8)

          ar(4,1)=am(8,1)
          ar(4,2)=am(8,2)
          ar(4,3)=am(8,7)
          ar(4,4)=am(8,8)
           
      End If
       
c-------------------------
c compute the determinant
c------------------------

      Isym_gel = 0   ! matrix is not symmetric
      Iwlpvt   = 1   ! pivoting enabled

      call gel
     +
     +  (n,ar
     +  ,rhs
     +  ,sol
     +  ,Isym_gel,Iwlpvt
c    +  ,l,u
     +  ,det
     +  ,Istop
     +  )

c---
c all done
c---

      Return
      End
