       subroutine chan2l0_s
     +
     +  (RL          ! wave length
     +  ,h           ! channel semi-width
     +  ,y0          ! position of the unperturbed interface
     +  ,th0         ! inclination angle
     +  ,vs1,vs2     ! viscosities
     +  ,den1,den2   ! densities
     +  ,gamma0
     +  ,Ds
     +  ,Ma         ! Marangoni number
     +  ,U1,U2
     +  ,pg
     +  ,gac
     +  ,Iframe
     +  ,Ishow
     +  ,xi1,xi2
     +  ,uI
     +  ,cr1,ci1
     +  ,sr1,si1
     +  ,cr2,ci2
     +  ,sr2,si2
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------------------------------
c  Instability of the two-layer channel flow
c  in the presence of an insoluble surfactant.
c
c  Computation of the
c  phase velocity and growth rate from analytical
c  expressions for Stokes flow.
c
c  Shear-, Pressure, or Gravity-driven flow
c
c  1 refers to lower fluid, 2 to upper fluid.
c
c       ----> U2
c   =========================
c            Fluid 2
c   --------------------------
c            Fluid 1
c   =========================
c       --> U1
c
c LEGEND:
c ------
c
c  pg = -dp/dx pressure gradient for Poiseuille flow
c
c  Iframe = 0	stationary frame
c	    1	moving with the interfacial velocity
c
c  uI:  interfacial velocity
c  th0: inclination angle
c
c  gac:  acceleration due to gravity.
c
c  NOTE: ci is unaffected by the base flow,
c        only cr depends on the base flow
c-----------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension QR(4,4),QI(4,4)

      Double Precision k,ks
      Double Precision Ma
      Double Precision l,lc,LAMR,LAMI

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

c-------- 
c prepare
c-------- 

      gx = gac*Dsin(th0) ! gravity vector
      gy =-gac*Dcos(th0)

      k  = pi2/RL        ! wave number
      ks = k*k

      h1 = h+y0          ! lower layer thickness
      h2 = h-y0          ! upper layer thickness
      hs = h*h

      r  = h2/h1         ! thickness ratio
      rs = r*r
      r1 = 1.0D0+r

      l = vs2/vs1       ! viscosity ratio
      lc = 1.0D0-l

      d = den2/den1     ! density ratio
      Drho  = den1-den2 ! density difference

      DU = U2-U1

      hh1 = k*h1
      hh2 = k*h2

      ch1 = Dcosh(hh1)
      ch2 = Dcosh(hh2)

      sh1 = Dsinh(hh1)
      sh2 = Dsinh(hh2)

      hh = k*h
      ch = Dcosh(hh)
      ch = 1.0D0

c----------------------------------------------
c  Shear rate one either side of the interface,
c
c  xi1, xi2
c----------------------------------------------

      xi1 = DU/h1 * l/(l+r)
     +     -pg*(l-rs)       /( vs1*r1*(l+r) )
     +     -den1*gx*(l-d*rs)/( vs1*r1*(l+r) )

      xi2 = xi1/l

c     write (6,*) " chan2l0_s: interface shear rates: ",xi1,xi2

c--------------------
c  interface velocity
c--------------------

      uI =  (r*U1+l*U2)/(r+l)
     +
     +  + (pg*hs/vs1) * 2.0D0*r/(r1*(l+r)) 
     +  + (den1*gx*h*h/vs1)*
     +           (1.0D0+d*r)*2.0D0*r /(r1*r1*(l+r)) 

c     write (6,*) " chan2l0: interfacial velocity: ",uI

c----------------------------
c  compile the secular matrix
c----------------------------

      Do i=1,4
       Do j=1,4
        QR(i,j) = 0.0D0   ! real
        QI(i,j) = 0.0D0   ! imaginary
       End Do
      End Do

c--------
c prepare
c--------

      BXi1 = xi1/k
      BXi2 = xi2/k
      DBXi = BXi2-BXi1

      BGx = Drho*gx/(2.0D0*vs1*ks)
      BGy = Drho*gy/(2.0D0*vs1*ks)

      BPI = gamma0/(2.0D0*vs1) - BGy

      C12R = hh1*( DBXi*ch1-BGx*sh1)
      C12I = BPI*(hh1*ch1-sh1)

      C22R = DBXi*(ch1+hh1*sh1)
     +        -BGx*(sh1+hh1*ch1)
      C22I = BPI*hh1*sh1

      C13 =  BXi2*hh1*sh1           ! srf
      C23 = -BXi2*(sh1+hh1*ch1)     ! srf

c---------
c zeta = 0
c---------

      zR = 0.0D0
      zI = 0.0D0

c---
c real part
c---

      QR(1,1) = ch2
      QR(1,2) = hh2*ch2
      QR(1,3) = sh2
      QR(1,4) = hh2*sh2

      QR(2,1) = sh2
      QR(2,2) = ch2 + hh2*sh2
      QR(2,3) = ch2
      QR(2,4) = sh2 + hh2*ch2

      QR31    = ch1 - lc*hh1*sh1
      QR(3,1) = QR31
      QR32    = -hh1*ch1
      QR(3,2) = QR32
      QR33    = -lc*hh1*ch1 - l*sh1
      QR(3,3) = QR33
      QR(3,4) =  l*hh1*sh1

      QR41    = -sh1 + lc*(sh1+hh1*ch1)
      QR(4,1) = QR41
      QR42    = ch1+hh1*sh1
      QR(4,2) = QR42
      QR43    = ch1 + lc*hh1*sh1
      QR(4,3) = QR43
      QR(4,4) = -l*(sh1+hh1*ch1)

c--------------------
c det(Q) = a zeta + b
c
c where b = Q(0)
c--------------------

      call det_44c
     +
     +   (QR,QI
     +   ,detR0,detI0
     +   )

c---
      If(Ishow.eq.1) then
       Do i=1,4
         write (6,101) (QR(i,j),QI(i,j),j=1,4)
       End Do
       write (6,102) detR0,detI0
      End If
c---

c---------
c zeta = pm 1
c---------

c===
      Do irepeat=1,3
c===

      if(irepeat.eq.1) then
       zR = 1.0D0
       zI = 0.0D0
      elseif(irepeat.eq.2) then
       zR =-1.0D0
       zI = 0.0D0
      else
       zR3 = 3.8734D0 ! arbitrary
       zI3 =-1.9875D0 ! arbitrary
       zR3 = 3.0
       zI3 = 3.0
       zR = zR3
       zI = zI3
      end if

      den = (1.0-Ds*k*zI)**2 + (Ds*k*zR)**2
      cf = 0.5D0*Ma*gamma0/(vs1*den)
      TMPR = -cf*Ds*k*zR
      TMPI = -cf*(1.0D0-Ds*k*zI)
      LAMR = zR*TMPR - zI*TMPI
      LAMI = zR*TMPI + zI*TMPR

      C12Ra = C12R - LAMR*C13
      C12Ia = C12I - LAMI*C13

      QR(3,1) = QR31 - (zR*C12Ra-zI*C12Ia)
      QI(3,1) =      - (zR*C12Ia+zI*C12Ra)

      QR(3,2) = QR32 + hh1*sh1*LAMR
      QI(3,2) =        hh1*sh1*LAMI

      QR(3,3) = QR33 + hh1*sh1*LAMR
      QI(3,3) =        hh1*sh1*LAMI

      C22Ra = C22R + LAMR*C23
      C22Ia = C22I + LAMI*C23

      QR(4,1) = QR41 + (zR*C22Ra-zI*C22Ia)
      QI(4,1) =        (zR*C22Ia+zI*C22Ra)

      QR(4,2) = QR42 - LAMR*(hh1*ch1+sh1)
      QI(4,2) =      - LAMI*(hh1*ch1+sh1)

      QR(4,3) = QR43 - LAMR*(hh1*ch1+sh1)
      QI(4,3) =      - LAMI*(hh1*ch1+sh1)

      call det_44c
     +
     +   (QR,QI
     +   ,detR,detI
     +   )


c---
c multiply the determinant by 1 + D_s k i zeta
c to render it a binomial in zeta
c---

      tmpR = 1.0D0-Ds*k*zI
      tmpI =       Ds*k*zR
      savR = detR*tmpR-detI*tmpI
      savI = detR*tmpI+detI*tmpR
      detR = savR
      detI = savI

c---
      If(Ishow.eq.1) then
        Do i=1,4
          write (6,101) (QR(i,j),QI(i,j),j=1,4)
        End Do
       write (6,102) detR, detI
      End If
c---

      if(irepeat.eq.1) then
       detR1 = detR
       detI1 = detI
      elseif(irepeat.eq.2) then
       detR2 = detR
       detI2 = detI
      else
       detR3 = detR
       detI3 = detI
      end if

c===
      End do
c===

c     write (6,*) detR0,DetI0
c     write (6,*) detR1,detI1
c     write (6,*) detR2,detI2
c     write (6,*) detR3,detI3

c===
c secular equation is: det(Q) = A Z^2 + B Z + det0 = 0
c
c where:
c
c   A + B = det1-det0
c   A - B = det2-det0
c===

      AR = 0.5D0*(detR1+detR2-2.0D0*detR0)
      AI = 0.5D0*(detI1+detI2-2.0D0*detI0)
      BR = 0.5D0*(detR1-detR2)
      BI = 0.5D0*(detI1-detI2)
      CR = detR0
      CI = detI0

c====
c confirm the secular equation
c====

      zR3s = zR3*zR3-zI3*zI3
      zI3s = 2.0D0*zR3*zI3

      QuadR = AR*zR3s-AI*zI3s + BR*zR3-BI*zI3 + CR - detR3
      QuadI = AR*zI3s+AI*zR3s + BR*zI3+BI*zR3 + CI - detI3

      eps = 0.00001
      if((abs(QuadR).gt.eps).or.(abs(QuadI).gt.eps)) then
       write (6,*) "these should be zero:"
       write (6,*) QuadR,QuadI
      end if

c====
c normalize the coefficients of the quadratic
c====

      fnorm = abs(AR)

      if(abs(AI).gt.fnorm) then
       fnorm = abs(AI)
      end if
      if(abs(BR).gt.fnorm) then
       fnorm = abs(BR)
      end if
      if(abs(BI).gt.fnorm) then
       fnorm = abs(BI)
      end if
      if(abs(CR).gt.fnorm) then
       fnorm = abs(CR)
      end if
      if(abs(CI).gt.fnorm) then
       fnorm = abs(CI)
      end if

      AR = AR/fnorm
      AI = AI/fnorm
      BR = BR/fnorm
      BI = BI/fnorm
      CR = CR/fnorm
      CI = CI/fnorm

c     write (6,104) AR,AI,BR,BI,CR,CI

c====
c roots of the quadratic
c====

      call quadc
     +
     +  (AR,AI,BR,BI,CR,CI
     +  ,ZR1,ZI1
     +  ,ZR2,ZI2
     +  )

c====
c confirm the roots
c====

c     write (6,*) 'zeta:'
c     write (6,*) ZR1,ZI1
c     write (6,*) ZR2,ZI2

      zR1s = zR1*zR1-zI1*zI1
      zI1s = 2.0D0*zR1*zI1
      QuadR = AR*zR1s-AI*zI1s + BR*zR1-BI*zI1 + CR
      QuadI = AR*zI1s+AI*zR1s + BR*zI1+BI*zR1 + CI
c     write (6,*) "these should be zero:"
c     write (6,*) QuadR,QuadI

      zR2s = zR2*zR2-zI2*zI2
      zI2s = 2.0D0*zR2*zI2
      QuadR = AR*zR2s-AI*zI2s + BR*zR2-BI*zI2 + CR
      QuadI = AR*zI2s+AI*zR2s + BR*zI2+BI*zR2 + CI
c     write (6,*) "these should be zero:"
c     write (6,*) QuadR,QuadI

c     ZR1 = 1.0
c     ZI1 = 0.0
c     ZR2 = 1.0
c     ZI2 = 0.0

c==============================================

c-------------------------------------
c complex phase velocity: c = uI + 1/z
c-------------------------------------

      Dnm = ZR1*ZR1+ZI1*ZI1
      cr1 =  ZR1/Dnm
      ci1 =- ZI1/Dnm
      if(Iframe.eq.0) cr1 = cr1+uI

      Dnm = ZR2*ZR2+ZI2*ZI2
      cr2 =  ZR2/Dnm
      ci2 =- ZI2/Dnm
      if(Iframe.eq.0) cr2 = cr2+uI

c--------------------
c complex growth rate
c--------------------

      sr1 = k*cr1
      si1 = k*ci1

      sr2 = k*cr2
      si2 = k*ci2

c------
c order the modes
c------

      if(si1.lt.si2) then

        save = cr1
        cr1  = cr2
        cr2 = save

        save = ci1
        ci1  = ci2
        ci2 = save

        save = sr1
        sr1  = sr2
        sr2  = save

        save = si1
        si1  = si2
        si2  = save

      end if

c-----
c Done
c-----

  101 Format (1x,4(3x,f13.5,1x,f13.5))
  102 Format ("Determinant:",1x,f15.10,1x,f15.10)
  103 Format ("zeta       :",1x,f15.10,1x,f15.10)
  104 Format ("quad coeff :",10(1x,f12.10))

      Return 
      End
