clear all
close all

%====================
% FDLIB
%
% Solution of the Orr--Sommerfeld equation
% in a domain -b<y<a
%
% N finite-difference divisions
%
% ====+ N+1   y=a
%     |
%     + N
%     |
%     +
%     |
%     + 2
%     |
% ====+ 1     y=-b
%
% iflow = 1 hyperbolic tangent profile
% iflow = 2 parabolic Hagen-Poseuille flow
%
% U0:  reference velocity
%====================

%---
% preferences
%---

iflow = 1;  % hyperbolic tangent profile
iflow = 2;  % Hagen-Poseuille flow (parabolic)

kmax = 1.2;  % maximum wave number
nkloop = 32; % number of wave numbers

N = 16;
N = 8;
N = 9;
N = 32;
N = 64;
N = 256;
N = 2*256;  % discretization level
N = 128;

%---
% plotting flags
%---

iplot2d = 1;
iplotcn = 1;
iplotms = 1;

%---
% choose the flow and set parameters
%---

if(iflow==1)
 a = 5.0;
 b = 5.0;
 U0 = 1.0;
 delta = 1.0;
 Restart = 1.0;  % starting Reynolds number
 Reend = 40.0;  % starting Reynolds number
 nReloop = 16;  % number of Re
elseif(iflow==2)
 a = 1.0;
 b = a;
 U0 = 1.0;
 Restart = 2000.0;
 Reend = 40000.0;
 nReloop = 32;
end

%---
% prepare
%---

Dy = (a+b)/N;  % grid size
Dys = Dy^2;
Dk = kmax/nkloop;
im = sqrt(-1);   % imaginary unit
fcRe = (Reend/Restart)^(1/(nReloop-1));  % factor for ramping Re

%---
% nodal velocity and second derivatives
%---

for i=1:N+1

  y(i) = -b+(i-1.0)*Dy;

  if(iflow==1)
    eta = y(i)/delta;
    U(i)   =  U0*tanh(eta);
    Upp(i) = -2.0*U0*sinh(eta)/cosh(eta)^3;
  elseif(iflow==2)
    U(i)   = U0*(1.0-y(i)^2/a^2);
    Upp(i) = -2.0*U0;   % second derivative
  end


end

%------------------------------------
% pentadiagonal matrix approximating
% the fourth derivative f''''
%------------------------------------

P = zeros(N-1,N-1);

P(1,1)= 7.0; P(1,2)=-4.0; P(1,3)= 1.0;
P(2,1)=-4.0; P(2,2)= 6.0; P(2,3)=-4.0; P(2,4)= 1.0;

for i=3:N-3
 P(i,i-2)=1; P(i,i-1)=-4; P(i,i)=6; P(i,i+1)=-4; P(i,i+2)=1;
end

P(N-2,N-4)= 1.0; P(N-2,N-3)=-4.0; P(N-2,N-2)= 6.0; P(N-2,N-1)=-4.0;
                 P(N-1,N-3)= 1.0; P(N-1,N-2)=-4.0; P(N-1,N-1)= 7.0;
P = P/Dy^4;

%------------------------------------
% tridiagonal matrix approximating
% the second derivative f''
%------------------------------------

T = zeros(N-1,N-1);

T(1,1) = -2.0; T(1,2) =1.0;

for i=2:N-2
 T(i,i-1)=1.0; T(i,i)=-2.0; T(i,i+1)=1.0;
end

T(N-1,N-2) = 1.0; T(N-1,N-1) =-2.0;

T = T/Dys;

%---------------------------
% another tridiagonal matrix
% approximating U f''
%---------------------------

T1 = zeros(N-1,N-1);

T1(1,1) = -2.0*U(2); T1(1,2) = U(2);

for i=2:N-2
 Unode = U(i+1);
 T1(i,i-1) = Unode; T1(i,i) = -2.0*Unode; T1(i,i+1) = Unode;
end

T1(N-1,N-2) = U(N); T1(N-1,N-1) = -2.0*U(N);

T1 = T1/Dys;

%==================
% scan the wave number (k)
%==================

%nkloop = 0;  % override

%---
for kloop=1:nkloop+1
%---

 k = (kloop-1.0)*Dk+0.0001;
% k  = 0.782;  % override

 ks = k*k;

%==================
% scan the Reynolds number
%==================

   Re = Restart;

%   nReloop = 1; % override

%---
 for Reloop=1:nReloop
%---

% Re = 46950.0;  % override

  if(iflow==1) 
   nu = U0*delta/Re;
  elseif(iflow==2) 
   nu = U0*a/Re;
  end

  A = im*nu/k *P + T1- 2.0*k*im*nu*T;
 
  for i=1:N-1
    A(i,i) = A(i,i) - U(i+1)*ks - Upp(i+1) + im*nu*k^3;
  end

  B = T;
  for i=1:N-1
    B(i,i) = B(i,i)-ks;
  end

%---
% eigenvalues
%---

%A = A*hs;
%B = B*hs;
%A = A*hs;
%B = B*hs;

%egv = eig(inv(B)*A)
egv = eig(A,B);

gr = imag(k*egv); % growth rate
%gr
%pause

%grsort = sort(gr,'descend');
 grsorted = -sort(-gr);
%grsort
%pause

Rey(Reloop) = Re;

ggrr(kloop,Reloop) = grsorted(1);

if(iflow==1)
 wn(kloop) = k*delta;
elseif(iflow==2)
 wn(kloop) = k*a;
end

Re = fcRe*Re;  % ramp up re

%---
end  % of Reloop
%---

Re = Re/fcRe;

%---
end  % of kloop
%---

%-------------------
% 2D plot
%-------------------

 if(iplot2d==1)

 figure(1)
 hold on
 set(gca,'fontsize',15)
 box on

 if(iflow==1)
  xlabel('k\delta','fontsize',15)
  ylabel('\sigma_I\delta/U_0','fontsize',15)
  axis([0 1.2 -0.2 0.2])
 elseif(iflow==2)
  xlabel('ka','fontsize',15)
  ylabel('\sigma_{I}a/U_0','fontsize',15)
  axis([0 1.2 -0.06 0.01])
 end

 plot(wn,ggrr(:,1),'k','linewidth',3);

 for Reloop=2:nReloop
  plot(wn,ggrr(:,Reloop),'k');
 end

 plot([0 1.2],[0,0],'--k')

 end

%----
% contour plot
%----

 if(iplotcn==1)

 figure(2)
 hold on
 set(gca,'fontsize',15)
 box on
 xlabel('Re','fontsize',15)

 if(iflow==1)
  [C,h]=contour(Rey,wn,ggrr,[0.0, 0.0],'k-');
   set(h, 'LineWidth', 3); 
   clabel(C,h);
  [C,h]= contour(Rey,wn,ggrr,[-0.2, -0.2],'k');
   clabel(C,h);
  [C,h]= contour(Rey,wn,ggrr,[-0.1, -0.1],'k');
   clabel(C,h);
  [C,h]=contour(Rey,wn,ggrr,[0.05, 0.05],'k');
   clabel(C,h);
  [C,h]=contour(Rey,wn,ggrr,[0.10, 0.10],'k');
   clabel(C,h);
  [C,h]=contour(Rey,wn,ggrr,[0.14, 0.14],'k');
   clabel(C,h);
   ylabel('k\delta','fontsize',15)
   axis([0 40 0 1])
 elseif(iflow==2)
   [C,h]=contour(Rey,wn,ggrr,[0.0, 0.0],'k-');
   plot([5572 5572],[0.55,1.2],'k--')
   ylabel('ka','fontsize',15)
   axis([0 40000 0.55 1.2])
 end

 end

%----
% mesh plot
%----

 if(iplotms==1)

 figure(3)
 set(gca,'fontsize',15)
 mesh(Rey,wn,ggrr);
 box on
 xlabel('Re','fontsize',15)

 if(iflow==1)
  zlabel('\sigma_I\delta/U_0','fontsize',15)
  ylabel('k\delta','fontsize',15)
  axis([0 40 0 1 -1.5 0.2])
 elseif(iflow==2)
  zlabel('\sigma_{I}a/U_0','fontsize',15)
  ylabel('ka','fontsize',15)
   axis([0 40000 0 1.2 -0.06 0.01])
 end

 end
