      program prony_dr

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-------------------------
c  Driver for Prony fitting
c
c  Prony-fitting of a signal described by a
c  time series, with a sum of normal modes:
c
c  Sum { [ar*cos(sr*t) + ai*sin(sr*t)] * exp(si*t) }
c
c  Each mode can be expressed as a pair of 
c  complex conjugate exponentials.
c
c  SYMBOLS:
c  --------
c
c  Npoints: Number of points in the time series
c  signal:  Time series
c  t0:      Time series will begin at time: t0
c  Dt:	    Time interval (sampling time)
c
c  Nexp:     Number of modes ( complex exponentials)
c            A mode is a combination:
c
c       [ ar*cos(sr*t) + ai*sin(sr*t)] * exp(si*t)
c
c  z:  z = exp(-i*sigma*Dt)
c
c--------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension signal(32000)
      Dimension ar(20),ai(20)
      Dimension sr(20),si(20)
      Dimension z(20,2),sigma(20,2),coeff(20,2)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Null = 0
      None = 1
      Ntwo = 2
      Nthr = 3
      Nfou = 4

c------
c input
c------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 to generate the time series"
      write (6,*) " 2 to read the time series from file: prony.inp"
      write (6,*) " ----------------------------------------------"
      read  (5,*) Igen

      If(Igen.eq.0) Go to 99

c-----------------------
      If(Igen.eq.1) then
c-----------------------

c-------------------------------------
c Read parameters from file: prony.dat
c and generate the sample signal:
c
c signal = SUM[ (ar*cos(sr*t) + ai*sin(sr*t)) * exp(si*t) ]
c
c using specified values
c-------------------------------------

      open (1,file="prony.dat")

       read (1,*) Npoints
       read (1,*) Dt
       read (1,*) Nexpr   ! number of complex exponentials (up to 4)
       read (1,*) t0
       read (1,*)
       read (1,*) cc
       read (1,*)

       write (6,*) " prony_dr: number of modes: ",Nexpr

       If(Nexpr.gt.4) then
         write (6,*) "cannot be larger than 4; abort"
         stop
       End If

       Do j=1,Nexpr
         read (1,*) ar(j),ai(j),sr(j),si(j)
       End Do

       close (1)

c--------------------
c generate the signal
c--------------------

       Do i=1,Npoints

         t = (i-1.0D0)*Dt-t0

         signal(i) = cc   ! mean

         Do j=1,Nexpr
          Irep = 0
          Do l=1,j-1
           same = Dsqrt( (sr(j)-sr(l))**2+(si(j)-si(l))**2 )
           If(same.le.0.00000001) Irep = Irep+1
          End Do
c         write (6,*) Irep
          arg = sr(j)*t
          comp = (ar(j)*Dcos(arg)+ai(j)*Dsin(arg)) * Dexp(si(j)*t)
          If(Irep.ne.0) comp = t**Irep * comp
          signal(i) = signal(i) + comp

         End Do 

       End Do 

c-----------------------
      Else If(Igen.eq.2) then  ! READ THE SIGNAL
c-----------------------

       open (4,file="prony.inp")

        read (4,*) Npoints,Dt

        sig_mean = 0.0D0

        Do i=1,Npoints
          read (4,*) idle,a,a,signal(i)
c         write (6,*) signal(i)
          sig_mean = sig_mean +signal(i)
        End Do
        sig_mean = sig_mean/Npoints
        Do i=1,Npoints
          signal(i)=signal(i)-sig_mean
        End Do

        write (6,*)
        write (6,*) " mean value: ",sig_mean

      close (4)

c-----------
      End If
c-----------

  98   Continue

       write (6,*)
       write (6,*) " How many normal modes to fit?"
       write (6,*)
       write (6,*) " Choose from 1, 2, 3"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Nexp

       If(Nexp.eq.0) Go to 99

c--------
c prepare
c--------

      Nexp2 = 2*Nexp     ! number of complex exponentials

      write (6,*) "2*Nexp  = ", Nexp2

      Iopt = 1
      If(abs(cc).gt.0.000000001) Iopt = 2

                    write (6,*) "Iopt: ",Iopt
                    write (6,*) "Caution:"
      If(Iopt.eq.1) write (6,*) "Npoints must be greater than 2*Nexp"
      If(Iopt.eq.2) write (6,*) "Npoints must be greater than 2*Nexp+1"
                    write (6,*) "Npoints = ", Npoints

c-----------
c call prony
c-----------

      call prony
     +
     +  (Npoints
     +  ,signal
     +  ,Dt
     +  ,Nexp
     +  ,Iopt
     +  ,z        ! output
     +  ,sigma    ! output
     +  ,coeff    ! output
     +  )

c---------
c printing
c---------

c----------------------
      If(Igen.eq.1) then
c----------------------

      write (6,*)
      write (6,*) " Specified sigma_r, sigma_i, a_r, a_i :"
      write (6,*) " --------------------------------------"
      write (6,*)

      Do j=1,Nexpr
        write (6,104) j,sr(j),si(j),ar(j),ai(j)
      End Do

      write (6,*)
      write (6,*) " Computed sigma_r, sigma_i, a_r, a_i :"
      write (6,*) " -------------------------------------"
      write (6,*)

      Do i=1,Nexp2
       write (6,104) i,sigma(i,1),sigma(i,2),coeff(i,1),coeff(i,2)
      End Do

c----------------------
      Else If(Igen.eq.2) then
c----------------------

      write (6,*)
      write (6,*) " Computed sigma_r, sigma_i, a_r, a_i :"
      write (6,*) " -------------------------------------"
      write (6,*)

      Do i=1,Nexp2
       write (6,104) i,sigma(i,1),sigma(i,2),coeff(i,1),coeff(i,2)
      End Do

c---
c construct the exponential fitting
c---

      open (9,file="prony.out")

       write (9,104) Npoints

       Do i=1,Npoints

         t = (i-1.0D0)*Dt
         sig = 0.0D0

         Do j=1,Nexp
          k = 2*j-1
          arg   = sigma(k,1)*t
          sig = sig + (coeff(k,1)*Dcos(arg)
     +                +coeff(k,2)*Dsin(arg))*Dexp(sigma(k,2)*t)
         End Do 

         signal(i) = signal(i) + sig_mean
         sig       = sig       + sig_mean

         write (9,104) i,t,signal(i),sig

       End Do 

       write (9,104) Null

      close (9)

c----------
      End If
c----------

      Go to 98

c-----
c Done
c-----

  99  Continue

 104  Format (1x,i4,10(1x,f15.10))

      Stop
      End
