      program lv_lia

c============================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c============================================

c-------------------------------------------------------
c Simulation of the motion of a closed 
c three-dimensional (3D) line vortex
c using sing the local induction approximation (LIA)
c
c Local stretching
c or compression of the line is handled by re-gridding.  
c
c The radius of the
c vortex core is updated by assuming constant volume.
c
c Time integration by the explicit Euler method.
c 
c Smoothing by the 5-point formula.
c
c NOTATION:
c --------
c
c  x,y,z        coordinates of marker points
c  vx,vy,vz     velocity of marker points
c  rcore        radius of the core
c  s	        arc length along the line vortex
c  xsm,ysm,zsm	Hold coordinates for smoothing
c  xc,yc,sc	center of the arc passing through 
c               three successive points
c  r            radius of the arc passing through
c               three successive points
c  al           arc length of an arc
c  spmi         minimum point sepatation
c  spmax        maximum point sepatation
c
c  al_save      saved arc length
c----------------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(0:501),y(0:501),z(0:501),rcore(0:501)

      Dimension unused1(0:501),unused2(0:501)
      Dimension unused3(0:501),unused4(0:501)

      Dimension xc(500),yc(500),zc(500),r(500),s(500)
      Dimension chi1(500),chi3(500)

      Dimension xsm(0:501),ysm(0:501),zsm(0:501)

      Dimension  vx(500),vy(500),vz(500)

      Dimension al_save(500)

      Character kbd*1        ! graphics
      Integer checkkey       ! graphics

c---
c common blocks
c---

      common/xxyyzz/x,y,z
      common/ARCC/xc,yc,zc,r,s,chi1,chi3

c---
c constants
c---

      pi  = 3.14159 265358 97932 384 D0
      pi2 = 2.0D0*pi
      pi4 = 4.0D0*pi

      null = 0

c---
c parameters
c---

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to read parameters from data file: lv_lia.dat"
      write (6,*) " 2 to enter them"
      write (6,*) " 0 to quit"
      write (6,*) '----------'
      read  (5,*) Ienrd

      if(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.1) then
c------------------------

      open (3,file="lv_lia.dat")

       read (3,*) npts
       read (3,*) zinit
       read (3,*) rinit
       read (3,*) circ
       read (3,*) 
       read (3,*) thmax
       read (3,*) spmin
       read (3,*) spmax
       read (3,*)
       read (3,*) Iread
       read (3,*)
       read (3,*) nsteps
       read (3,*) Dt

      close (3)

c---
      else    ! read the parameters
c---

      write (6,*)
      write (6,*) 'Enter the number of marker points'
      write (6,*) 'along over the line vortex'
      write (6,*) '--------------------------'
      read  (5,*) npts

      write (6,*)
      write (6,*) 'Enter the initial z position'
      write (6,*) '----------------------------'
      read  (5,*) zinit

      write (6,*)
      write (6,*)
      write (6,*) 'Enter initial core radius'
      write (6,*) '-------------------------'
      read  (5,*) rinit

      write (6,*)
      write (6,*) 'Enter circulation around core'
      write (6,*) '----------------------------'
      read  (5,*) circ

      write (6,*)
      write (6,*) " Enter the maximum angle subtended by an arc"
      write (6,*) " in multiples of pi"
      write (6,*) '-------------------'
      read  (5,*) thmax

      write (6,*)
      write (6,*) " Enter the minimum and maximum point separation"
      write (6,*) '-----------------------------------------------'
      read  (5,*) spmin,spmax

      write (6,*)
      write (6,*) " Enter 0 to generate initial shape"
      write (6,*) "       1 to read it from file: lv_lia.inp"
      write (6,*) '----------------------------------------'
      read  (5,*) Iread

      write (6,*)
      write (6,*) 'Enter number of time steps till pause'
      write (6,*) '------------------------------------'
      read  (5,*) nsteps

      write (6,*)
      write (6,*) 'Enter time step'
      write (6,*) '---------------'
      read (5,*) Dt

c-----------
      end if
c-----------

c-----------------------------------------
c Assign the initial position of marker points
c and core radius
c-----------------------------------------

c------------------------
      if(Iread.eq.0) then
c------------------------

       Dth = pi2/npts

       Do i=1,npts
        th   = (i-1.0D0)*dth
        x(i) = 1.0D0*Dcos(th)
        y(i) = 1.2D0*Dsin(th)
c       z(i) = zinit+0.7D0*Dcos(th)**2
        z(i) = zinit+1.3D0*Dcos(th)**2
        rcore(i) = rinit             ! constant core radius
       End Do

c---------
      else
c---------

       open (2,file="lv_lia.inp")
 
       read (2,*) npts
       Do i=1,npts
        read (2,*) x(i),y(i),z(i),rcore(i)
       End Do

       close (2)

c-----------
      end if
c-----------
  
c--------
c prepare
c--------

      thmax = thmax*pi

      open (4,file="lv_lia.out")

      npts1 = npts+1

c---
c  wrap-around 
c---

      x(0) = x(npts)
      y(0) = y(npts)
      z(0) = z(npts)
      rcore(0) = rcore(npts)

      x(npts1) = x(1)
      y(npts1) = y(1)
      z(npts1) = z(1)
      rcore(npts1) = rcore(1)

c---
c print initial shape
c---

      write (4,101) npts1
      Do i=1,npts1
       write (4,101) i,x(i),y(i),z(i),rcore(i)
      End Do

c-----------
c initialize
c-----------

      time = 0.0D0

      Itry = 1                    ! graphics

      write (6,*)
      write (6,*) " Press s to pause"
      write (6,*) "       q to quit"
      write (6,*)

c-.-.-.-.-.-.-.-.-.-.-.-.-.-.
c BEGINNING OF THE SIMULATION
c-.-.-.-.-.-.-.-.-.-.-.-.-.-.

      Do istep=1,nsteps           !  loop over time steps

c---
c compute properties of arcs
c and regrid if necessary
c---

      Ich1 = 1
      Ich2 = 1
      Ich3 = 1

      call prd_3d
     +
     +    (Npts
     +    ,ICH1,thMAX
     +    ,ICH2,spMAX
     +    ,ICH3,spMIN
     +    ,rcore,unused1
     +    ,unused2,unused3,unused4
     +    ,Istop
     +    )

      if(Istop.eq.1) Go to 99

      npts1 = npts+1
      npts2 = npts+2
      npts3 = npts+3

c---
c  wrap-around 
c---

          x(0) = x(npts)
          y(0) = y(npts)
          z(0) = z(npts)
      rcore(0) = rcore(npts)

      x(npts1) =         x(1)
      y(npts1) =         y(1)
      z(npts1) =         z(1)
      rcore(npts1) = rcore(1)
 
c---
c compute the binormal vector
c---

      Do i=1,npts           !  loop over nodes on vortex line

        ia = i-1
        i1 = i+1

        x1 = x(ia)
        y1 = y(ia)
        z1 = z(ia)
        x2 = x(i)
        y2 = y(i)
        z2 = z(i)
        x3 = x(i1)
        y3 = y(i1)
        z3 = z(i1)

        a31 = (y2-y1)*(z3-z1)-(y3-y1)*(z2-z1)
        a32 = (x3-x1)*(z2-z1)-(x2-x1)*(z3-z1)
        a33 = (x2-x1)*(y3-y1)-(x3-x1)*(y2-y1)

        tmp = Dsqrt(a31**2 + a32**2 + a33**2)

        bx= -a31/tmp
        by= -a32/tmp
        bz= -a33/tmp

c---
c computethe  velocity at node 'i' based on 
c the Local Induction Approximiation (LIA)
c---

        vel = -circ/(pi4*r(i)) 
     +      * (log(8.0*r(i)/rcore(i)) - 0.25)

        vx(i) = vel * bx
        vy(i) = vel * by
        vz(i) = vel * bz

       End Do   !  end of loop over nodes

c---
c  update nodal positions 
c  by the explicit euler method
c---

      Do i=1,npts
       x(i) = x(i)+vx(i)*Dt
       y(i) = y(i)+vy(i)*Dt
       z(i) = z(i)+vz(i)*Dt
      End Do

      time = time+Dt

c---
c  wrap-around 
c---

      x(0) = x(npts)
      y(0) = y(npts)
      z(0) = z(npts)
      rcore(0) = rcore(npts)

      x(npts1) = x(1)
      y(npts1) = y(1)
      z(npts1) = z(1)
      rcore(npts1) = rcore(1)
 
c---
c save old arc length
c to be used for updating the core radius
c---

      Do i=1,npts
       al_save(i) = r(i)*(chi3(i)-chi1(i))
      End Do

c---
c  update core radius by comparing initial and final volumes of
c  arc segments.
c
c  Point redistribution disabled
c---

      Ich1 = 0
      Ich2 = 0
      Ich3 = 0

      call prd_3d
     +
     +  (Npts
     +  ,ICH1,thMAX
     +  ,ICH2,dsMAX
     +  ,ICH3,dsMIN
     +  ,rcore,unused1,unused2,unused3,unused4
     +  ,Istop
     +  )

      if(Istop.eq.1) Go to 99

c---
c compute current core radius by
c assuming volume of core segments
c is constant.
c---

      Do i=1,npts
        tmp      = r(i)*(chi3(i)-chi1(i))
        rcore(i) = rcore(i)*sqrt( al_save(i)/tmp )
c       write (6,101) i,x(i),y(i),z(i),rcore(i)
      End Do

c---
c  wrap-around 
c---

          x(0) =     x(npts)
          y(0) =     y(npts)
          z(0) =     z(npts)
      rcore(0) = rcore(npts)

          x(npts1) =     x(1)
          y(npts1) =     y(1)
          z(npts1) =     z(1)
      rcore(npts1) = rcore(1)

c---
c smoothing by
c the five-point formula
c---

      x(npts2) = x(2)
      y(npts2) = y(2)
      z(npts2) = z(2)
      x(npts3) = x(3)
      y(npts3) = y(3)
      z(npts3) = z(3)

      Do i=2,npts1
       ib = i-2
       ia = i-1
       i1 = i+1
       i2 = i+2
       xsm(i) = (-x(ib)+4.*x(ia)+10.*x(i)+4.*x(i1)-x(i2))/16.0
       ysm(i) = (-y(ib)+4.*y(ia)+10.*y(i)+4.*y(i1)-y(i2))/16.0
       zsm(i) = (-z(ib)+4.*z(ia)+10.*z(i)+4.*z(i1)-z(i2))/16.0
      End Do

      Do i=2,npts1
       x(i) = xsm(i)
       y(i) = ysm(i)
       z(i) = zsm(i)
      End Do

c---
c  wrap-around 
c---

      x(0) = x(npts)
      y(0) = y(npts)
      z(0) = z(npts)
      x(1) = xsm(npts1)
      y(1) = ysm(npts1)
      z(1) = zsm(npts1)

c-----------------
c printing session
c-----------------

      write (6,*)
      write (6,*) "istep = ",istep,"time = ",time
c     write (6,*)
c     write (6,*) " x, y, z, core"
c     write (6,*)

      write (4,101) npts+1

      Do i=1,npts+1
       write (4,101) i,x(i),y(i),z(i),rcore(i)
c      write (6,101) i,x(i),y(i),z(i),rcore(i)
      End Do

c---
      End Do  !  end loop over time steps
c---

 99   Continue

c-----
c done
c-----

      write (4,*) null
      close (1)

 101  Format (1x,i3,10(1x,f10.5))

      stop
      end
