      program ring

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------------------------
c  Computation of the axial velocity at the centerline 
c  of a vortex ring with core of finite (non-zero) radius
c  and circular cross-section, for specified:
c
c  ring radius,
c  core radius,
c  core vorticity distribution 
c
c  (see Pozrikidis 1977, secs. 2.9 and 11.7).  
c
c
c  SYMBOLS:
c  -------
c
c  rr ... ring radius
c  cr ... core radius
c
c  u, v ... axial and radial velocity at the
c           center of the vortex ring
c
c  nrad ... number of radial segments for the
c           core discretization
c
c  nchi ... number of azimuthal segments for the
c           core discretization
c
c  rad ... radial distance from core center
c  sig ... distance from the  x axis
c
c  w ... vorticity 
c
c  circ ... circulation around vortex core.
c
c  Positive value corresponds to vorticity
c  in the direction of the meridional unit vector
c  so that the core rotates
c  the counterclocksise direction.
c--------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension ar(900,900)

c----------
c constants
c----------

      pi  = 3.14159265358D0
      pi2 = 2.0D0*pi
      pi4 = 4.0D0*pi
      pis = pi*pi

c--------
c prepare
c--------

      open (2, file="PLOTDAT")

c-----------
c preferences
c-----------

  98  write (6,*) 

      write (6,*)
      write (6,*) ' Select the core vorticity distribution:  '
      write (6,*) 
      write (6,*) ' Enter 0 to quit'
      write (6,*) '       1 for constant vorticity'
      write (6,*) '       2 for cosine bell distribution'
      write (6,*) ' -------------------------------------'
      read (5,*) menu

      If(menu.eq.0) Go to 99

      write (6,*) 
      write (6,*) 'Enter the ring radius'
      write (6,*) '---------------------'
      read  (5,*) rr

      write (6,*) 
      write (6,*) 'Enter the core radius'
      write (6,*) '---------------------'
      read  (5,*) cr

      write (6,*) 
      write (6,*) 'Enter the circulation around core'
      write (6,*) '---------------------------------'
      read  (5,*) circ

      write (6,*) 
      write (6,*) 'Enter the number of radial divisions'
      write (6,*) 
      write (6,*) 'for numerical integration'
      write (6,*) '-------------------------'
      read  (5,*) nrad

      write (6,*) 
      write (6,*) 'Enter the number of angular divisions'
      write (6,*) 
      write (6,*) 'for numerical integration'
      write (6,*) '-------------------------'
      read  (5,*) nchi

c--------------------
c     Do 888 ii=1,100
c     cr = 0.01*ii
c--------------------

c--------
c prepare
c--------

      dcr  = cr/nrad   ! radial divisions
      dchi = pi2/nchi  ! angular divisions
                       ! for numerical integration

c-----------------------------------------
c Discretize the vortex core into elements
c
c Compute the area of each element: ar
c-----------------------------------------

      Do i=1,nrad
       Do j=1,nchi
         ar(i,j)=0.5*dchi*dcr*dcr*(2.0D0*i-1.0D0)
       End Do
      End Do
 
c-----------
c initialize
c-----------

      arintv = 0.0         ! areal integral of the vorticity
      u = 0.0
      v = 0.0

c---------------------- 
c self-induced velocity  
c
c Integrate by the trapezoidal rule
c----------------------

      Do i=1,nrad          ! loop over radial elements

        rad=(i-0.5)*dcr    ! center of the ring elements

c---
c compute vorticity of the radial division
c---
   
        If(menu.eq.1) then        

          w = circ/(pi*cr*cr)     ! uniform vorticity

        Else If(menu.eq.2) then

          w = circ*pi/(cr*cr)/(pis-4.0)
     +         *(cos(pi*rad/cr)+1.0)      ! normalized bell shape

        End If

        Do j=1,nchi             !  loop over azimuthal elements
     
         chi = (j-0.5)*dchi
         sig = rr+rad*sin(chi)     
         Dx  = rad*cos(chi)     !  (x-hat from Pozrikidis)
         Dxs = Dx**2

c---
c  Calculate I and J integrals
c  formulae (2.9.8)-(2.9.10), 
c  p. 84 of Pozrikidis (1997)
c---

	 rks = 4.0*rr*sig/(Dxs+(rr+sig)**2)

         call ell_int (rks,F,E)

         RJ30 = E/(1.0-rks)
         RJ31 = (-2.0*F + E*(2.0-rks)/(1.0-rks))/rks

         cf = 4.0/sqrt((Dxs+(rr+sig)**2)**3)

         RI30 = cf*RJ30
         RI31 = cf*RJ31

c---
c compute filament contributions
c to axial and radial velocity
c---
   
         cf = ar(i,j)*sig*w/pi4

         u = u + cf* (-rr*RI31+sig*RI30)
         v = v + cf*   Dx*RI31

         arintv = arintv + w*ar(i,j)
        
       End Do                !  end of azimuthal loop

       End Do        !  end of radial loop

c------------------------
c Helmholtz approximation
c for comparison purposes
c------------------------

      uh = circ/(pi4*rr) * (log(8.0*rr/cr)-0.25)

c---
c printing session
c---

      write (6,100) u
      write (6,101) uh
      write (6,102) v
      write (6,*)
      write (6,103) arintv
      write (6,104) circ
      write (2,200) ii,cr,u,uh

c-------------
c 888 Continue
c-------------

      Go to 98    ! repeat

c-----
c Done
c-----

 99   Continue  

      close (2)

  100 Format ("Computed  axial velocity = ",f15.10)
  101 Format ("Helhmoltz axial velocity = ",f15.10)
  102 Format ("Radial velocity          = ",f15.10)
  103 Format ("Areal integral of vorticity = ",f15.10)
  104 Format ("Circulation                 = ",f15.10)
  200 Format (1x,i3,3(1x,f15.10))

      Stop
      End
