close all
clear all

%===================================
% FDLIB
%
% Computation of an evolving flow
% in a rectangular cavity
% in primary variables using the
% velocity/pressure formulation
%
% The flow is computed using a
% projection method
%
% SYMBOLS:
%
% x,y:      grid nodes
% ux, uy:   velocity components
% chi:      projection function
% uxi, uyi: intermediate velocity (generic)
%===================================

%-----------------------
% settings and parameters
%-----------------------

Lx = 1.0; % cavity length
Ly = 1.0; % cavity depth 
Ly = 0.5;

Nx=32; % grid size
Ny=32;
Nx=2*8; % grid size
Ny=2*4;
Nx=4*8; % grid size
Ny=4*4;
Nx=8; % grid size
Ny=4;

Dt = 0.001; % time step
Dt = 0.1;

visc = 0.01; % viscosity
dens = 1.0; % density

Vlidamp = 1.0; % lid velocity amplitude

Nstep = 2000;  % number of steps

%--------------------------------------------
% parameters for solving the Poisson equation
% for the projection function
%--------------------------------------------

itermax = 50000;
tol = 0.000001;
relax = 0.2;

qleft = 0.0;  % Neumann boundary condition
qright = 0.0;
qbot = 0.0;
qtop = 0.0;

Ishift = 1;

%----------------------------------------
% parameters for slip velocity iterations
%----------------------------------------

slipN = 50;        % max number of iterations
sliptol = 0.00001; % tolerance
sliprel = 1.0;     % relaxation

%--------
% prepare
%--------

nu = visc/dens; % kinematic viscosity

Dx = Lx/Nx;
Dy = Ly/Ny;

Dx2 = 2.0*Dx;
Dy2 = 2.0*Dy;

Dxs = Dx*Dx;
Dys = Dy*Dy;

Dtor = Dt/dens;

%--------
% lid velocity
%--------

for i=1:Nx+1
  Vlid(i) = Vlidamp;
end

%-----------------------------------
% define grid lines
% initialize velocity (ux, uy)
% and the projection function (chi)
%-----------------------------------

time = 0.0;

for j=1:Ny+1
 for i=1:Nx+1
   x(i,j) = (i-1.0)*Dx;
   y(i,j) = (j-1.0)*Dy;
   ux(i,j) = 0.0;   % x velocity
   uy(i,j) = 0.0;   % y velocity
   chi(i,j) = 0.0;   % projection function
   chroma(i,j) = 0.0;
 end
end

for i=1:Nx+1
   ux(i,Ny+1) = Vlid(i);
end

%-----------------------------------
% naive velocity boundary conditions
%-----------------------------------

for i=1:Nx+1
  BCxt(i) = Vlid(i);  % top wall
  BCxb(i) = 0.0;      % bottom wall
end

for j=1:Ny+1
  BCyl(j) = 0.0; % left wall
  BCyr(j) = 0.0; % right wall
end

%--------------
% time stepping
%--------------

%===============
for step=1:Nstep
%===============

%---
% animation
%---

if(step==1)
  Handle1 = quiver(x,y,ux,uy,'k');
  set(Handle1, 'erasemode', 'xor');
  axis ([-0.10, Lx+0.10, -0.10, Ly+0.10])
  axis equal
  set(gca,'fontsize',15)
  xlabel('x','fontsize',15)
  ylabel('y','fontsize',15)
  hold on
  plot([0, Lx, Lx, 0, 0],[0, 0, Ly, Ly, 0],'k');
else
% set(Handle1,'UData',ux,'VData',uy);
  %---
  quiver(x,y,ux,uy,'k');
  hold on
  plot([0, Lx, Lx, 0, 0],[0, 0, Ly, Ly, 0],'k');
  axis ([0.00, Lx+0.00, 0.00, Ly+0.00])
  axis equal
  set(gca,'fontsize',15)
  xlabel('x','fontsize',15)
  ylabel('y','fontsize',15)
  hold off
  %---
  pause(0.01)
  drawnow
end

%-------------------------------------
% initialize the intermediate velocity
%-------------------------------------

for j=1:Ny+1
  for i=1:Nx+1
   uxi(i,j) = ux(i,j);
   uyi(i,j) = uy(i,j);
  end
end

%=============================
% perform inner iterations
% for the projection function
% to satisfy the no-slip
% boundary condition
%=============================

 for inner=1:slipN

%---
% zero the entries of the tridiagonal matrix
%---

 for i=1:Nx-1
   atr(i) = 0.0;
   btr(i) = 0.0;
   ctr(i) = 0.0;
 end

%-------------------------------------------------
%  Integrate conv-diff equation in the x direction
%  using the Crank-Nicolson method
%  Advance the velocity from u^n to u*
%-------------------------------------------------

  Iskip = 0;
  if(Iskip==0)

  al = nu*Dt/Dxs;

%--------------
  for j=2:Ny          % run over rows
%----

    for i=2:Nx
      Rc = ux(i,j)*Dt/Dx;     % cell Reynolds number
      C1 =   Rc + 2.0*al;
      C2 =        4.0*(1.0-al);
      C3 =  -Rc + 2.0*al;
      ctr(i-1) =  -Rc - 2.0*al;
      atr(i-1) =        4.0*(1.0+al);
      btr(i-1) =   Rc - 2.0*al;
      rhsx(i-1) = C1*ux(i-1,j) + C2*ux(i,j) + C3*ux(i+1,j);  % right-hand side
      rhsy(i-1) = C1*uy(i-1,j) + C2*uy(i,j) + C3*uy(i+1,j);  % right-hand side
    end

    rhsy(1)    = rhsy(1)    - ctr(1)   *BCyl(j);
    rhsy(Nx-1) = rhsy(Nx-1) - btr(Nx-1)*BCyr(j);

    solx = thomas(Nx-1,atr,btr,ctr,rhsx);  % x component
    soly = thomas(Nx-1,atr,btr,ctr,rhsy);  % y component

    for k=1:Nx-1
     uxi(k+1,j) = solx(k);
     uyi(k+1,j) = soly(k);
    end

%---
   end  % End of running over rows
%-----------

  for i=1:Nx-1     % reset the tridiagonal matrix
    atr(i) = 0.0;
    btr(i) = 0.0;
    ctr(i) = 0.0;
  end

 end  % of skip

%-------------------------------------------------
%  Integrate Conv-Diff equation in the y direction
%  using the Crank-Nicolson method
%  Advance the velocity from u* to u**
%-------------------------------------------------

  Iskip = 0;
  if(Iskip==0)

  al = nu*Dt/Dys;

%---
  for i=2:Nx     % run over columns
%---
    for j=2:Ny     % from bottom to top
      Rc = uy(i,j)*Dt/Dy;       % cell Reynolds number
      C1 =  Rc +2.0*al;
      C2 =      4.0*(1.0-al);
      C3 = -Rc +2.0*al;
      ctr(j-1) = -Rc -2.0*al;
      atr(j-1) =      4.0*(1.0+al);
      btr(j-1) =  Rc -2.0*al;
      rhsx(j-1) = C1*uxi(i,j-1) + C2*uxi(i,j) + C3*uxi(i,j+1); % right-hand side
      rhsy(j-1) = C1*uyi(i,j-1) + C2*uyi(i,j) + C3*uyi(i,j+1); % right-hand side
    end

    rhsx(1)    = rhsx(1)    - ctr(1)   *BCxb(i); %    boundary conditions
    rhsx(Ny-1) = rhsx(Ny-1) - btr(Ny-1)*BCxt(i); %    boundary conditions

    solx = thomas (Ny-1,atr,btr,ctr,rhsx);  % x component
    soly = thomas (Ny-1,atr,btr,ctr,rhsy);  % y component

    for k=1:Ny-1
      uxi(i,k+1) = solx(k);
      uyi(i,k+1) = soly(k);
    end
%---
  end   % of run over columns
%---

 end  % of skip

%-------------------------------------
% Compute intermediate compressibility
% by centered differences
%
%       Divus = Div u**
%-------------------------------------

% initialize

  for j=1:Ny+1
    for i=1:Nx+1
      Divus(i,j) = 0.0;
    end
  end

% interior nodes

  for i=2:Nx
    for j=2:Ny
      DuDx = (uxi(i+1,j)-uxi(i-1,j))/Dx2;
      DvDy = (uyi(i,j+1)-uyi(i,j-1))/Dy2;
      Divus(i,j) = DuDx+DvDy;
     end
   end

% left wall

  for j=1:Ny+1
   DuDx = (-3.0*uxi(1,j)+4.0*uxi(2,j)-uxi(3,j))/Dx2;
   DvDy = 0.0;
   Divus(1,j) = DuDx+DvDy;
  end

   save11l = Divus(1,j);     % save for corners
   save12l = Divus(1,Ny+1);   % save for corners

% bottom wall

   for i=1:Nx+1
     DuDx = 0.0;
     DvDy = (-3.0*uyi(i,1)+4.0*uyi(i,2)-uyi(i,3))/Dy2;
     Divus(i,1) = DuDx+DvDy;
   end

   save11b = Divus(1,1);     % save for corners
   save21b = Divus(Nx+1,1);   % save for corners

% right wall

   for j=1:Ny+1
     DuDx = (3.0*uxi(Nx+1,j)-4.0*uxi(Nx,j)+uxi(Nx-1,j))/Dx2;
     DvDy = 0.0;
     Divus(Nx+1,j) = DuDx+DvDy;
   end

   save21r = Divus(Nx+1,1);     % save for corners
   save22r = Divus(Nx+1,Ny+1);   % save for corners

% top wall

   for i=1:Nx+1
%     DuDx = (Vlid(i+1)-Vlid(i-1))/Dx2;
     DuDx = 0.0;
     DvDy = (3.0*uyi(i,Ny+1)-4.0*uyi(i,Ny)+uyi(i,Ny-1))/Dy2;
     Divus(i,Ny+1) = DuDx+DvDy;
   end

   save12t = Divus(1,   Ny+1);      % save for corners
   save22t = Divus(Nx+1,Ny+1);      % save for corners

% corners by averaging

   Divus(1,      1) = 0.5*(save11l+save11b);
   Divus(1,   Ny+1) = 0.5*(save12l+save12t);
   Divus(Nx+1,   1) = 0.5*(save21b+save21r);
   Divus(Nx+1,Ny+1) = 0.5*(save22r+save22t);

%----------------------------------
% Solve for the projection function
% by Gauss-Siedel (GS) iterations
%----------------------------------

   Iskip = 0;
   if(Iskip==0)

%---
% source term
%-----

   for i=1:Nx+1
    for j=1:Ny+1
      source(i,j) = -Divus(i,j)/Dtor;
     end
   end

  [chi,iter,Iflag] = pois_gs_nnnn ...
 ...
    (Nx,Ny,Dx,Dy,source,itermax,tol,relax ...
 ...
    ,qleft,qright,qbot,qtop,chi,Ishift);

  if(Iflag==0)
   disp "cvt_pm: Poisson solver did not converge"
   break
  end

%----------------------------------
% project the velocity at all nodes
% except at the corner nodes
%----------------------------------

%---
% interior nodes
%----

  for i=2:Nx
    for j=2:Ny
     DchiDx = (chi(i+1,j)-chi(i-1,j))/Dx2;
     DchiDy = (chi(i,j+1)-chi(i,j-1))/Dy2;
     uxi(i,j) = uxi(i,j) - Dtor*DchiDx;
     uyi(i,j) = uyi(i,j) - Dtor*DchiDy;
    end
  end

%--------------
% lower boundary
%
% Use forward differences with
% special treatment of the near-corner nodes
%--------------

   for i=2:Nx
      DchiDy = 0.0;
      if(i==2)
         DchiDx = (-3.0*chi(2,1)+4.0*chi(3,1)-chi(4,1))/Dx2;
      elseif(i==Nx)
         DchiDx =-(-3.0*chi(Nx,1)+4.0*chi(Nx-1,1)-chi(Nx-2,1))/Dx2;
      else
         DchiDx = (chi(i+1,1)-chi(i-1,1))/Dx2;
      end
      uxi(i,1) = BCxb(i) - Dtor*DchiDx;
      uyi(i,1) =         - Dtor*DchiDy;
    end

%--------------
% upper boundary
%
% Use backward differences
% special treatment for the near-corner nodes
%--------------

   for i=2:Nx
      DchiDy = 0.0;
      if(i==2)
        DchiDx = (-3.0*chi(2,Ny+1)+4.0*chi(3,Ny+1)-chi(4,Ny+1))/Dx2;
      elseif(i==Nx)
        DchiDx =-(-3.0*chi(Nx,Ny+1)+4.0*chi(Nx-1,Ny+1)-chi(Nx-2,Ny+1))/Dx2;
      else
        DchiDx = (chi(i+1,Ny+1)-chi(i-1,Ny+1))/Dx2;
      end
      uxi(i,Ny+1) = BCxt(i) - Dtor*DchiDx;
      uyi(i,Ny+1) =         - Dtor*DchiDy;
   end

%-------------
% left boundary
%
% Use forward differences
% special treatment for the near-corner nodes
%-------------

   for j=2:Ny
      DchiDx = 0.0;
      if(j==2)
        DchiDy = (-3.0*chi(1,2)+4.0*chi(1,3)-chi(1,4))/Dy2;
      elseif(j==Ny)
        DchiDy =-(-3.0*chi(1,Ny)+4.0*chi(1,Ny-1)-chi(1,Ny-2))/Dy2;
      else
        DchiDy = (chi(1,j+1)-chi(1,j-1))/Dy2;
      end
      uxi(1,j) =         - Dtor*DchiDx;
      uyi(1,j) = BCyl(j) - Dtor*DchiDy;
   end

%--------------
% right boundary
%
% Use backward differences
% special treatment for the near-corner nodes
%--------------

    for j=2:Ny
      DchiDx = 0.0;
      if(j==2)
        DchiDy = (-3.0*chi(Nx+1,2)+4.0*chi(Nx+1,3)-chi(Nx+1,4))/Dy2;
      elseif(j==Ny)
        DchiDy =-(-3.0*chi(Nx+1,Ny)+4.0*chi(Nx+1,Ny-1)-chi(Nx+1,Ny-2))/Dy2;
      else
        DchiDy = (chi(Nx+1,j+1)-chi(Nx+1,j-1))/Dy2;
      end
      uxi(Nx+1,j) =         - Dtor*DchiDx;
      uyi(Nx+1,j) = BCyr(j) - Dtor*DchiDy;
    end

  end % of Iskip

%-------------------------------------- 
% Compute the wall slip velocity
%
% and modify the boundary conditions
% for the intermediate (star) velocities
%-------------------------------------- 

    slipmax = 0.0;

% top and bottom:

    for i=1:Nx+1
      cor = uxi(i,Ny+1)-Vlid(i);
      if(abs(cor)>slipmax) slipmax = cor; end
      BCxt(i) = BCxt(i)-sliprel*cor;
        cor = uxi(i,1);
        if(abs(cor)>slipmax) slipmax = cor; end
        BCxb(i) = BCxb(i)-sliprel*cor;
    end

% left and right:

    for j=1:Ny+1
      cor = uyi(1,j);
      if(abs(cor)>slipmax) slipmax = cor; end
      BCyl(j) = BCyl(j)-sliprel*cor;
        corr = uyi(Nx+1,j);
        if(abs(cor)>slipmax) slipmax = cor; end
        BCyr(j) = BCyr(j)-sliprel*cor;
    end

    slipmax

    if(slipmax<sliptol) break; end


%=============================
  end % of inner iterations
%=============================

%-----------------------------------
% update velocity to the final value
%-----------------------------------

  for j=1:Ny+1
    for i=1:Nx+1
      ux(i,j) = uxi(i,j);
      uy(i,j) = uyi(i,j);
    end
  end

%------------
% update time
%------------

  time = time + Dt

%===========
 end  % of time stepping
%===========

figure(2)
mesh(x,y,ux);
set(gca,'fontsize',15)
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('u_x','fontsize',15)
set(gca,'fontsize',15)
axis([0 1 0 0.5 -0.2 1])
box

figure(3)
mesh(x,y,uy);
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('u_y','fontsize',15)
set(gca,'fontsize',15)
axis([0 1 0 0.5 -0.3 0.2])
box

figure(4)
mesh(x,y,chi,chroma);
xlabel('x','fontsize',15)
ylabel('y','fontsize',15)
zlabel('\chi','fontsize',15)
set(gca,'fontsize',15)
axis([0 1 0 0.5 -0.5 1.0])
box
