      subroutine fft 
     +
     +  (nexp
     +  ,g
     +  ,a,b
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c------------------------------------------------
c This program accompanies the book:
c
c         C. Pozrikidis
c
c Numerical Computation in Science and Engineering
c          Oxford University Press
c------------------------------------------------

c----------------------------------------
c Fast Fourier Transform on the (complex) 
c data set g(i).
c
c This subroutine returns the (complex) 
c frequency space representation.
c 
c Number of complex members of the data set g
c must be an integral power of 2:
c
c   n =  2^nexp
c
c Input data set is numbered from 0 to n-1
c
c The Fourier coefficients a and b are computed with
c the convention that elements 1 to n/2-1 
c correspond to positive frequency,
c and elements n/2+1 to n-1 correspond to negative
c frequency.  
c
c Element n/2 represents the nyquist frequency, both
c positive and negative.
c
c SYMBOLS:
c --------
c
c  g .... complex data set (g(i,1)=real, g(i,2)=imag.)
c  gr ... real part of frequency space representation
c  gi ... imaginary part of frequency space representation
c
c  nexp . size of data set (power of 2)
c 
c  p .... ordering vector
c  mu ... integer power of two  
c  a .... real (cos) elements of frequency representation
c  b .... imaginary (sin) elements of frequency representation
c
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension g(0:32768,2),gr(0:32768),gsr(0:32768)
      Dimension              gi(0:32768),gsi(0:32768)
      Dimension a(0:32768),b(0:32768),c(0:32768),s(0:32768)

      Integer p(0:32768)

c----------
c constants
c----------

      pi = 3.14159 265358D0
      pi2 = 2.0D0*pi

c--------
c prepare
c--------

      n = 2**nexp    ! set size
      na = n-1

c----------------------------
c copy the complex data set g 
c onto the vectors gr and gi
c----------------------------

      Do i=0,na
       gr(i) = g(i,1)
       gi(i) = g(i,2)
      End Do

c--------------------------------
c establish the ordering vector p 
c--------------------------------

      Do i=1,nexp
        mu = 2**(i-1)
        Do j=1,mu
          p(j) = 2*p(j)
        End Do
        Do j=0,mu-1
          p(j+mu) = p(j)+1
        End Do
      End Do

c------------------------------------------
c compute the sine and cosine of the angles
c------------------------------------------

      fc = pi2/n

      Do i=0,na
        tmp  = fc*i
        c(i) = cos(tmp)
        s(i) = sin(tmp) 
      End Do

c----------------
c perform the FFT 
c----------------

      nsets = 1
      ndel  = n/2

      Do nstage=1,nexp
       ig = 0
       Do iset=1,nsets
        nrt = n/nsets-1
        Do m=0,nrt
         j = Mod(m,ndel) + 2*ndel*(iset-1)
         l = Int(ig/ndel)
         l = p(l)
         gsr(ig) =  gr(j)
     +                  +c(l)*gr(j+ndel)
     +                  -s(l)*gi(j+ndel)
         gsi(ig) = gi(j)
     +                  +c(l)*gi(j+ndel)
     +                  +s(l)*gr(j+ndel)
         ig = ig+1

         End Do
        End Do

        Do i=0,n-1
          gr(i) = gsr(i)
          gi(i) = gsi(i)
        End Do
 
        nsets = 2*nsets
        ndel  = ndel/2
      End Do

c--------------------------------------
c unscramble with the indexing vector p
c--------------------------------------

      Do i=0,n-1
       a(i) = 2.0*gr(p(i))
       b(i) = 2.0*gi(p(i))
      End Do

c-----
c Done
c-----

      Return
      End
